<?php
/*
#####################
#POSTGRESTATS LICENSE
#####################
Copyright 2013 NWLYNX <http://www.nwlynx.com/>.

This file is part of PostgreStats <http://www.postgrestats.com/>.

PostgreStats is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

PostgreStats is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with PostgreStats. If not, see <http://www.gnu.org/licenses/>.
*/


##################################################################################################################
#THIS SCRIPT SHOULD BE RAN BY ROOT'S CRON - THE FREQUENCY IS UP TO YOU
#IT WILL CREATE / UPDATE THE LOG FILE AND IS THEN RESPONSIBLE FOR RESETTING THE PG_STAT_DATABASE VIEW FOR EACH DB
#IMPORTANT - MAKE CERTAIN YOU'VE CUSTOMIZED THE conf.php WITH YOUR VALUES, GEARED FOR YOUR SERVER'S IMPLEMENTATION
##################################################################################################################

########################################################################
#SCRIPT ERROR REPORTING - UNCOMMENT THIS TO VIEW SCRIPT EXECUTION ERRORS
########################################################################
error_reporting(E_ALL);

##################################################################################
#REQUIRE THE MAIN CONF.PHP FILE TO GET OUR DB INFO, CREDENTIALS AND STAT DIRECTORY
##################################################################################
require("conf.php");

############
#ECHO UPDATE
############
echo "Launching PostgreStats update script...\n";

################
#GET TODAYS DATE
################
$log_date_month = date("m");
$log_date_day = date("d");
$log_date_year = date("Y");

##########################################
#INITIALIZE VARIABLES THIS SCRIPT REQUIRES
##########################################
$total_inserts = "";
$total_updates = "";
$total_deletes = "";
$total_seq_scans = "";
$total_idx_scans = "";
$total_seq_rows = "";
$total_idx_rows = "";
$log_buff = "";
$log_email_snapshot_indi = "";

##########################################################################
#VERIFY THE POSTGRESTATS_DIR VAR AND MAKE SURE IT EXSIST BEFORE PRECEEDING
##########################################################################
if(empty($postgrestats_dir) || !file_exists("$postgrestats_dir/postgrestats"))
{
	echo "The postgrestats folder does not exist inside your postgrestats_dir directory. Verify the setting in conf.php and that the directory exists.\n";
	exit;
}

###########################################################################################
#MAKE SURE OUR DATA DIRECTORY EXISTS - IF NOT (1ST TIME RUN), CREATE IT AND SET PERMISSIONS
###########################################################################################
if(!file_exists("$postgrestats_dir/postgrestats/data"))
{
	##############################
	#CREATE IT AND SET PERMISSIONS
	##############################
	mkdir("$postgrestats_dir/postgrestats/data") or die("Error creating data directory.");
	chmod("$postgrestats_dir/postgrestats/data",0777) or die("Error updating data directory permissions.");
}
else
{
	######################################################################################
	#IT EXISTS, DOUBLE CHECK PERMISSIONS ON THIS DIRECTORY TO MAKE SURE WE CAN WRITE TO IT
	######################################################################################
	if(substr(sprintf("%o",fileperms("$postgrestats_dir/postgrestats/data")),-4) != "0777")
	{
		chmod("$postgrestats_dir/postgrestats/data",0777) or die("Error updating data directory permissions.");
	}
}

######################################################################
#MAKE SURE OUR DATABASE CREDENTIALS WERE PASSED FROM THE conf.php FILE
######################################################################
if(empty($db_name) || empty($db_user) || empty($db_pass) || empty($db_host))
{
	echo "Database credentials were not passed to this script from the conf.php script. Please verify they are set.\n";
	exit;
}

##################################################################
#MAKE SURE OUR PSQL LOCATION VAR WAS PASSED FROM THE conf.php FILE
##################################################################
if(empty($psql_loc))
{
	echo "The PSQL location variable was not passed from the conf.php script. Please verify it exists and is accurate.\n";
	exit;
}
elseif(!file_exists($psql_loc))
{
	############################################
	#MAKING DOUBLE SURE THE PSQL PATH IS CORRECT
	############################################
	echo "PSQL cannot be found. Please verify the psql_loc variable in conf.php is accruate.\n";
	exit;
}

####################################################################################################
#MAKE SURE OUR POSTGRES SUPER USER VAR WAS PASSED SO WE CAN SU TO ROOT/POSTGRES AND RESET STATISTICS
####################################################################################################
if(empty($db_su))
{
	echo "The PostgreSQL super-user variable was not passed. Please verify it in conf.php.\n";
	exit;
}

################################################################
#VALIDATE OUR USE_VACUUM_ANALYZE VARIABLE FROM CONF.PHP FOR SAFE
################################################################
if($use_vacuum_analyze != 1)
{
	$use_vacuum_analyze = 2;
	$email_vacuum_analyze = "No";
}
else
{
	$email_vacuum_analyze = "Yes";
}

##############################################################
#VALIDATE OUR FTP_TO_WEBSERVER VARIABLE FROM CONF.PHP FOR SAFE
##############################################################
if($ftp_to_webserver == 1 && !empty($ftp_user_name) && !empty($ftp_user_pass) && !empty($ftp_server_ip) && !empty($src_dir) && !empty($dst_dir) && file_exists("$src_dir") && substr($src_dir,-14) === "/postgrestats/" && substr($dst_dir,-14) === "/postgrestats/")
{
	$email_ftp_to_webserver = "Yes";
}
else
{
	$ftp_to_webserver = 2;
	$email_ftp_to_webserver = "No";
}

################################################################################
#CHECK TO SEE IF AUTO BACKUP OR REMOTE EMAIL BACK-UP IS ENABLED IN THE CONF FILE
#IF SET TO 1 - WE'LL BACKUP THE ENTIRE DATA DIRECTORY AND TAR/GZIP IT
################################################################################
if($use_auto_backup == 1 || $attach_data_backup == 1)
{
	####################################################################
	#FIRST CHECK TO SEE IF A BACK-UP DIRECTORY EXISTS - IF SO, REMOVE IT
	####################################################################
	$backup_dir = "$postgrestats_dir/postgrestats/data_bak.tar.gz";
	
	if(file_exists($backup_dir)) unlink($backup_dir);
	
	############################################################################
	#CD TO THE BACK-UP DIRECTORY, TAR/GZIP THE FILE, UPDATE PERMISSIONS AND EXIT
	############################################################################
	exec("cd $postgrestats_dir/postgrestats/; tar -czf data_bak.tar.gz data/; chmod 777 data_bak.tar.gz; exit;");
}

####################################################################################
#KEEP A RECORD OF WHETHER WE ARE ATTACHING THE DB BACKUP TO THE EMAIL IF ONE IS SENT
####################################################################################
if($attach_data_backup == 1) $email_data_backup = "Yes";
else $email_data_backup = "No";

#######################################################################################################
#SEE IF THE LOG FILE EXISTS - IF IT DOES, CHECK TO SEE IF STATS HAVE ALREADY BEEN RECORDED FOR THIS DAY
#IF NOT, WE'LL CREATE A NEW ENTRY AND SAVE THE LOG ($update_log_mode = false)
#IF SO, WE'LL CREATE A NEW LOG FILE, UPDATE THE VALUES FOR TODAY, AND REPLACE THE OLD LOG FILE
#######################################################################################################
$update_log_mode = false;

$log_file_check = "$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month";

if(file_exists($log_file_check))
{
	####################################################
	#PUT OUR SEARCH STRING TOGETHER AND SEE IF IT EXISTS
	####################################################
	$log_search = "$log_date_month,$log_date_day,$log_date_year,";
	$log_file_check = file_get_contents($log_file_check);
	$log_file_check = strpos($log_file_check,$log_search);
	
	##################################################################
	#USE === BECAUSE STRING POSITION WILL BE AT POSITION ZERO IF FOUND
	##################################################################
	if(!($log_file_check === false)) $update_log_mode = true;
}

#####################
#DB CONNECTION SCRIPT
#####################
function db_connect($db_name,$db_user,$db_pass,$db_host)
{	
	$pg_conn = pg_connect("dbname=$db_name user=$db_user password=$db_pass host=$db_host") or die(pg_errormessage());
	
	############################
	#MAKE SURE WE ARE SUCCESSFUL
	############################
	if(!$pg_conn)
	{
		echo "Cannot connect to the database. Please verify the DB host and log-in credentials are correct.\n";
		exit;
	}
}

###########################################
#UPDATE MODE OR NOT, WE NEED TO CONNECT NOW
###########################################
db_connect($db_name,$db_user,$db_pass,$db_host);

##################################################################
#RETRIEVE DATABASE STATISTICS FROM PG_STAT_DATABASE 
#MAKING SURE TO PULL DATA FROM ONLY DBS ACCEPTING DATA CONNECTIONS
##################################################################
$sqlselect_stats = pg_exec("SELECT a.datid, a.datname, a.xact_commit, a.xact_rollback, a.blks_read, a.blks_hit 
							FROM pg_stat_database AS a, pg_database AS b 
							WHERE a.datname = b.datname 
							AND b.datallowconn = true 
							ORDER BY a.xact_commit DESC") or die(pg_errormessage());

###############################################
#QUERY THE DB FOR OUR pg_stat_database SUM DATA
###############################################
$sqlselect_commit_total = pg_exec("SELECT SUM(a.xact_commit),SUM(a.xact_rollback),SUM(a.blks_read),SUM(a.blks_hit) 
									FROM pg_stat_database AS a, pg_database AS b 
									WHERE a.datname = b.datname 
									AND b.datallowconn = true") or die(pg_errormessage());

###############################
#RETRIEVE OUR COMMIT TOTAL DATA
###############################
$query_data_commit = pg_fetch_row($sqlselect_commit_total);
$total_commits = $query_data_commit[0];
$total_rollbacks = $query_data_commit[1];
$total_block_reads = $query_data_commit[2];
$total_buffer_hits = $query_data_commit[3];

############
#ECHO UPDATE
############
echo "Logging statistics...\n";

############################################################
#CHECK TO SEE WHICH METHOD WE'LL BE USING - UPDATE OR APPEND
############################################################
if($update_log_mode == false)
{
	#########################################################################################################
	#WE ARE IN APPEND MODE - FORMAT THE NEW LOG ENTRY AND WRITE IT TO THE BOTTOM OF THE LOG FILE
	#FIRST CREATE A LOCAL ARRAY JUST OF THE DB NAMES WE'LL USE TO ITERATE THROUGH LATER TO WIPE THE STAT VIEW
	#########################################################################################################
	$db_array = array();

	##########################
	#START UP THE DB ITERATION
	##########################
	while($query_data_stats = pg_fetch_row($sqlselect_stats))
	{	
		###################################################
		#OUR GENERAL STAT INFORMATION FROM db_stat_database
		###################################################
		$db_id = $query_data_stats[0];
		$db_name = $query_data_stats[1];
		$db_commits = $query_data_stats[2];
		$db_rollbacks = $query_data_stats[3];
		$db_block_reads = $query_data_stats[4];
		$db_buffer_hits = $query_data_stats[5];

		##############################################################
		#FORMAT STRINGS FOR WRITING TO LOG/DATA FILE AND LATER VIEWING
		##############################################################
		$log_buff .= "$db_id,";
		$log_buff .= "$db_name,";
		$log_buff .= "$db_commits,";
		$log_buff .= "$db_rollbacks,";
		$log_buff .= "$db_block_reads,";
		$log_buff .= "$db_buffer_hits,";

		#######################################################################################
		#TO GET INS/UPD/DEL DATA FOR EACH DATABASE FROM pg_stat_user_tables AND ADD TO LOG FILE
		#FIRST - CONNECT TO EACH DATABASE
		#######################################################################################
		$db_solo_conn = pg_connect("dbname=$db_name user=$db_user password=$db_pass host=$db_host") or die(pg_errormessage());

		##############
		#RUN THE QUERY
		##############
		$sqlselect_ins_upd_del = pg_exec("SELECT SUM(n_tup_ins),SUM(n_tup_upd),SUM(n_tup_del),SUM(seq_scan),SUM(idx_scan),SUM(seq_tup_read),SUM(idx_tup_fetch) FROM pg_stat_user_tables") or die(pg_errormessage());

		#################################
		#GET OUR DATA AND CREATE OUR VARS
		#################################
		$query_data_ins_upd_del = pg_fetch_row($sqlselect_ins_upd_del);
		$db_inserts = $query_data_ins_upd_del[0];
		$db_updates = $query_data_ins_upd_del[1];
		$db_deletes = $query_data_ins_upd_del[2];
		$db_seq_scans = $query_data_ins_upd_del[3];
		$db_idx_scans = $query_data_ins_upd_del[4];
		$db_seq_rows = $query_data_ins_upd_del[5];
		$db_idx_rows = $query_data_ins_upd_del[6];

		###############################################
		#FORCE LOG VALUE TO ZERO IF DB VALUES ARE EMPTY
		###############################################
		if(empty($db_inserts)) $db_inserts = 0;
		if(empty($db_updates)) $db_updates = 0;
		if(empty($db_deletes)) $db_deletes = 0;
		if(empty($db_seq_scans)) $db_seq_scans = 0;
		if(empty($db_idx_scans)) $db_idx_scans = 0;
		if(empty($db_seq_rows)) $db_seq_rows = 0;
		if(empty($db_idx_rows)) $db_idx_rows = 0; 

		###########################################################
		#ADD INSERT/UPDATE/DELETE STATS TO THE LOG FILE FOR THIS DB
		###########################################################
		$log_buff .= "$db_inserts,";
		$log_buff .= "$db_updates,";
		$log_buff .= "$db_deletes,";
		$log_buff .= "$db_seq_scans,";
		$log_buff .= "$db_idx_scans,";
		$log_buff .= "$db_seq_rows,";
		$log_buff .= "$db_idx_rows";

		###################################################################
		#TALLY UP TOTAL INSERTS/UPDATES/DELETES FOR OUR LOG FILE FIRST LINE
		###################################################################
		$total_inserts += $db_inserts;
		$total_updates += $db_updates;
		$total_deletes += $db_deletes;
		$total_seq_scans += $db_seq_scans;
		$total_idx_scans += $db_idx_scans;
		$total_seq_rows += $db_seq_rows;
		$total_idx_rows += $db_idx_rows;

		####################################
		#CLOSE UP THIS CONNECTION RIGHT AWAY
		####################################
		pg_close($db_solo_conn);

		############################
		#NEWLINE AFTER EACH DBS DATA
		############################
		$log_buff .= "\n";

		############################################
		#ADD THIS ENTRY TO OUR EMAIL SNAPSHOT STRING
		############################################
		$log_email_snapshot_indi .= "$db_name: " . number_format($db_commits) . "\n";

		##################################################
		#ADD DB NAME TO OUR $db_array FOR LATER PROCESSING
		##################################################
		$db_array[] = $db_name;
	}
	
	############################################################################################
	#WE ARE IN APPEND MODE - FORMAT THE NEW LOG ENTRY AND WRITE IT TO THE BOTTOM OF THE LOG FILE
	#START UP THE LOG FILE, FIRST LINE SHOULD BE GENERAL TOTALS / STATS
	############################################################################################
	$log_first_line = "$log_date_month,$log_date_day,$log_date_year,$total_commits,$total_rollbacks,$total_block_reads,$total_buffer_hits,$total_inserts,$total_updates,$total_deletes,$total_seq_scans,$total_idx_scans,$total_seq_rows,$total_idx_rows\n";
	
	##############################################################
	#PUT OUR FIRST LINE IN THE LOG ENTRY AND ADD OUR LOG BUF LINES
	##############################################################
	$log = $log_first_line . $log_buff;
	
	######################################################
	#ADD GENERAL STATISTICS TO THE TOP OF OUR EMAIL STRING
	######################################################
	$log_email_snapshot = "";
	$log_email_snapshot .= "Updated: $log_date_month-$log_date_day-$log_date_year\n";
	$log_email_snapshot .= "DBs Vacuumed/Analyzed: $email_vacuum_analyze\n";
	$log_email_snapshot .= "FTP to Web-server: $email_ftp_to_webserver\n";
	$log_email_snapshot .= "Data Backup Attached: $email_data_backup\n\n";
	$log_email_snapshot .= "GENERAL STATISTICS:\n";
	$log_email_snapshot .= "--------------------------------------------------------------------\n";
	$log_email_snapshot .= "Total Commits: " . number_format($total_commits) . "\n";
	$log_email_snapshot .= "Total Rollbacks: " . number_format($total_rollbacks) . "\n";
	$log_email_snapshot .= "Total Block Reads: " . number_format($total_block_reads) . "\n";
	$log_email_snapshot .= "Total Buffer Hits: " . number_format($total_buffer_hits) . "\n";
	$log_email_snapshot .= "Total User Table Inserts: " . number_format($total_inserts) . "\n";
	$log_email_snapshot .= "Total User Table Updates: " . number_format($total_updates) . "\n";
	$log_email_snapshot .= "Total User Table Deletes: " . number_format($total_deletes) . "\n";
	$log_email_snapshot .= "Total Sequence Scans: " . number_format($total_seq_scans) . " (Rows used: " . number_format($total_seq_rows) . ")\n";
	$log_email_snapshot .= "Total Index Scans: " . number_format($total_idx_scans) . " (Rows used: " . number_format($total_idx_rows) . ")\n\n";
	$log_email_snapshot .= "INDIVIDUAL DATABASE COMMIT SUMMARY:\n";
	$log_email_snapshot .= "--------------------------------------------------------------------\n";
	$log_email_snapshot .= "$log_email_snapshot_indi";
	
	#################################################
	#ADD OUR DAILY DELIMITER TO END OF THIS LOG ENTRY
	#################################################
	$log = $log . "\n";
	
	#########################################################################
	#FIND OUT IF YEAR DIRECTORY EXISTS, IF NOT, CREATE IT AND SET PERMISSIONS
	#########################################################################
	if(!file_exists("$postgrestats_dir/postgrestats/data/$log_date_year"))
	{
		mkdir("$postgrestats_dir/postgrestats/data/$log_date_year") or die("Error creating year log directory.");
		chmod("$postgrestats_dir/postgrestats/data/$log_date_year",0777) or die("Error updating log year directory permissions.");
	}
	
	##############################
	#GET THE NAME OF THE DATA FILE
	##############################
	$data_file = $log_date_month;
	
	if(!empty($data_file) && is_numeric($data_file) && strlen($data_file) == 2)
	{
		#############################
		#FIRST SEE IF THE FILE EXISTS
		#############################
		if(!file_exists("$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month"))
		{
			######################################################
			#CREATE FRESH FILE FOR THIS MONTH - OPEN IN WRITE MODE
			######################################################
			$fh = fopen("$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month", 'w') or die("Error writing to log file.");
		}
		else
		{
			##############################################
			#FILE HANDLING FUNCTIONS - OPEN IN APPEND MODE
			##############################################
			$fh = fopen("$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month", 'a') or die("Error appending to log file.");
		}
		
		################
		#WRITE AND CLOSE
		################
		fwrite($fh, $log);
		fclose($fh);
		
		##########################################
		#MAKE SURE PERMISSIONS ARE SET ON THE FILE
		##########################################
		chmod("$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month",0777) or die("Error updating log file permissions");
	}
}
else
{
	#########################################################################
	#WE ARE IN UPDATE LOG MODE
	#THERE IS ALREADY AN ENTRY FOR THIS DAY AND WE'LL NEED TO ITERATE THROUGH
	#IT'S DATA VALUES, PROCESS THEM, AND RE-OUTPUT THE RESULTS
	#########################################################################
	
	###########################################################################################
	#CREATE AN ARRAY TO STORE OUR DATA - WILL HOUSE ANOTHER ARRAY OF OUR UPDATED DB STAT VALUES
	###########################################################################################
	$db_array = array();
	
	########################################################
	#FIRST STORE THE UPDATED DB STATS IN OUR TEMPORARY ARRAY
	########################################################
	while($query_data_stats = pg_fetch_row($sqlselect_stats))
	{
		######################
		#CREATE THE SOLO ARRAY
		######################
		$db_solo_array = array();

		##############################
		#OBTAIN OUR PRIMARY STATISTICS
		##############################
		$db_id = $query_data_stats[0];
		$db_name = $query_data_stats[1];
		$db_commits = $query_data_stats[2];
		$db_rollbacks = $query_data_stats[3];
		$db_block_reads = $query_data_stats[4];
		$db_buffer_hits = $query_data_stats[5];

		#############################################
		#ASSIGN GENERAL UPDATED STATS TO OUR DB ARRAY
		#############################################
		$db_solo_array[] = $db_id;
		$db_solo_array[] = $db_name;
		$db_solo_array[] = $db_commits;
		$db_solo_array[] = $db_rollbacks;
		$db_solo_array[] = $db_block_reads;
		$db_solo_array[] = $db_buffer_hits;

		##########################################################################
		#GET UPDATED STATS FROM pg_stat_user_tables SO WE CAN UPDATE THESE AS WELL
		#FIRST CONNECT TO EACH DATABASE
		##########################################################################
		$db_solo_conn = pg_connect("dbname=$db_name user=$db_user password=$db_pass host=$db_host") or die(pg_errormessage());

		##############
		#RUN THE QUERY
		##############
		$sqlselect_ins_upd_del = pg_exec("SELECT SUM(n_tup_ins),SUM(n_tup_upd),SUM(n_tup_del),SUM(seq_scan),SUM(idx_scan),SUM(seq_tup_read),SUM(idx_tup_fetch) FROM pg_stat_user_tables") or die(pg_errormessage());

		#################################
		#GET OUR DATA AND CREATE OUR VARS
		#################################
		$query_data_ins_upd_del = pg_fetch_row($sqlselect_ins_upd_del);
		$db_inserts = $query_data_ins_upd_del[0];
		$db_updates = $query_data_ins_upd_del[1];
		$db_deletes = $query_data_ins_upd_del[2];
		$db_seq_scans = $query_data_ins_upd_del[3];
		$db_idx_scans = $query_data_ins_upd_del[4];
		$db_seq_rows = $query_data_ins_upd_del[5];
		$db_idx_rows = $query_data_ins_upd_del[6];

		###############################################
		#FORCE LOG VALUE TO ZERO IF DB VALUES ARE EMPTY
		###############################################
		if(empty($db_inserts)) $db_inserts = 0;
		if(empty($db_updates)) $db_updates = 0;
		if(empty($db_deletes)) $db_deletes = 0;
		if(empty($db_seq_scans)) $db_seq_scans = 0;
		if(empty($db_idx_scans)) $db_idx_scans = 0;
		if(empty($db_seq_rows)) $db_seq_rows = 0;
		if(empty($db_idx_rows)) $db_idx_rows = 0; 

		#############################################################
		#TALLY UP OUR INSERT/UPDATE/DELETE STATS FOR LATER PROCESSING
		#############################################################
		$total_inserts += $db_inserts;
		$total_updates += $db_updates;
		$total_deletes += $db_deletes;
		$total_seq_scans += $db_seq_scans;
		$total_idx_scans += $db_idx_scans;
		$total_seq_rows += $db_seq_rows;
		$total_idx_rows += $db_idx_rows;

		###########################
		#CLOSE THE DB CONNECTION UP
		###########################	
		pg_close($db_solo_conn);

		#################################################
		#ASSIGN INS/UPD/DEL STATS AS WELL TO THE DB ARRAY
		#################################################
		$db_solo_array[] = $db_inserts;
		$db_solo_array[] = $db_updates;
		$db_solo_array[] = $db_deletes;
		$db_solo_array[] = $db_seq_scans;
		$db_solo_array[] = $db_idx_scans;
		$db_solo_array[] = $db_seq_rows;
		$db_solo_array[] = $db_idx_rows;

		#################################################################################
		#NOW ADD THIS ARRAY TO OUR $db_array SO WE CAN ITERATE LATER AND TALLY THE VALUES
		#################################################################################
		$db_array[] = $db_solo_array;
	}
	
	##############################################################
	#CREATE A NEW TEMP FILE WE CAN USE TO TRANSFER UPDATED DATA TO
	##############################################################
	$temp_file = "$postgrestats_dir/postgrestats/data/$log_date_year/" . $log_date_month . "_temp";
	$open_temp_log = fopen($temp_file, 'w') or die("Error creating temp transfer file.");
	
	##############################
	#OPEN THIS FILE UP FOR READING
	##############################
	$open_log_file = "$postgrestats_dir/postgrestats/data/$log_date_year/$log_date_month";
	$open_log = fopen($open_log_file, 'r') or die("Error opening log file for reading.");
	$log = "";
	
	#################################################################################################################
	#CREATE ARRAY WHICH WILL STORE TODAY'S UPDATED DB STATISTICS WHERE EACH FIELD WILL BE AN ARRAY OF EACH DBS VALUES
	#################################################################################################################
	$on_current_day = false;
	$db_update_array = array();
	
	############################################################
	#GO THROUGH THE LOG - LINE BY LINE AND RETRIEVE ALL THE DATA
	############################################################	
	while(!feof($open_log))
	{
		#######################
		#GET ONE LINE AT A TIME
		#######################
		$db_line_single = fgets($open_log);
		
		####################################
		#PUT OUR SEARCH DATE STRING TOGETHER
		####################################
		$search_date = substr($db_line_single,0,11);
		
		#################################
		#CHECK THE BEGINNING OF EACH LINE
		#################################
		if($search_date != "$log_date_month,$log_date_day,$log_date_year,")
		{
			################################################################################
			#WRITE OUT EVERY LINE WHICH IS NOT PART OF THIS DAY - NO MODIFICATIONS NECESSARY
			################################################################################
			if($on_current_day == false)
			{
				$log .= $db_line_single;
			}
			else
			{
				###################################################################
				#HANDLED DIFFERENTLY - IF NOT A NEWLINE - WE'LL PUSH ALL DB ENTRIES
				#INTO AN ARRAY SO WE CAN SORT AND FORMAT THEM LATER
				###################################################################
				if($db_line_single != "\n" && $db_line_single != "" && !empty($db_line_single) && $on_current_day == true)
				{
					#########################################
					#WE HAVE A LINE WHICH NEEDS TO BE UPDATED
					#########################################
					$log_line = explode(",",$db_line_single);

					###############################################################
					#SAMPLE LINE FROM LOG FILE
					#19504,db_name_psql,11602,1,1072,1013126,0,0,0,10638,0,680219,0
					###############################################################

					##########################
					#PUSH THE SOLO DB STATS IN
					##########################
					$old_db_id = $log_line[0];
					$old_db_name = $log_line[1];
					$old_db_commits = $log_line[2];
					$old_db_roll_backs = $log_line[3];
					$old_db_block_reads = $log_line[4];
					$old_db_buffer_hits = $log_line[5];
					$old_db_inserts = $log_line[6];
					$old_db_updates = $log_line[7];
					$old_db_deletes = $log_line[8];
					$old_db_seq_scans = $log_line[9];
					$old_db_idx_scans = $log_line[10];
					$old_db_seq_rows = $log_line[11];
					$old_db_idx_rows = $log_line[12];

					#########################################################################################
					#CREATE TEMPORARY ARRAY TO STORE THESE VALUES - WILL BE PUSHED INTO MULTI ARRAY FOR LATER
					#########################################################################################
					$db_solo_update_array = array();

					###################################################################
					#NOW ITERATE THROUGH OUR NEW DB VALUE ARRAYS TO FIND THIS DBS STATS
					###################################################################										
					for($i=0; $i<count($db_array); $i++)
					{
						################
						#CHECK FOR MATCH
						################
						if($old_db_name == $db_array[$i][1])
						{
							$db_found = true;
	
							#####################################
							#TALLY UP OLD STATS WITH NEW DB STATS
							#####################################
							$db_solo_update_array[] = $old_db_id;								#DATABASE ID
							$db_solo_update_array[] = $old_db_name;								#DATABASE NAME
							$db_solo_update_array[] = $old_db_commits + $db_array[$i][2];		#DATABASE COMMITS
							$db_solo_update_array[] = $old_db_roll_backs + $db_array[$i][3];	#DATABASE ROLL BACKS
							$db_solo_update_array[] = $old_db_block_reads + $db_array[$i][4];	#DATABASE BLOCK READS
							$db_solo_update_array[] = $old_db_buffer_hits + $db_array[$i][5];	#DATABASE BUFFER HITS
							$db_solo_update_array[] = $old_db_inserts + $db_array[$i][6];		#DATABASE INSERTS
							$db_solo_update_array[] = $old_db_updates + $db_array[$i][7];		#DATABASE UPDATES
							$db_solo_update_array[] = $old_db_deletes + $db_array[$i][8];		#DATABASE DELETES
							$db_solo_update_array[] = $old_db_seq_scans + $db_array[$i][9];		#DATABASE SEQUENCE SCANS
							$db_solo_update_array[] = $old_db_idx_scans + $db_array[$i][10];	#DATABASE INDEX SCANS
							$db_solo_update_array[] = $old_db_seq_rows + $db_array[$i][11];		#DATABASE SEQUENCE SCANS - ROWS USED
							$db_solo_update_array[] = $old_db_idx_rows + $db_array[$i][12];		#DATABASE INDEX SCANS - ROWS USED

							################################
							#UPDATE THE MAIN DB UPDATE ARRAY
							################################
							$db_update_array[] = $db_solo_update_array;

							#######################################################
							#GET OUT OF THIS LOOP - NO NEED TO MODIFY ANYTHING ELSE
							#######################################################
							break;
						}
					}
				}
			}
		}
		else
		{
			####################################################
			#TALLY UP OLD DAILY LOG SNAPSHOT LINE WITH DB TOTALS
			####################################################
			$log_line = explode(",",$db_line_single);
			$total_commits += $log_line[3];
			$total_rollbacks += $log_line[4];
			$total_block_reads += $log_line[5];
			$total_buffer_hits += $log_line[6];

			##################################################################################################################
			#FOR INSERTS/UPDATES/DELETES WE NEED TO GET THESE VALUES AND LATER ADD THE UPDATED STATS FROM OUR ABOVE ITERATIONS
			##################################################################################################################
			$total_inserts_pre = $log_line[7];
			$total_updates_pre = $log_line[8];
			$total_deletes_pre = $log_line[9];
			$total_seq_scans_pre = $log_line[10];
			$total_idx_scans_pre = $log_line[11];
			$total_seq_rows_pre = $log_line[12];
			$total_idx_rows_pre = $log_line[13];

			########################################################
			#SET OUR CURRENT DAY VAR TO TRUE FOR THE ITERATION CATCH
			########################################################
			$on_current_day = true;
		}
	}
	
	#######################################################################################################
	#NOW THAT OUR $db_update_array HAS BEEN COMPLETED
	#WE NEED COMPARE THE DB ARRAY WITH THE UPDATE ARRAY - FIND NEW DBS WHICH WERE NOT IN THE LOG PREVIOUSLY
	#IF FOUND - WE'LL TAKE THE STAT VALUES STRAIGHT FROM THE DATABASE
	#######################################################################################################
	for($i = 0; $i<count($db_array); $i++)
	{
		###################
		#OUR ITERATION FLAG
		###################
		if(isset($db_flag)) unset($db_flag);
		
		######################
		#GET OUR DB ARRAY NAME
		######################
		$db_array_name = $db_array[$i][1];
		
		###########################
		#START THE SECOND ITERATION
		###########################
		for($m = 0; $m<count($db_update_array); $m++)
		{
			#########################
			#OUR UPDATE ARRAY DB NAME
			#########################
			$db_log_name = $db_update_array[$m][1];
			
			####################
			#CHECK FOR NON-MATCH
			####################
			if($db_log_name == $db_array_name) $db_flag = true;
		}
		
		###############
		#CHECK OUR FLAG
		###############
		if(!isset($db_flag))
		{
			###########################################
			#PULL VALUES RIGHT FROM THE DB - UNMODIFIED
			###########################################
			$db_solo_update_array[] = $db_array[$i][0];		#DATABASE ID
			$db_solo_update_array[] = $db_array[$i][1];		#DATABASE NAME
			$db_solo_update_array[] = $db_array[$i][2];		#DATABASE COMMITS
			$db_solo_update_array[] = $db_array[$i][3];		#DATABASE ROLL BACKS
			$db_solo_update_array[] = $db_array[$i][4];		#DATABASE BLOCK READS
			$db_solo_update_array[] = $db_array[$i][5];		#DATABASE BUFFER HITS
			$db_solo_update_array[] = $db_array[$i][6];		#DATABASE INSERTS
			$db_solo_update_array[] = $db_array[$i][7];		#DATABASE UPDATES
			$db_solo_update_array[] = $db_array[$i][8];		#DATABASE DELETES
			$db_solo_update_array[] = $db_array[$i][9];		#DATABASE SEQUENCE SCANS
			$db_solo_update_array[] = $db_array[$i][10];	#DATABASE INDEX SCANS
			$db_solo_update_array[] = $db_array[$i][11];	#DATABASE SEQUENCE SCANS - ROWS AFFECTED
			$db_solo_update_array[] = $db_array[$i][12];	#DATABASE INDEX SCANS - ROWS AFFECTED
			
			##########################################
			#UPDATE THE MAIN DB UPDATE ARRAY ONCE MORE
			##########################################
			$db_update_array[] = $db_solo_update_array;
		}
	}

	######################################################################################
	#WE NEED TO PUT OUR PRE-INSERT/UPDATE/DELETE VALUES TOGETHER WITH OUR ITERATION TOTALS
	#AND UPDATE THE $log_first_line NOW WITH UPDATED INSERT/UPDATE/DELETE STATS
	######################################################################################
	$total_inserts += $total_inserts_pre;
	$total_updates += $total_updates_pre;
	$total_deletes += $total_deletes_pre;
	$total_seq_scans += $total_seq_scans_pre;
	$total_idx_scans += $total_idx_scans_pre;
	$total_seq_rows += $total_seq_rows_pre;
	$total_idx_rows += $total_idx_rows_pre;

	###############################################################################################
	#START UP THE LOG FILE ENTRY FOR THIS DAY, FIRST LINE SHOULD BE GENERAL, UPDATED TOTALS / STATS
	###############################################################################################
	$log_first_line = "$log_date_month,$log_date_day,$log_date_year,$total_commits,$total_rollbacks,$total_block_reads,$total_buffer_hits,$total_inserts,$total_updates,$total_deletes,$total_seq_scans,$total_idx_scans,$total_seq_rows,$total_idx_rows";
	
	#############################################
	#FIRST LINE COMPLETED - NOW ADD IT TO THE LOG
	#############################################
	$log .= $log_first_line;
	
	######################################################
	#ADD GENERAL STATISTICS TO THE TOP OF OUR EMAIL STRING
	######################################################
	$log_email_snapshot = "";
	$log_email_snapshot .= "Updated: $log_date_month-$log_date_day-$log_date_year\n";
	$log_email_snapshot .= "DBs Vacuumed/Analyzed: $email_vacuum_analyze\n";
	$log_email_snapshot .= "FTP to Web-server: $email_ftp_to_webserver\n";
	$log_email_snapshot .= "Data Backup Attached: $email_data_backup\n\n";
	$log_email_snapshot .= "GENERAL STATISTICS:\n";
	$log_email_snapshot .= "--------------------------------------------------------------------\n";
	$log_email_snapshot .= "Total Commits: " . number_format($total_commits) . "\n";
	$log_email_snapshot .= "Total Rollbacks: " . number_format($total_rollbacks) . "\n";
	$log_email_snapshot .= "Total Block Reads: " . number_format($total_block_reads) . "\n";
	$log_email_snapshot .= "Total Buffer Hits: " . number_format($total_buffer_hits) . "\n";
	$log_email_snapshot .= "Total User Table Inserts: " . number_format($total_inserts) . "\n";
	$log_email_snapshot .= "Total User Table Updates: " . number_format($total_updates) . "\n";
	$log_email_snapshot .= "Total User Table Deletes: " . number_format($total_deletes) . "\n";
	$log_email_snapshot .= "Total Sequence Scans: " . number_format($total_seq_scans) . " (Rows used: " . number_format($total_seq_rows) . ")\n";
	$log_email_snapshot .= "Total Index Scans: " . number_format($total_idx_scans) . " (Rows used: " . number_format($total_idx_rows) . ")\n\n";
	$log_email_snapshot .= "INDIVIDUAL DATABASE COMMIT SUMMARY:\n";
	$log_email_snapshot .= "--------------------------------------------------------------------\n";
	
	#####################################################################
	#DATA ADDITIONS AND ADJUSTMNETS ARE COMPLETED
	#WE NOW NEED TO RE-SORT OUR UPDATED ARRAYS TO KEEP THE LOG CONSISTENT
	#####################################################################
	function compare($x,$y)
	{
		if($x[2] == $y[2]) return 0;
		elseif($x[2] < $y[2]) return 1;
		else return -1;
	}

	###########################################
	#SORT IT OUT BY COMMITS - HIGHEST TO LOWEST
	###########################################
	usort($db_update_array,"compare");

	#################
	#ADD IN A NEWLINE
	#################
	$log .= "\n";

	########################################################################
	#LOOP THROUGH OUR UPDATED/SORTED ARRAY AND WRITE THE LAST OF THE NEW LOG
	########################################################################
	for($i=0; $i<count($db_update_array); $i++)
	{
		$line = "";
		
		###################################################################################
		#ITERATE THROUGH OUR SUB-MULTI ARRAY TO GET INDI VALUES AND PUT THE STRING TOGETHER
		###################################################################################
		for($m=0; $m<13; $m++)
		{	
			$line .= $db_update_array[$i][$m];

			###########################################
			#ADD COMMA FOR FORMAT EXCEPT FOR LAST INDEX
			###########################################
			if($m < 12) $line .= ",";
		}

		#######################
		#NEWLINE TO KEEP FORMAT
		#######################
		$line .= "\n";

		############
		#WRITE IT UP
		############
		$log .= $line;

		#######################################
		#ADD ENTRY TO OUR EMAIL SNAPSHOT STRING
		#######################################
		$log_email_snapshot .= $db_update_array[$i][1] . ": " . number_format($db_update_array[$i][2]) . "\n";
	}

	################################
	#ONE MORE NEWLINE TO KEEP FORMAT
	################################
	$log .= "\n";
	
	###############################
	#WRITE OUR LOG TO THE TEMP FILE
	###############################
	fwrite($open_temp_log,$log);
	
	########################
	#CLOSE BOTH FILES UP NOW
	########################
	fclose($open_log);
	fclose($open_temp_log);
	
	####################
	#UNLINK THE ORIGINAL
	####################
	unlink($open_log_file) or die("Error removing old log file.");
	
	#####################################
	#CHANGE NAME OF TEMP FILE TO ORIGINAL
	#####################################
	rename($temp_file,$open_log_file) or die("Error renaming updated log file.");
	
	###################################
	#UPDATE PERMISSIONS ON THE NEW FILE
	###################################
	chmod($open_log_file,0777) or die("Error updating data directory permissions.");
}

############
#ECHO UPDATE
############
echo "Resetting stat views...\n";

#######################################################################################################
#TIME TO RESET STATISTICS
#WE'LL DO THIS DIFFERENTLY DEPENDING ON WHETHER WE ARE IN UPDATE (EXISTING) OR APPEND (NEW) LOG MODE
#IF APPEND - $db_array WILL SIMPLY HOUSE A LIST OF THE DB NAMES WE'LL NEED
#IF UPDATE - $db_array HOUSES THE $db_solo_array ARRAY WHICH WE'LL NEED TO RETRIEVE DB NAMES FROM
#ONCE WE HAVE THEM - WE'LL ITERATE AND EXECUTE THE SYSTEM COMMAND AS POSTGRES, EMPTYING OUT THE PG VIEW
#######################################################################################################

###########################################################
#FIRST - ESCAPE OUR $db_su AND $psql_loc VARIABLES FOR SAFE
###########################################################
$db_su = escapeshellcmd($db_su);
$psql_loc = escapeshellcmd($psql_loc);

##################################################################
#FIRST CHECK TO SEE WHICH METHOD WE'LL BE USING - UPDATE OR APPEND
##################################################################
if($update_log_mode == false)
{
	##############################################
	#WE'RE IN APPEND MODE - START UP OUR ITERATION
	##############################################
	for($i = 0; $i<count($db_array); $i++)
	{
		######################
		#GET OUR DB ARRAY NAME
		######################
		$db_array_name = $db_array[$i];
		
		################################################################
		#MAKE DOUBLE SURE BEFORE WE RUN THE COMMAND THIS VAR HAS A VALUE
		################################################################
		if(!empty($db_array_name))
		{
			#########################################
			#ESCAPE OUR DATNAME AS WELL JUST FOR SAFE
			#########################################
			$db_array_name = escapeshellcmd($db_array_name);
			
			########################################
			#RUN THE EXEC - SU TO POSTGRES AND RESET
			########################################
			exec("su - $db_su -c \"$psql_loc -d $db_array_name -c 'select pg_stat_reset()'\"");
			
			##################################################################
			#CHECK OUR USE_VACUUM_ANALYZE VAR FROM CONF.PHP
			#IF SET TO 1, WE WILL EXEC THE VACUUM ANALYZE STATEMENT ON EACH DB
			##################################################################
			if($use_vacuum_analyze == 1)
			{
				############
				#ECHO UPDATE
				############
				echo "Running vacuum / analyze ($db_array_name)...\n";
			
				######
				#DO IT
				######
				exec("su - $db_su -c \"$psql_loc -d $db_array_name -c 'VACUUM ANALYZE'\"");
			}	
		}
	}
}
else
{
	###############################################
	#WE'RE IN UPDATE MODE - START UP THE ITERATION
	###############################################
	for($i = 0; $i<count($db_array); $i++)
	{
		######################
		#GET OUR DB ARRAY NAME
		######################
		$db_array_name = $db_array[$i][1];
		
		################################################################
		#MAKE DOUBLE SURE BEFORE WE RUN THE COMMAND THIS VAR HAS A VALUE
		################################################################
		if(!empty($db_array_name))
		{
			#########################################
			#ESCAPE OUR DATNAME AS WELL JUST FOR SAFE
			#########################################
			$db_array_name = escapeshellcmd($db_array_name);
			
			########################################
			#RUN THE EXEC - SU TO POSTGRES AND RESET
			########################################
			exec("su - $db_su -c \"$psql_loc -d $db_array_name -c 'select pg_stat_reset()'\"");	
			
			##################################################################
			#CHECK OUR USE_VACUUM_ANALYZE VAR FROM CONF.PHP
			#IF SET TO 1, WE WILL EXEC THE VACUUM ANALYZE STATEMENT ON EACH DB
			##################################################################
			if($use_vacuum_analyze == 1)
			{
				############
				#ECHO UPDATE
				############
				echo "Running vacuum / analyze ($db_array_name)...\n";

				######
				#DO IT
				######
				exec("su - $db_su -c \"$psql_loc -d $db_array_name -c 'VACUUM ANALYZE'\"");
			}
		}
	}
}

#####################################################################################
#PREP EMAIL TO ADMIN LETTING THEM KNOW STATS ARE UPDATED
#FIRST LET'S DEFINE A VALID EMAIL FUNCTION AND AT LEAST MAKE SURE AN MX RECORD EXISTS
#####################################################################################
function is_email_valid($email)
{
	#FIRST VALIDATE SYNTAX
	if(!preg_match('/^[a-zA-Z0-9._-]+@[a-zA-Z0-9._-]+\.([a-zA-Z]{2,4})$/i', $email)) return false;

	#NOW CHECK FOR "MX "RECORD
	list($user,$domain) = explode('@',$email);
	if(!checkdnsrr($domain,"MX")) return false;
	else return true;
}

#######################################
#SEND IT OUT ONLY IF IT'S SAFE TO DO SO
#######################################
if($send_completion_email == 1 && !empty($stats_email) && is_email_valid($stats_email))
{
	############
	#ECHO UPDATE
	############
	echo "Sending completion email notification...\n";

	##########################################################################
	#REGARDLESS OF THE EMAIL TYPE - WE'LL FIRST PUT SOME GENERAL VARS TOGETHER
	##########################################################################
	$to = $stats_email;
	$from = $stats_email;
	$subject = "PostgreSQL Statistics Logs Updated!";
	$body = "PostgreStats have been updated.\n";
	$body .= "Below is a snapshot summary of statistics on the day for this update:\n\n";
	$body .= "$log_email_snapshot\n";
	
	#####################################################################################################
	#FIRST DETERMINE WHETHER WE ARE USING LOCAL AUTO BACK-UP AND WHETHER WE ARE ATTACHING THE TAR/GZ FILE
	#####################################################################################################
	if($attach_data_backup != 1)
	{		
		################################################################
		#FIRE IT OFF WITH PHP MAIL() - NO NEED FOR FURTHER PREP FUNCTION
		################################################################
		mail($to,$subject,$body,"From: $from");
	}
	else
	{
		######################################################################
		#SENDING OUT SUMMARY EMAIL WITH THE MOST RECENT DATA TAR/GZ ATTACHMENT
		#PUT THE FILE AND PATH NAME TOGETHER
		######################################################################
		$file_name = "data_bak.tar.gz";
		$backup_file = "$postgrestats_dir/postgrestats/$file_name";
		
		##############################
		#MAKE SURE OUR BACK-UP IS HERE
		##############################
		if(file_exists($backup_file))
		{	
			####################################################################
			#PUT THE MESSAGE BODY TOGETHER FOR BOTH PLAIN TEXT AND HTML VERSIONS
			####################################################################
			$text_msg = $body;
			$html_msg = nl2br($body);
			
			###########################################################
			#CREATE OUR MULTI-PART BOUNDARIES TO SEPARATE THE EMAIL OUT
			###########################################################
			$boundary1 = "b1" . md5(time());
			$boundary2 = "b2" . md5(time());
			
			######################################
			#PUT OUR FILE ATTACHMENT DATA TOGETHER
			######################################
			$filedata = file_get_contents($backup_file);
			$filedata = chunk_split(base64_encode($filedata));

#########################
#PUT OUR HEADERS TOGETHER
#########################
$headers=<<<AKAM
From: $from
MIME-Version: 1.0
Content-Type: multipart/mixed;
    boundary="$boundary1"
AKAM;

#################################
#PUT OUR ATTACHMENT DATA TOGETHER
#################################
$attachment = <<<ATTA
--$boundary1
Content-Type: application/octet-stream; 
    name="$file_name"
Content-Transfer-Encoding: base64
Content-Disposition: attachment;
    filename="$file_name"

$filedata
ATTA;

########################################################################
#PUT TOGETHER BOTH PLAIN TEXT AND HTML VERSIONS WITH BOUNDARY SEPARATION
########################################################################
$body =<<<AKAM
This is a multi-part message in MIME format.

--$boundary1
Content-Type: multipart/alternative;
    boundary="$boundary2"

--$boundary2
Content-Type: text/plain;
    charset="windows-1256"
Content-Transfer-Encoding: quoted-printable

$text_msg

--$boundary2
Content-Type: text/html;
    charset="windows-1256"
Content-Transfer-Encoding: quoted-printable

$html_msg

--$boundary2--

$attachment
--$boundary1--
AKAM;

			#########################
			#GOOD TO GO - FIRE IT OFF
			#########################
			mail($to,$subject,$body,$headers);
		}
	}
}

#############################################
#FUNCTION TO DELETE FTP DESTINATION DIRECTORY
#############################################
function ftp_rec_delete($conn_id,$dst_dir)
{
	############
	#LET'S DO IT
	############
	if(!(@ftp_rmdir($conn_id, $dst_dir) || @ftp_delete($conn_id, $dst_dir)))
	{
		###########
		#GET A LIST
		###########
		$dir_list = ftp_nlist($conn_id, $dst_dir);
		
		########
		#BE SURE
		########
		if(!empty($dir_list))
		{
			#######
			#REMOVE
			#######
			foreach($dir_list as $value) ftp_rec_delete($conn_id,$value);
		}
	}

	########
	#FINALLY
	########
	@ftp_rmdir($conn_id, $dst_dir);
}

##########################################
#FUNCTION TO CREATE NEW FTP WEBSERVER COPY
##########################################
function ftp_rec_send($conn_id, $src_dir, $dst_dir)
{
	$d = dir($src_dir);
	ftp_mkdir($conn_id, $dst_dir);

	##############################
	#ITERATE THROUGH THE DIRECTORY
	##############################
	while($file = $d -> read())
	{
		#############################################
		#NO LOOPS AND OMIT CRON_LOG_UPDATE.PHP SCRIPT
		#############################################
		if($file != "." && $file != ".." && $file != "cron_log_update.php" && $file != "data_bak.tar.gz")
		{
			################
			#FOR DIRECTORIES
			################
			if(is_dir($src_dir . "/" . $file))
			{
				ftp_rec_send($conn_id, $src_dir . "/" . $file, $dst_dir . "/" . $file);
			}
			else
			{
				#################
				#IT'S A FILE FILE
				#################
				ftp_put($conn_id, $dst_dir . "/" . $file, $src_dir . "/" . $file, FTP_BINARY);
			}
		}
	}

	#########
	#CLEAN UP
	#########
	$d -> close();
}

##############################
#CHECK FOR WEB-SERVER FTP COPY
##############################
if($ftp_to_webserver == 1 && !empty($ftp_user_name) && !empty($ftp_user_pass) && !empty($ftp_server_ip) && !empty($src_dir) && !empty($dst_dir) && file_exists("$src_dir") && substr($src_dir,-14) === "/postgrestats/" && substr($dst_dir,-14) === "/postgrestats/")
{
	##################
	#CREATE CONNECTION
	##################
	$conn_id = ftp_connect($ftp_server_ip) or die("FTP connection failed. Please check FTP service status.");
	
	#############
	#ATTEMP LOGIN
	#############
	if(ftp_login($conn_id, $ftp_user_name, $ftp_user_pass))
	{
		############
		#ECHO UPDATE
		############
		echo "Launching FTP to web-server...\n";

		#######################################################################
		#DELETE OLD FTP DESTINATION DIR IF EXISTS - FIRST GET CURRENT DIRECTORY
		#######################################################################
		$cwd = ftp_pwd($conn_id);

		######################
		#CHDIR TO MAKE CERTAIN
		######################
		if(@ftp_chdir($conn_id,$dst_dir))
		{
			#################
			#CD BACK TO START
			#################
			ftp_chdir($conn_id, $cwd);
			
			######
			#DO IT
			######
			ftp_rec_delete($conn_id,$dst_dir);
		}
	
		#####################
		#FTP TO WEBSERVER NOW
		#####################
		ftp_rec_send($conn_id, $src_dir, $dst_dir);
	}
	else
	{
		##############
		#LOG-IN FAILED
		##############
		echo "FTP log-in failed. Please check credentials.\n";
		exit;
	}
	
	############
	#CLEAN IT UP
	############
	ftp_close($conn_id);
}

######################################
#STAT LOG CREATED / UPDATED - ALL DONE
######################################
echo "--------------------------------------------\n";
echo "PostgreSQL Stats Log File Created/Updated\n";
echo "--------------------------------------------\n";
?>