<?php
/*
#####################
#POSTGRESTATS LICENSE
#####################
Copyright 2013 NWLYNX <http://www.nwlynx.com/>.

This file is part of PostgreStats <http://www.postgrestats.com/>.

PostgreStats is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

PostgreStats is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with PostgreStats. If not, see <http://www.gnu.org/licenses/>.
*/


########################################################################
#REQUIRE THE MAIN CONF.PHP FILE TO GET OUR POSTGRESTATS SERVER DIRECTORY
########################################################################
require("conf.php");

######################################################################################
#HANDLE THE INTERFACE COLORS - CHECK OUR CONF INTERFACE VAR AND DEFAULT TO 1 (REVERSE)
######################################################################################
if($interface == 2) $graphics_src = "graphics/int_pos";
else $graphics_src = "graphics/int_rev";

#####################################################################
#MAKE SURE OUR STAT FUNCTION VARIABLE WAS PASSED FROM OUR AJAX SCRIPT
#####################################################################
$stat_function = $_POST["stat_function"];

if(empty($stat_function))
{
	$stat_msg = "Error no Stat Function passed.";
	echo $stat_msg;
	exit;
}

##########################################################################
#VERIFY THE POSTGRESTATS_DIR VAR AND MAKE SURE IT EXSIST BEFORE PRECEEDING
##########################################################################
if(empty($postgrestats_dir) || !file_exists("$postgrestats_dir/postgrestats"))
{
	$stat_msg = "The postgrestats folder does not exist inside your postgrestats_dir directory. Verify the setting in conf.php and that the directory exists.";
	echo $stat_msg;
	exit;
}

############################################################################################################
#FIND OUT WHICH FUNCTION WE ARE LOOKING FOR - GATHER OUR DEPENDENT VARS AND PASS TO THE APPROPRIATE FUNCTION
############################################################################################################

##########################
#RETRIEVE_MONTHLY_SNAPSHOT
##########################
if($stat_function == "retrieve_monthly_snapshot")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$log_month = $_POST["month"];
	$log_year = $_POST["year"];
	$snap_chartting = $_POST["snap_chartting"];

	##################
	#CALL THE FUNCTION
	##################
	retrieve_monthly_snapshot($postgrestats_dir,$log_month,$log_year,$graphics_src,$snap_chartting);
}

########################
#RETRIEVE_DAILY_DB_STATS
########################
if($stat_function == "retrieve_daily_db_stats")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$log_month = $_POST["month"];
	$log_day = $_POST["day"];
	$log_year = $_POST["year"];
	$daily_chartting = $_POST["daily_chartting"];

	##################
	#CALL THE FUNCTION
	##################
	retrieve_daily_db_stats($postgrestats_dir,$log_month,$log_day,$log_year,$graphics_src,$daily_chartting);
}

######################
#RETRIEVE_MVM_DB_STATS
######################
if($stat_function == "retrieve_mvm_db_stats")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$month_a = $_POST["month_a"];
	$year_a = $_POST["year_a"];
	$month_b = $_POST["month_b"];
	$year_b = $_POST["year_b"];
	$vs_chartting = $_POST["vs_chartting"];
	
	##################
	#CALL THE FUNCTION
	##################
	retrieve_mvm_db_stats($postgrestats_dir,$month_a,$year_a,$month_b,$year_b,$graphics_src,$vs_chartting);
}

#######################
#RETRIEVE_INDI_DB_STATS
#######################
if($stat_function == "retrieve_indi_db_stats")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$db_name = $_POST["db"];
	$log_month = $_POST["log_month"];
	$log_day = $_POST["log_day"];
	$log_year = $_POST["log_year"];
	
	##################
	#CALL THE FUNCTION
	##################
	retrieve_indi_db_stats($postgrestats_dir,$db_name,$log_month,$log_day,$log_year,$graphics_src);
}

######################
#RETRIEVE_WEEKLY_TREND
######################
if($stat_function == "retrieve_weekly_trend")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$log_month = $_POST["month"];
	$log_year = $_POST["year"];
	$weekly_chartting = $_POST["weekly_chartting"];

	##################
	#CALL THE FUNCTION
	##################
	retrieve_weekly_trend($postgrestats_dir,$log_month,$log_year,$graphics_src,$weekly_chartting);
}

#########################
#RETRIEVE_YEARLY_SNAPSHOT
#########################
if($stat_function == "retrieve_yearly_snapshot")
{
	###########################
	#RETRIEVE OUR INCOMING VARS
	###########################
	$log_year = $_POST["year"];
	$year_chartting = $_POST["year_chartting"];

	##################
	#CALL THE FUNCTION
	##################
	retrieve_yearly_snapshot($postgrestats_dir,$log_year,$graphics_src,$year_chartting);
}

##########################################################################################################
##########################################################################################################
#FUNCTION: NUMBER_ABBR
##########################################################################################################
##########################################################################################################
function number_abbr($num,$round_max)
{
	################################
	#FORMAT ONLY THOUSANDS AND ABOVE
	################################
	if(strlen($num) < 4) return $num;

	################################
	#VALIDATE ROUND_MAX VAR FOR SAFE
	################################
	if(empty($round_max) || !is_numeric($round_max) || $round_max < 0) $round_max = 2;

	############################################
	#DECIPHER STRING LENGTH & FORMAT ACCORDINGLY
	############################################
	if(strlen($num) > 3 && strlen($num) < 7)
	{
		$num = $num / 1000;
		$num = number_format($num, $round_max, '.', ',') . "k";
	}
	elseif(strlen($num) > 6 && strlen($num) < 10)
	{
		$num = $num / 1000000;
		$num = number_format($num, $round_max, '.', ',') . "m";
	}
	elseif(strlen($num) > 9 && strlen($num) < 13)
	{
		$num = $num / 1000000000;
		$num = number_format($num, $round_max, '.', ',') . "b";
	}
	elseif(strlen($num) > 12 && strlen($num) < 16)
	{
		$num = $num / 1000000000000;
		$num = number_format($num, $round_max, '.', ',') . "t";
	}
	elseif(strlen($num) > 15)
	{
		$num = $num / 1000000000000000;
		$num = number_format($num, $round_max, '.', ',') . "q";
	}

	##########
	#RETURN IT
	##########
	return $num;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: DATE_GET_FRIENDLY_MONTH
##########################################################################################################
##########################################################################################################
function date_get_friendly_month($month)
{
	#########################################
	#GET INCOMING AND CREATE FRIENDLY VERSION
	#########################################
	switch($month)
	{
		case '01': $month_friendly = "Jan"; break;
		case '02': $month_friendly = "Feb"; break;
		case '03': $month_friendly = "Mar"; break;
		case '04': $month_friendly = "Apr"; break;
		case '05': $month_friendly = "May"; break;
		case '06': $month_friendly = "Jun"; break;
		case '07': $month_friendly = "Jul"; break;
		case '08': $month_friendly = "Aug"; break;
		case '09': $month_friendly = "Sep"; break;
		case '10': $month_friendly = "Oct"; break;
		case '11': $month_friendly = "Nov"; break;
		case '12': $month_friendly = "Dec";
	}

	##########
	#RETURN IT
	##########
	return $month_friendly;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_MONTHLY_SNAPSHOT
##########################################################################################################
##########################################################################################################
function retrieve_monthly_snapshot($postgrestats_dir,$log_month,$log_year,$graphics_src,$snap_chartting)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($log_month) || empty($log_year))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}

	##################################################################
	#DOUBLE CHECK THE CHARTTING VAR AND MAKE SURE IT HAS A LEGIT VALUE
	##################################################################
	if($snap_chartting != "commits" && $snap_chartting != "rollbacks" && $snap_chartting != "block_reads" && $snap_chartting != "buffer_hits" && $snap_chartting != "inserts" && $snap_chartting != "updates" && $snap_chartting != "deletes" && $snap_chartting != "seq_scans" && $snap_chartting != "idx_scans")
	{
		###########################################
		#DEFAULT THE CHART TO SHOWING COMMIT VALUES
		###########################################
		$snap_chartting = "commits";
	}

	#############################################################
	#PUT THE LOG FILE NAME TOGETHER AND MAKE SURE THE FILE EXISTS
	#############################################################
	$log_file_name = $log_month;

	###########################
	#MAKE DOUBLE SURE IT EXISTS
	###########################
	if(!file_exists("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name"))
	{
		$stat_msg = "Zero stats have been recorded for this month.";
		echo $stat_msg;
		exit;
	}

	#########################
	#ATTEMPT TO OPEN THE FILE
	#########################
	if(!$open_log = fopen("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name", "r"))
	{
		$stat_msg = "Unable to open this log file.";
		echo $stat_msg;
		exit;
	}

	###########################
	#INITIATE SOME GENERAL VARS
	###########################
	$bar_height_vert = 150;
	$bar_width = 23;
	$total_days_tracked = 0;

	######################################################
	#VARS WHICH WILL HOUSE BOTH CURRENT AND PREVIOUS STATS
	######################################################
	$total_commits = 0;
	$total_rollbacks = 0;
	$total_block_reads = 0;
	$total_buffer_hits = 0;
	$total_inserts = 0;
	$total_updates = 0;
	$total_deletes = 0;
	$total_seq_scans = 0;
	$total_idx_scans = 0;
	$total_seq_rows = 0;
	$total_idx_rows = 0;

	################################################
	#VARS WHICH WILL HOUSE OUR ITERATION STAT VALUES
	################################################
	$commits = 0;
	$rollbacks = 0;
	$block_reads = 0;
	$buffer_hits = 0;
	$user_table_inserts = 0;
	$user_table_updates = 0;
	$user_table_deletes = 0;
	$seq_scans = 0;
	$idx_scans = 0;
	$seq_rows = 0;
	$idx_rows = 0;

	#############
	#FOR PREVIOUS
	#############
	$total_commits_prev = 0;
	$total_rollbacks_prev = 0;
	$total_block_reads_prev = 0;
	$total_buffer_hits_prev = 0;
	$total_inserts_prev = 0;
	$total_updates_prev = 0;
	$total_deletes_prev = 0;
	$total_seq_scans_prev = 0;
	$total_idx_scans_prev = 0;
	$total_seq_rows_prev = 0;
	$total_idx_rows_prev = 0;

	################################################
	#VARS WHICH WILL HOUSE STAT COMPARISON & CHANGES
	################################################
	$commits_change = "";
	$rollbacks_change = "";
	$block_reads_change = "";
	$buffer_hits_change = "";
	$inserts_change = "";
	$updates_change = "";
	$deletes_change = "";
	$seq_scans_change = "";
	$idx_scans_change = "";
	$iud_inserts_change = "";
	$iud_updates_change = "";
	$iud_deletes_change = "";

	#####################
	#BAR CHART COLOR VARS
	#####################
	$bar_vert_alt_color = "#FFFFFF";
	$bar_vert_peak_color = "#FF6633";
	$bar_vert_color = "#7B7B7B";
	$bar_border_color = "#444444";

	##############
	#GRAPHICS VARS
	##############
	$stat_a1_src = "$graphics_src/stat_arrow_up.png";
	$stat_a2_src = "$graphics_src/stat_arrow_down.png";
	$stat_a3_src = "$graphics_src/stat_arrow_equal.png";
	$stat_a4_src = "$graphics_src/stat_arrow_no_data.png";

	#########
	#IUD VARS
	#########
	$iud_insert_color = "#7B7B7B";
	$iud_update_color = "#C8C8C8";
	$iud_delete_color = "#F1F1F1";
	$iud_nodata_color = "#000000";
	$iud_bar_width = 865;
	$iud_bar_height = 20;

	#####################################################################
	#VARS WHICH WILL TRACK THE HIGHEST STAT PER TYPE AND DATE OF THE PEAK
	#####################################################################
	$highest_commits = 0;
	$highest_commits_day = "";
	$highest_commits_date = "";

	$highest_rollbacks = 0;
	$highest_rollbacks_day = "";
	$highest_rollbacks_date = "";

	$highest_block_reads = 0;
	$highest_block_reads_day = "";
	$highest_block_reads_date = "";

	$highest_buffer_hits = 0;
	$highest_buffer_hits_day = "";
	$highest_buffer_hits_date = "";
	
	$highest_inserts = 0;
	$highest_inserts_day = "";
	$highest_inserts_date = "";

	$highest_updates = 0;
	$highest_updates_day = "";
	$highest_updates_date = "";

	$highest_deletes = 0;
	$highest_deletes_day = "";
	$highest_deletes_date = "";

	$highest_seq_scans = 0;
	$highest_seq_scans_day = "";
	$highest_seq_scans_date = "";

	$highest_idx_scans = 0;
	$highest_idx_scans_day = "";
	$highest_idx_scans_date = "";

	$highest_seq_rows = 0;
	$highest_seq_rows_day = "";
	$highest_seq_rows_date = "";

	$highest_idx_rows = 0;
	$highest_idx_rows_day = "";
	$highest_idx_rows_date = "";

	############################################################
	#GO THROUGH THE LOG - LINE BY LINE AND RETRIEVE ALL THE DATA
	############################################################
	while(!feof($open_log))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log);
		
		###################################
		#MAKE SURE WE HAVE A LINE WITH DATA
		###################################
		if(!empty($db_line_single))
		{
			#####################################################################################
			#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE - GET THE COMMIT TOTALS
			#####################################################################################
			
			################################################################
			#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
			#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
			################################################################
			
			if(substr($db_line_single,0,2) == $log_month && substr($db_line_single,6,4) == $log_year)
			{
				#############
				#FLAG THE DAY
				#############
				$day_flag = substr($db_line_single,3,2);

				#############
				#EXPLODE LINE
				#############
				$log_line = explode(",",$db_line_single);
				
				#############################
				#GET THE COMMITS FOR THIS DAY
				#############################
				if(!empty($log_line[3])) { $commits = $log_line[3] + 0; ${'db_commits_' . $day_flag} = $commits; } else { ${'db_commits_' . $day_flag} = 0; }

				##########################################################################################################
				#OBTAIN OUR ROLLBACKS / BLOCK READS / BUFFER HITS AND USER TABLE INSERT/UPDATE/DELETE STATS FROM THIS LINE
				#FORCE TYPE CAST BY ADDING ZERO TO EACH STAT VALUE
				##########################################################################################################
				if(!empty($log_line[4])) { $rollbacks = $log_line[4] + 0; ${'db_rollbacks_' . $day_flag} = $rollbacks; } else { ${'db_rollbacks_' . $day_flag} = 0; }
				if(!empty($log_line[5])) { $block_reads = $log_line[5] + 0; ${'db_block_reads_' . $day_flag} = $block_reads; } else { ${'db_block_reads_' . $day_flag} = 0; }
				if(!empty($log_line[6])) { $buffer_hits = $log_line[6] + 0; ${'db_buffer_hits_' . $day_flag} = $buffer_hits; } else { ${'db_buffer_hits_' . $day_flag} = 0; }
				if(!empty($log_line[7])) { $user_table_inserts = $log_line[7] + 0; ${'db_inserts_' . $day_flag} = $user_table_inserts; } else { ${'db_inserts_' . $day_flag} = 0; }
				if(!empty($log_line[8])) { $user_table_updates = $log_line[8] + 0; ${'db_updates_' . $day_flag} = $user_table_updates; } else { ${'db_updates_' . $day_flag} = 0; }
				if(!empty($log_line[9])) { $user_table_deletes = $log_line[9] + 0; ${'db_deletes_' . $day_flag} = $user_table_deletes; } else { ${'db_deletes_' . $day_flag} = 0; }
				if(!empty($log_line[10])) { $seq_scans = $log_line[10] + 0; ${'db_seq_scans_' . $day_flag} = $seq_scans; } else { ${'db_seq_scans_' . $day_flag} = 0; }
				if(!empty($log_line[11])) { $idx_scans = $log_line[11] + 0; ${'db_idx_scans_' . $day_flag} = $idx_scans; } else { ${'db_idx_scans_' . $day_flag} = 0; }
				if(!empty($log_line[12])) { $seq_rows = $log_line[12] + 0; ${'db_seq_rows_' . $day_flag} = $seq_rows; } else { ${'db_seq_rows_' . $day_flag} = 0; }
				if(!empty($log_line[13])) { $idx_rows = $log_line[13] + 0; ${'db_idx_rows_' . $day_flag} = $idx_rows; } else { ${'db_idx_rows_' . $day_flag} = 0; }

				#####################################################################
				#TALL UP THE DAYS TRACKED, TOTAL COMMIT, INSERTS, UPDATES AND DELETES
				#####################################################################
				$total_days_tracked++;
				$total_commits += $commits;
				$total_rollbacks += $rollbacks;
				$total_block_reads += $block_reads;
				$total_buffer_hits += $buffer_hits;
				$total_inserts += $user_table_inserts;
				$total_updates += $user_table_updates;
				$total_deletes += $user_table_deletes;
				$total_seq_scans += $seq_scans;
				$total_idx_scans += $idx_scans;
				$total_seq_rows += $seq_rows;
				$total_idx_rows += $idx_rows;

				############################################################################
				#TALLY UP OUR "HIGHEST" VARS SO THAT WE CAN DETERMINE BAR CHART VALUES LATER
				############################################################################
				if($commits > $highest_commits)
				{
					$highest_commits = $commits;
					$highest_commits_day = $day_flag;
					$highest_commits_date = "$log_month-$day_flag-$log_year";	
				}
				if($rollbacks > $highest_rollbacks)
				{
					$highest_rollbacks = $rollbacks;
					$highest_rollbacks_day = $day_flag;
					$highest_rollbacks_date = "$log_month-$day_flag-$log_year";
				}
				if($block_reads > $highest_block_reads)
				{
					$highest_block_reads = $block_reads;
					$highest_block_reads_day = $day_flag;
					$highest_block_reads_date = "$log_month-$day_flag-$log_year";
				}
				if($buffer_hits > $highest_buffer_hits)
				{
					$highest_buffer_hits = $buffer_hits;
					$highest_buffer_hits_day = $day_flag;
					$highest_buffer_hits_date = "$log_month-$day_flag-$log_year";
				}
				if($user_table_inserts > $highest_inserts)
				{
					$highest_inserts = $user_table_inserts;
					$highest_inserts_day = $day_flag;
					$highest_inserts_date = "$log_month-$day_flag-$log_year";
				}
				if($user_table_updates > $highest_updates)
				{
					$highest_updates = $user_table_updates;
					$highest_updates_day = $day_flag;
					$highest_updates_date = "$log_month-$day_flag-$log_year";
				}
				if($user_table_deletes > $highest_deletes)
				{
					$highest_deletes = $user_table_deletes;
					$highest_deletes_day = $day_flag;
					$highest_deletes_date = "$log_month-$day_flag-$log_year";
				}
				if($seq_scans > $highest_seq_scans)
				{
					$highest_seq_scans = $seq_scans;
					$highest_seq_scans_day = $day_flag;
					$highest_seq_scans_date = "$log_month-$day_flag-$log_year";
				}
				if($idx_scans > $highest_idx_scans)
				{
					$highest_idx_scans = $idx_scans;
					$highest_idx_scans_day = $day_flag;
					$highest_idx_scans_date = "$log_month-$day_flag-$log_year";
				}
				if($seq_rows > $highest_seq_rows)
				{
					$highest_seq_rows = $seq_rows;
					$highest_seq_rows_day = $day_flag;
					$highest_seq_rows_date = "$log_month-$day_flag-$log_year";
				}
				if($idx_rows > $highest_idx_rows)
				{
					$highest_idx_rows = $idx_rows;
					$highest_idx_rows_day = $day_flag;
					$highest_idx_rows_date = "$log_month-$day_flag-$log_year";
				}
			}
		}	
	}
	
	#############################
	#CLOSE UP THE FILE - ALL DONE
	#############################
	fclose($open_log);
	
	#########################################################################
	#CHECK AND SEE IF STATS ARE AVAILABLE FOR THE PREVIOUS CYCLE
	#IF SO, WE'LL PROVIDE UP/DOWN ARROWS TO SHOW A QUICK A/B OF THE TWO CYCLE
	#########################################################################
	
	########################################
	#PUT THE PREVIOUS LOG FILE NAME TOGETHER
	########################################
	if($log_file_name == "01")
	{
		$prev_log_file_name = "12";
		$prev_log_year = $log_year - 1;
	}
	else
	{
		$prev_log_file_name = $log_file_name;
		
		##################################
		#REVERSE LEADING ZERO IF NECESSARY
		#AND FORCE TYPE CAST TO INTEGER
		##################################
		if(substr($prev_log_file_name,0,1) == "0")
		{
			$prev_log_file_name += 0;
			$prev_log_file_name --;
			
			####################################################
			#FORCE TYPE CAST BACK TO STRING AND ADD LEADING ZERO
			####################################################
			$prev_log_file_name = "0$prev_log_file_name"; 
		}
		else
		{
			#############################
			#TWO DIGIT MONTH WITH NO ZERO
			#############################
			$prev_log_file_name = $prev_log_file_name -1;
			
			##############################
			#CHECK AGAIN FOR LEADING ZEROS
			##############################
			if(strlen($prev_log_file_name) == 1) $prev_log_file_name = "0$prev_log_file_name";
		}
		
		##########################
		#PUT OUR YEAR VAR TOGETHER
		##########################
		$prev_log_year = $log_year;
	}

	######################################
	#DEFAULT PREV_STATS_AVAILABLE TO FALSE
	######################################
	$prev_stats_available = false;

	###########################################################
	#NOW LETS SEE IF STATS ARE AVAILABLE FOR THE PREVIOUS MONTH
	###########################################################
	if(file_exists("$postgrestats_dir/postgrestats/data/$prev_log_year/$prev_log_file_name"))
	{
		############################
		#UPDATE PREV_STATS_AVAILABLE
		############################
		$prev_stats_available = true;

		#########################
		#ATTEMPT TO OPEN THE FILE
		#########################
		if(!$open_prev_log = fopen("$postgrestats_dir/postgrestats/data/$prev_log_year/$prev_log_file_name", "r"))
		{
			$stat_msg = "Unable to open previous stats log file.";
			echo $stat_msg;
			exit;
		}

		##########################################################
		#ALL IS GOOD, LET'S DIG IN AND GET OUR PREVIOUS STATISTICS
		##########################################################

		#################################
		#LET'S GO THROUGH IT LINE BY LINE
		#################################
		while(!feof($open_prev_log))
		{
			#############
			#GET ONE LINE
			#############
			$db_line_single = fgets($open_prev_log);
			
			###################################
			#MAKE SURE WE HAVE A LINE WITH DATA
			###################################
			if(!empty($db_line_single))
			{
				#####################################################################################
				#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE - GET THE COMMIT TOTALS
				#####################################################################################
				
				################################################################
				#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
				#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
				################################################################

				if(substr($db_line_single,0,2) == $prev_log_file_name && substr($db_line_single,6,4) == $prev_log_year)
				{
					#############
					#FLAG THE DAY
					#############
					$day_flag = substr($db_line_single,3,2);
					
					#############
					#EXPLODE LINE
					#############
					$log_line = explode(",",$db_line_single);
					
					#############################
					#GET THE COMMITS FOR THIS DAY
					#############################
					if(!empty($log_line[3])) { $commits = $log_line[3] + 0; ${'db_commits_prev_' . $day_flag} = $commits; } else { ${'db_commits_prev_' . $day_flag} = 0; }

					##########################################################################################################
					#OBTAIN OUR ROLLBACKS / BLOCK READS / BUFFER HITS AND USER TABLE INSERT/UPDATE/DELETE STATS FROM THIS LINE
					#FORCE TYPE CAST TO EACH STAT VALUE TO MAKE CERTAIN IT'S NUMERIC
					##########################################################################################################
					if(!empty($log_line[4])) { $rollbacks = $log_line[4] + 0; ${'db_rollbacks_prev_' . $day_flag} = $rollbacks; } else { ${'db_rollbacks_prev_' . $day_flag} = 0; }
					if(!empty($log_line[5])) { $block_reads = $log_line[5] + 0; ${'db_block_reads_prev_' . $day_flag} = $block_reads; } else { ${'db_block_reads_prev_' . $day_flag} = 0; }
					if(!empty($log_line[6])) { $buffer_hits = $log_line[6] + 0; ${'db_buffer_hits_prev_' . $day_flag} = $buffer_hits; } else { ${'db_buffer_hits_prev_' . $day_flag} = 0; }
					if(!empty($log_line[7])) { $user_table_inserts = $log_line[7] + 0; ${'db_inserts_prev_' . $day_flag} = $user_table_inserts; } else { ${'db_inserts_prev_' . $day_flag} = 0; }
					if(!empty($log_line[8])) { $user_table_updates = $log_line[8] + 0; ${'db_updates_prev_' . $day_flag} = $user_table_updates; } else { ${'db_updates_prev_' . $day_flag} = 0; }
					if(!empty($log_line[9])) { $user_table_deletes = $log_line[9] + 0; ${'db_deletes_prev_' . $day_flag} = $user_table_deletes; } else { ${'db_deletes_prev_' . $day_flag} = 0; }
					if(!empty($log_line[10])) { $seq_scans = $log_line[10] + 0; ${'db_seq_scans_prev_' . $day_flag} = $seq_scans; } else { ${'db_seq_scans_prev_' . $day_flag} = 0; }
					if(!empty($log_line[11])) { $idx_scans = $log_line[11] + 0; ${'db_idx_scans_prev_' . $day_flag} = $idx_scans; } else { ${'db_idx_scans_prev_' . $day_flag} = 0; }
					if(!empty($log_line[12])) { $seq_rows = $log_line[12] + 0; ${'db_seq_rows_prev_' . $day_flag} = $seq_rows; } else { ${'db_seq_rows_prev_' . $day_flag} = 0; }
					if(!empty($log_line[13])) { $idx_rows = $log_line[13] + 0; ${'db_idx_rows_prev_' . $day_flag} = $idx_rows; } else { ${'db_idx_rows_prev_' . $day_flag} = 0; }

					################################################################################
					#TALL UP IUD AND GENERAL STATISTICS FOR THE IUD QUICK VIEW AND GENERAL +/- STATS
					################################################################################
					$total_commits_prev += $commits;
					$total_rollbacks_prev += $rollbacks;
					$total_block_reads_prev += $block_reads;
					$total_buffer_hits_prev += $buffer_hits;
					$total_inserts_prev += $user_table_inserts;
					$total_updates_prev += $user_table_updates;
					$total_deletes_prev += $user_table_deletes;
					$total_seq_scans_prev += $seq_scans;
					$total_idx_scans_prev += $idx_scans;
					$total_seq_rows_prev += $seq_rows;
					$total_idx_rows_prev += $idx_rows;
				}
			}	
		}
		
		#############################
		#CLOSE UP THE FILE - ALL DONE
		#############################
		fclose($open_prev_log);
	}

################################
#START CREATING OUR INITIAL HTML
################################
$ajax_return = <<<AJR
<div class="stat_wrapper" style="margin:10px 0 25px 0;">
<table class="stat_chart_table" style="width:auto;">
<tr>
AJR;
	
	#####################################################################################
	#PUT TOGETHER THE MONTH TABLEWORK - ITERATING THROUGH OUR DB VARS - CHECKING EACH DAY
	#####################################################################################
	for($i=1; $i<32; $i++)
	{
		################################
		#INITIATE/RESET VARS IN THE LOOP
		################################
		$tooltip = "";
		$bar_height_percent = 0;
		$bar_height = 0;
		$bar_height_final = 0;
		$bar_height_alt = 0;

		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";

		###############################################################################################
		#DETERMINE WHAT TYPES OF STATS WE'LL BE BAR CHARTTING FIRST
		#CHECK FOR EACH DAY AND HANDLE DAYS WITH ZERO STATS - DEFAULT THE LIGHT GREY BAR HEIGHT TO 100%
		###############################################################################################
		$no_stat = false;

		################################################
		#CHECK THE STAT TYPE AND SEE IF THERE IS A VALUE
		################################################
		if(!isset(${'db_' . $snap_chartting . '_' . $i}))
		{
			$bar_height_alt = $bar_height_vert;
			$no_stat = true;
		}

		##################################################
		#IF WE HAVE STATS, PREPARE THE TOOLTIP, BARS, ETC.
		##################################################
		if($no_stat == false)
		{	
			##################################
			#FIRST PREP OUR COMMIT TOOLTIP VAR
			##################################
			$tt_commits = ${'db_commits_' . $i};
			if($tt_commits == 0 || empty($tt_commits)) $tt_commits = "0";
			else $tt_commits = number_format($tt_commits);

			###############################
			#PREP OUR ROLLBACKS TOOLTIP VAR
			###############################
			$tt_rollbacks = ${'db_rollbacks_' . $i};
			if($tt_rollbacks == 0 || empty($tt_rollbacks)) $tt_rollbacks = "0";
			else $tt_rollbacks = number_format($tt_rollbacks);

			#################################
			#PREP OUR BLOCK READS TOOLTIP VAR
			#################################
			$tt_block_reads = ${'db_block_reads_' . $i};
			if($tt_block_reads == 0 || empty($tt_block_reads)) $tt_block_reads = "0";
			else $tt_block_reads = number_format($tt_block_reads);

			#################################
			#PREP OUR BUFFER HITS TOOLTIP VAR
			#################################
			$tt_buffer_hits = ${'db_buffer_hits_' . $i};
			if($tt_buffer_hits == 0 || empty($tt_buffer_hits)) $tt_buffer_hits = "0";
			else $tt_buffer_hits = number_format($tt_buffer_hits);

			############################
			#PREP OUR INSERT TOOLTIP VAR
			############################
			$tt_inserts = ${'db_inserts_' . $i};
			if($tt_inserts == 0 || empty($tt_inserts)) $tt_inserts = "0";
			else $tt_inserts = number_format($tt_inserts);

			############################
			#PREP OUR UPDATE TOOLTIP VAR
			############################
			$tt_updates = ${'db_updates_' . $i};
			if($tt_updates == 0 || empty($tt_updates)) $tt_updates = "0";
			else $tt_updates = number_format($tt_updates);

			############################
			#PREP OUR DELETE TOOLTIP VAR
			############################
			$tt_deletes = ${'db_deletes_' . $i};
			if($tt_deletes == 0 || empty($tt_deletes)) $tt_deletes = "0";
			else $tt_deletes = number_format($tt_deletes);

			###############################
			#PREP OUR SEQ SCANS TOOLTIP VAR
			###############################
			$tt_seq_scans = ${'db_seq_scans_' . $i};
			if($tt_seq_scans == 0 || empty($tt_seq_scans)) $tt_seq_scans = "0";
			else $tt_seq_scans = number_format($tt_seq_scans);

			###############################
			#PREP OUR IDX SCANS TOOLTIP VAR
			###############################
			$tt_idx_scans = ${'db_idx_scans_' . $i};
			if($tt_idx_scans == 0 || empty($tt_idx_scans)) $tt_idx_scans = "0";
			else $tt_idx_scans = number_format($tt_idx_scans);

			#######################################################
			#CHECK TO SEE IF WE HAVE STATS FOR PREVIOUS CYCLE
			#IF SO, CALCULATE THE PERCENTAGE UP OR DOWN AND DISPLAY
			#######################################################
			if($prev_stats_available == true)
			{
				##############################################################################
				#FIRST CHECK TO SEE IF PREVIOUS STAT DATE IS A REAL DATE
				#WE DON'T WANT TO SHOW PREVIOUS STATS FOR MARCH 31ST IF FEB 31ST DOESN'T EXIST
				##############################################################################
				if(substr($i,0,1) == "0") $dc_day = substr($i,1,1);
				else $dc_day = "$i";

				if(substr($prev_log_file_name,0,1) == "0") $dc_month = substr($prev_log_file_name,1,1);
				else $dc_month = "$prev_log_file_name";

				if(checkdate($dc_month,$dc_day,$prev_log_year)) $dc = true;
				else $dc = false;

				##############################
				#DEFAULT OUR CHANGE VAR VALUES
				##############################
				$change = "";
				$commits_change = "";
				$rollbacks_change = "";
				$block_reads_change = "";
				$buffer_hits_change = "";
				$inserts_change = "";
				$updates_change = "";
				$deletes_change = "";
				$seq_scans_change = "";
				$idx_scans_change = "";

				##############################################################################
				#FIRST CHECK TO SEE IF PREVIOUS STAT DATE IS A REAL DATE
				#WE DON'T WANT TO SHOW PREVIOUS STATS FOR MARCH 31ST IF FEB 31ST DOESN'T EXIST
				##############################################################################
				if($dc == true)
				{
					#############################################
					#CHECK STATISTICS INCREASE / DECREASE
					#SHOW PREVIOUS FIGURES IN THE TOOLTIP AS WELL
					#############################################
					
					###################################################################################
					#SET UP AN ARRAY OF THE STAT TYPES SO WE CAN ITERATE THROUGH THEM FOR THE NEXT STEP
					###################################################################################
					$stat_type_array = array("commits","rollbacks","block_reads","buffer_hits","inserts","updates","deletes","seq_scans","idx_scans");
					
					###############################
					#ITERATE THROUGH EACH STAT TYPE
					###############################
					foreach($stat_type_array as &$stat_type)
					{
						#######################
						#FORCE NUMBER TYPE CASE
						#######################
						if(isset(${'db_' . $stat_type . '_prev_' . $i})) ${'db_' . $stat_type . '_prev_' . $i} += 0;
						if(isset(${'db_' . $stat_type . '_' . $i})) ${'db_' . $stat_type . '_' . $i} += 0;
						
						##############################################################
						#OBTAIN CHANGE STAT ONLY IF BOTH CURRENT AND PREV STATS EXISTS
						##############################################################
						if(isset(${'db_' . $stat_type . '_' . $i}) && isset(${'db_' . $stat_type . '_prev_' . $i}))
						{
							####################
							#HANDLE THE INCREASE
							####################
							if(${'db_' . $stat_type . '_' . $i} > ${'db_' . $stat_type . '_prev_' . $i})
							{
								if(${'db_' . $stat_type . '_prev_' . $i} == 0)
								{
									$change = ${'db_' . $stat_type . '_' . $i} * 100;
								}
								else
								{
									$change = ${'db_' . $stat_type . '_' . $i} - ${'db_' . $stat_type . '_prev_' . $i};
									$change = ($change / ${'db_' . $stat_type . '_prev_' . $i}) * 100;
								}
								
								$change = number_format($change, 2, '.', ',') . "%";
								${$stat_type . '_change'} = "&nbsp;(+" . $change . ")";
							}
							####################
							#HANDLE THE DECREASE
							####################
							elseif(${'db_' . $stat_type . '_prev_' . $i} > ${'db_' . $stat_type . '_' . $i})
							{
								if(${'db_' . $stat_type . '_' . $i} == 0)
								{
									$change = ${'db_' . $stat_type . '_prev_' . $i} * 100;
								}
								else
								{
									$change = ${'db_' . $stat_type . '_prev_' . $i} - ${'db_' . $stat_type . '_' . $i};
									$change = ($change / ${'db_' . $stat_type . '_' . $i}) * 100;
								}
								
								$change = number_format($change, 2, '.', ',') . "%";
								${$stat_type . '_change'} = "&nbsp;(-" . $change . ")";
							}
						}
					}
				}
			}

			################
			#PUT IT TOGETHER
			################
			$tooltip = "<div style=line-height:17px;>";

			########
			#COMMITS
			########
			if($snap_chartting == "commits") $tooltip .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits . $commits_change . "</strong></span><br>";
			else $tooltip .= "Commits:&nbsp;<strong>" . $tt_commits . "</strong>" . $commits_change . "<br>";

			##########
			#ROLLBACKS
			##########
			if($snap_chartting == "rollbacks") $tooltip .= "<span class=body_highlight><strong>Rollbacks:&nbsp;" . $tt_rollbacks . $rollbacks_change . "</strong></span><br>";
			else $tooltip .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks . "</strong>" . $rollbacks_change . "<br>";

			############
			#BLOCK READS
			############
			if($snap_chartting == "block_reads") $tooltip .= "<span class=body_highlight><strong>Block&nbsp;Reads:&nbsp;" . $tt_block_reads . $block_reads_change . "</strong></span><br>";
			else $tooltip .= "Block&nbsp;Reads:&nbsp;<strong>" . $tt_block_reads . "</strong>" . $block_reads_change . "<br>";

			############
			#BUFFER HITS
			############
			if($snap_chartting == "buffer_hits") $tooltip .= "<span class=body_highlight><strong>Buffer&nbsp;Hits:&nbsp;" . $tt_buffer_hits . $buffer_hits_change . "</strong></span><br>";
			else $tooltip .= "Buffer&nbsp;Hits:&nbsp;<strong>" . $tt_buffer_hits . "</strong>" . $buffer_hits_change . "<br>";

			########
			#INSERTS
			########
			if($snap_chartting == "inserts") $tooltip .= "<span class=body_highlight><strong>Inserts:&nbsp;" . $tt_inserts . $inserts_change . "</strong></span><br>";
			else $tooltip .= "Inserts:&nbsp;<strong>" . $tt_inserts . "</strong>" . $inserts_change . "<br>";

			########
			#UPDATES
			########
			if($snap_chartting == "updates") $tooltip .= "<span class=body_highlight><strong>Updates:&nbsp;" . $tt_updates . $updates_change . "</strong></span><br>";
			else $tooltip .= "Updates:&nbsp;<strong>" . $tt_updates . "</strong>" . $updates_change . "<br>";

			########
			#DELETES
			########
			if($snap_chartting == "deletes") $tooltip .= "<span class=body_highlight><strong>Deletes:&nbsp;" . $tt_deletes . $deletes_change . "</strong></span><br>";
			else $tooltip .= "Deletes:&nbsp;<strong>" . $tt_deletes . "</strong>" . $deletes_change . "<br>";

			##########
			#SEQ SCANS
			##########
			if($snap_chartting == "seq_scans") $tooltip .= "<span class=body_highlight><strong>Sequence&nbsp;Scans:&nbsp;" . $tt_seq_scans . $seq_scans_change . "</strong></span><br>";
			else $tooltip .= "Sequence&nbsp;Scans:&nbsp;<strong>" . $tt_seq_scans . "</strong>" . $seq_scans_change . "<br>";

			##########
			#IDX SCANS
			##########
			if($snap_chartting == "idx_scans") $tooltip .= "<span class=body_highlight><strong>Index&nbsp;Scans:&nbsp;" . $tt_idx_scans . $idx_scans_change . "</strong></span><br>";
			else $tooltip .= "Index&nbsp;Scans:&nbsp;<strong>" . $tt_idx_scans . "</strong>" . $idx_scans_change . "<br>";

			#########################
			#COMPLETE THE TOOLTIP DIV
			#########################
			$tooltip .= "</div>";

			###################################################################################################
			#GET THE STAT PERCENT FOR THE APPROPRIATE STAT TYPE AND DEFINE THE BAR HEIGHT FOR THE DARK GREY BAR
			###################################################################################################	
			if(!empty(${'highest_' . $snap_chartting})) { $stat_percent = ${'db_' . $snap_chartting . '_' . $i} / ${'highest_' . $snap_chartting}; $bar_height_percent = $stat_percent; } else { $bar_height_percent = 0; }

			#################################
			#COMPLETE OUR MATH FOR THE HEIGHT
			#################################
			$bar_height = $bar_height_vert * $bar_height_percent;
			$bar_height_final = number_format($bar_height, 0, '.', '');
			
			############################
			#CHECK THE HEIGHT - FOR SAFE
			############################
			if($bar_height_final > $bar_height_vert) $bar_height_final = $bar_height_vert;
			
			####################################################################################
			#CREATE THE HEIGHT FOR THE ALT LIGHT GREY BAR WHICH SITS ON TOP OF THE DARK GREY BAR
			####################################################################################
			$bar_height_alt = $bar_height_vert - $bar_height_final;
			
			#############################################################################
			#HANDLE THE FINAL SOURCE GRAPHIC AND CHECK FOR THE PEAK DAY - FLAGGED EARLIER
			#############################################################################
			if($i == ${'highest_' . $snap_chartting . '_day'}) $bar_vert_color_final = $bar_vert_peak_color;
			else $bar_vert_color_final = $bar_vert_color;
		}
	
		$ajax_return .= "<td valign=\"bottom\" style=\"text-align:center; padding-right:5px;\">";

		#############################
		#CREATE FINAL CELL DIMENSIONS
		#############################
		$bar_width_px = $bar_width . "px";

		#######################################################
		#START UP THE TABLEWORK WHICH WILL HOUSE THE BAR ITSELF
		#######################################################
		$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$bar_width_px; empty-cells:show;\">";

		############################
		#DETERMINE BORDER STYLES NOW
		############################
		$bar_border = "border:1px solid $bar_border_color";
		if($bar_height_final <= 0) $bar_alt_border = "border:1px solid $bar_border_color";
		else $bar_alt_border = "border-color:$bar_border_color; border-width: 1px 1px 0px 1px; border-style: solid;"; 

		#####################################################################################
		#HANDLE THE ALT COLOR - WILL EITHER BE THE ENTIRE BAR (NO DATA) OR THE TOP OF THE BAR
		#####################################################################################
		if($bar_height_alt > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_alt_px = $bar_height_alt . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_px; background-color:$bar_vert_alt_color; $bar_alt_border\"></td></tr>";
		}
		
		################################################
		#HANDLE THE BOTTOM OF THE BAR WHERE WE HAVE DATA
		################################################
		if($bar_height_final > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_final_px = $bar_height_final . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<tr><td onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()' style=\"width:$bar_width_px; height:$bar_height_final_px; background-color:$bar_vert_color_final; $bar_border\"></td></tr>";
		}

		########################
		#COMPLETE CELL TABLEWORK
		########################
		$ajax_return .= "</table>";

		################################################
		#FINISHED WITH THE BAR TABLE - CONTINUE OUR HTML
		################################################
		$ajax_return .= "</td>";
	}

$ajax_return .= <<<AJR
</tr>
<tr>
AJR;
	
	###############################################################################################
	#CREATE THE MONTH NUMBERS AND IF THERE ARE COMMITS FOR EACH DAY - MAKE A QUICK LINK TO THAT DAY
	###############################################################################################
	for($i=1; $i<32; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";
		
		##############################
		#HANDLE THE TOOLTIP COMMIT MSG
		##############################
		$tooltip = "Retrieve&nbsp;Stats";
		$ajax_return .= "<td class=\"body_secondary\" style=\"padding-top:5px; text-align:center; padding-right:5px;\">";

		###############################
		#CHECK COMMITS AND SET THE LINK
		###############################
		if(isset(${'db_commits_' . $i}))
		{
			$ajax_return .= "<span onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$log_month','$i','$log_year')>$i</a></span>";
		}
		else
		{
			$ajax_return .= $i;
		}
		
		$ajax_return .= "</td>";
	}
	
	#############################################
	#HANDLE PREVIOUS STATS FOR THE IUD QUICK VIEW
	#############################################
	if($prev_stats_available == true)
	{
		###################################################################################
		#SET UP AN ARRAY OF THE STAT TYPES SO WE CAN ITERATE THROUGH THEM FOR THE NEXT STEP
		###################################################################################
		$stat_type_array = array("inserts","updates","deletes");
		
		#######################
		#START UP THE ITERATION
		#######################
		foreach($stat_type_array as &$stat_type)
		{
			##########################################
			#FOR IUD COMPARISONS - HANDLE THE INCREASE
			##########################################
			if(${'total_' . $stat_type} > ${'total_' . $stat_type . '_prev'})
			{
				if(${'total_' . $stat_type . '_prev'} == 0)
				{
					$change = ${'total_' . $stat_type} * 100;
				}
				else
				{
					$change = ${'total_' . $stat_type} - ${'total_' . $stat_type . '_prev'};
					$change = ($change / ${'total_' . $stat_type . '_prev'}) * 100;
				}
				
				$change = number_format($change, 2, '.', ',') . "%";
				${'iud_' . $stat_type . '_change'} = "&nbsp;(+" . $change . ")";
			}
			####################
			#HANDLE THE DECREASE
			####################
			elseif(${'total_' . $stat_type . '_prev'} > ${'total_' . $stat_type})
			{
				if(${'total_' . $stat_type} == 0)
				{
					$change = ${'total_' . $stat_type . '_prev'} * 100;
				}
				else
				{
					$change = ${'total_' . $stat_type . '_prev'} - ${'total_' . $stat_type};
					$change = ($change / ${'total_' . $stat_type}) * 100;
				}
				
				$change = number_format($change, 2, '.', ',') . "%";
				${'iud_' . $stat_type . '_change'} = "&nbsp;(-" . $change . ")";
			}
		}
	}
	
	##################################
	#PUT THE LAST OF OUR HTML TOGETHER
	##################################
$ajax_return .= <<<AJR
</tr>
<tr>
AJR;

	###########################################################################################################
	#CALCULATE THE STAT ARROWS SHOWING VISITOR WHETHER PREVIOUS STATS ARE UP, DOWN OR EQUAL TO THE CURRENT STAT
	#CREATE THE MONTH NUMBERS AND IF THERE ARE COMMITS FOR EACH DAY - MAKE A QUICK LINK TO THAT DAY
	###########################################################################################################
	for($i=1; $i<32; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";
		
		##################
		#START-UP OUR HTML
		##################
		$ajax_return .= "<td class=\"body_secondary\" style=\"padding-top:3px; text-align:center; padding-right:5px;\">";

		#############################################################################
		#CHECK TO SEE IF WE HAVE STATS FOR PREVIOUS CYCLE
		#ALSO CHECK TO MAKE SURE WE ONLY SHOW ICONS FOR DAYS WITH STATS WHICH WE HAVE
		#SHOW 'NO-DATA' ICON FOR ALL OTHER DAYS AFTER
		#############################################################################	
		if($prev_stats_available == true && isset(${'db_' . $snap_chartting . '_' . $i}))
		{
			###########################
			#CHECK FOR NO PREVIOUS DATA
			###########################
			if(!isset(${'db_' . $snap_chartting . '_prev_' . $i})) $ajax_return .= "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";

			###########################
			#CHECK FOR STATS INCREASING
			###########################
			elseif(${'db_' . $snap_chartting . '_' . $i} > ${'db_' . $snap_chartting . '_prev_' . $i}) $ajax_return .= "<img src=\"$stat_a1_src\" width=\"10\" height=\"8\"/>";

			###########################
			#CHECK FOR STATS DECREASING
			###########################
			elseif(${'db_' . $snap_chartting . '_' . $i} < ${'db_' . $snap_chartting . '_prev_' . $i}) $ajax_return .= "<img src=\"$stat_a2_src\" width=\"10\" height=\"8\"/>";

			#################################
			#CHECK FOR STATS EXACTLY THE SAME
			#################################
			elseif(${'db_' . $snap_chartting . '_' . $i} == ${'db_' . $snap_chartting . '_prev_' . $i}) $ajax_return .= "<img src=\"$stat_a3_src\" width=\"10\" height=\"8\"/>";
		}
		else
		{
			################################
			#FORCE THE NO-DATA ICON FOR SAFE
			################################
			$ajax_return .= "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		}

		##################
		#COMPLETE THE CELL
		##################
		$ajax_return .= "</td>";
	}

	###############################################################################################
	#DETERMINE THE TOTAL STATS FOR THIS TYPE - THIS WILL BE DISPLAYED TO THE RIGHT OF THE BAR CHART
	###############################################################################################
	$recap_average_day = ${'total_' . $snap_chartting} / $total_days_tracked;
	$recap_average_day = number_format($recap_average_day, 0, '', ',');
	$recap_peak = number_format(${'highest_' . $snap_chartting}, 0, '', ',');
	$recap_total = number_format(${'total_' . $snap_chartting});

	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($snap_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes"; break;
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}

	###############################################################
	#PARSE THE PEAK DATE SO WE CAN LINK IT TO THE DAIL DB STAT VIEW
	###############################################################
	$peak_month = substr(${'highest_' . $snap_chartting . '_date'},0,2);
	$peak_day = substr(${'highest_' . $snap_chartting . '_date'},3,2);
	$peak_year = $log_year;
	$tooltip = "Retrieve Stats";

###################################################
#PUT THE LAST OF OUR HTML TOGETHER FOR THE BAR AREA
###################################################
$ajax_return .= <<<AJR
</tr>
</table>
AJR;

###########################
#DISPLAY TARGET STAT TOTALS
###########################
$ajax_return .= "<div class=\"stat_sub_wrapper body_primary\" style=\"float:left; margin:20px 15px 0 0; padding:10px; font-weight:bold;\">";
$ajax_return .= "<span class=\"body_highlight\">Total&nbsp;$stat_title:&nbsp;</span>";
if($recap_total > 0) $ajax_return .= $recap_total;
else $ajax_return .= "N/A";
$ajax_return .= "</div>";

#######################################
#SHOW PEAK DAY STATS ONLY IF APPLICABLE
#######################################
$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
$ajax_return .= "Peak&nbsp;Day:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">";
if($recap_peak > 0) $ajax_return .= $recap_peak;
else $ajax_return .= "N/A";
$ajax_return .= "</span>";
if(!empty($peak_month) && !empty($peak_day) && !empty($peak_year)) $ajax_return .= "&nbsp;<span class=\"body_highlight\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'>(<a href=javascript:retrieve_daily_db_stats('$peak_month','$peak_day','$peak_year')>${'highest_' . $snap_chartting . '_date'}</a>)</span>";
$ajax_return .= "</div>";

##########################################
#SHOW AVERAGE/DAY STATS ONLY IF APPLICABLE
##########################################
$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
$ajax_return .= "Average&nbsp;/&nbsp;Day:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">";
if($recap_average_day > 0) $ajax_return .= $recap_average_day;
else $ajax_return .= "N/A";
$ajax_return .= "</span></div>";

#####################################################
#CLEAR FLOATS, COMPLETE DIV AND PROCEED TO CORE STATS
#####################################################
$ajax_return .= "<div style=\"clear:both;\"></div></div>";

###############################################################
#START UP THE TABLE WORK WHICH WILL HOUSE OUR IUD BAR AND STATS
###############################################################
$ajax_return .= <<<AJR
<div class="body_secondary stat_wrapper" style="margin-bottom:25px;"> 
<div class="body_highlight" style="margin-right:10px; float:left; font-weight:bold;">Insert&nbsp;/&nbsp;Update&nbsp;/&nbsp;Delete&nbsp;Quick&nbsp;View:&nbsp;</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_insert_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Inserts</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_update_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Updates</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_delete_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Deletes</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_nodata_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">No&nbsp;Data</div>
<div style="clear:both;"></div>
AJR;

	##############################################################
	#PUT TOGETHER OUR IUD STATS - FIRST INITIATE SOME GENERAL VARS
	##############################################################
	$total_iud = $total_inserts + $total_updates + $total_deletes;
	
	##############################################
	#GET THE PERCENTAGES OF EACH AGAINST THE TOTAL
	##############################################
	if(!empty($total_iud)) { $iud_ins_percent = number_format($total_inserts / $total_iud, 2, '.', ','); } else { $iud_ins_percent = 0; }
	if(!empty($total_iud)) { $iud_upd_percent = number_format($total_updates / $total_iud, 2, '.', ','); } else { $iud_upd_percent = 0; }
	if(!empty($total_iud)) { $iud_del_percent = number_format($total_deletes / $total_iud, 2, '.', ','); } else { $iud_del_percent = 0; }

	#################################################
	#PREPARE THE BAR WIDTHS AND ROUND THE NUMBERS OUT
	#################################################
	$iud_ins_bar_w = number_format($iud_bar_width * $iud_ins_percent);
	$iud_upd_bar_w = number_format($iud_bar_width * $iud_upd_percent);
	$iud_del_bar_w = number_format($iud_bar_width * $iud_del_percent);
	
	#######################################################################################################
	#TALLY UP THE WIDTHS AND TAKE ANY NOMINAL REMAINDER +/- IUD_BAR_WIDTH AND +/- IT FROM THE LARGEST SCORE
	#######################################################################################################
	$iud_bar_total = $iud_ins_bar_w + $iud_upd_bar_w + $iud_del_bar_w;
	
	if($iud_bar_total < $iud_bar_width && $iud_bar_total > 0) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_width-$iud_bar_total;
		
		############################
		#ADD IT TO HIGHEST IUD SCORE
		############################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w += $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w += $iud_remain;
		else $iud_del_bar_w += $iud_remain;
	}
	elseif($iud_bar_total > $iud_bar_width) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_total-$iud_bar_width;
		
		###################################
		#SUBTRACT IT FROM HIGHEST IUD SCORE
		###################################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w -= $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w -= $iud_remain;
		else $iud_del_bar_w -= $iud_remain;
	}
	
	##################################
	#PUT TOGETHER OUR IUD TOOLTIP VARS
	##################################
	$iud_tt_ins = "<div style=line-height:12px;>";
	$iud_tt_ins .= "Total Inserts:&nbsp;<strong class=body_highlight>" . number_format($total_inserts) . "</strong>" . $iud_inserts_change;
	$iud_tt_ins .= "</div>";
	
	$iud_tt_upd = "<div style=line-height:12px;>";
	$iud_tt_upd .= "Total Updates:&nbsp;<strong class=body_highlight>" . number_format($total_updates) . "</strong>" . $iud_updates_change;
	$iud_tt_upd .= "</div>";
	
	$iud_tt_del = "<div style=line-height:12px;>";
	$iud_tt_del .= "Total Deletes:&nbsp;<strong class=body_highlight>" . number_format($total_deletes) . "</strong>" . $iud_deletes_change;
	$iud_tt_del .= "</div>";
	
	###########################################################################################
	#SHOW EITHER THE THREE IUD BARS TOGETHER OR THE DEFAULT NO-DATA BAR IF THERE IS NO IUD DATA
	###########################################################################################

	################################
	#PREP SOME FINAL CELL DIMENSIONS
	################################
	$iud_bar_width_px = $iud_bar_width . "px";
	$iud_bar_height_px = $iud_bar_height . "px";

	####################################
	#START WITH OUR BEGINNING TABLE WORK
	####################################
	$ajax_return .= "<div style=\"margin-top:10px;\">";
	$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$iud_bar_width_px; empty-cells:show;\">";
	$ajax_return .= "<tr>";

	if($total_iud > 0)
	{
		###########
		#INSERT BAR
		###########
		if($iud_ins_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			$ins_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_ins_bar_w_px = $iud_ins_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_ins_bar_w_px; height:$iud_bar_height_px; background-color:#7B7B7B; $ins_border\" onMouseOver=\"tooltip.show('$iud_tt_ins')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#UPDATE BAR
		###########
		if($iud_upd_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			if($iud_ins_bar_w > 0) $upd_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $upd_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_upd_bar_w_px = $iud_upd_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_upd_bar_w_px; height:$iud_bar_height_px; background-color:#C9C9C9; $upd_border\" onMouseOver=\"tooltip.show('$iud_tt_upd')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#DELETE BAR
		###########
		if($iud_del_bar_w > 0) 
		{
			############
			#PREP BORDER
			############
			if($iud_upd_bar_w > 0 || $iud_ins_bar_w > 0) $del_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $del_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_del_bar_w_px = $iud_del_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_del_bar_w_px; height:$iud_bar_height_px; background-color:#F1F1F1; $del_border\" onMouseOver=\"tooltip.show('$iud_tt_del')\" onMouseOut='tooltip.hide()'></td>";
		}
	}
	else
	{
		##################################
		#DEFAULT THE NO-DATA BAR AND WIDTH
		##################################
		$ajax_return .= "<td style=\"width:$iud_bar_width_px; height:$iud_bar_height_px; background-color:#000000; border:1px solid $bar_border_color;\" onMouseOver=\"tooltip.show('No data to report.')\" onMouseOut='tooltip.hide()'></td>";
	}

	##############################################
	#COMPLETE OUR HTML TABLE WORK FOR THE IUD AREA
	##############################################
	$ajax_return .= "</tr>";
	$ajax_return .= "</table>";
	$ajax_return .= "</div></div>";

	######################################################################
	#DETERMINE THE +/- GRAPHIC SOURCES BASED ON TOTALS AND PREVIOUS TOTALS
	#WE'LL SHOW THESE BELOW THE BAR CHART IN OUR TABLE
	######################################################################
	if($prev_stats_available == true)
	{
		###################################################################################
		#SET UP AN ARRAY OF THE STAT TYPES SO WE CAN ITERATE THROUGH THEM FOR THE NEXT STEP
		###################################################################################
		$stat_type_array = array("commits","rollbacks","block_reads","buffer_hits","inserts","updates","deletes","seq_scans","idx_scans","seq_rows","idx_rows");

		###############################
		#ITERATE THROUGH EACH STAT TYPE
		###############################
		foreach($stat_type_array as &$stat_type)
		{
			if(${'total_' . $stat_type} > ${'total_' . $stat_type . '_prev'}) ${'total_' . $stat_type . '_change'} = "<img src=\"$stat_a1_src\" width=\"10\" height=\"8\"/>";
			elseif(${'total_' . $stat_type . '_prev'} > ${'total_' . $stat_type}) ${'total_' . $stat_type . '_change'} = "<img src=\"$stat_a2_src\" width=\"10\" height=\"8\"/>";
			elseif(${'total_' . $stat_type} == ${'total_' . $stat_type . '_prev'}) ${'total_' . $stat_type . '_change'} = "<img src=\"$stat_a3_src\" width=\"10\" height=\"8\"/>";
			else ${'total_' . $stat_type . '_change'} = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		}
	}
	else
	{
		################################################
		#NO PREVIOUS STATS - SHOW GREY 'NO-DATA' GRAPHIC
		################################################
		$total_commits_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_rollbacks_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_block_reads_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_buffer_hits_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_inserts_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_updates_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_deletes_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_seq_scans_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_idx_scans_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_seq_rows_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
		$total_idx_rows_change = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
	}
	
	######################################
	#FORMAT GENERAL STATISTICS FOR COMMITS
	######################################
	$average_day = $total_commits / $total_days_tracked;
	$average_hour = $average_day / 24;
	$average_minute = $average_hour / 60;
	$average_second = $average_minute / 60;
	$average_day = number_format($average_day, 2, '.', ',');
	$average_hour = number_format($average_hour, 2, '.', ',');
	$average_minute = number_format($average_minute, 2, '.', ',');
	$average_second = number_format($average_second, 4, '.', ',');
	$total_commits = number_format($total_commits);
	$highest_commits = number_format($highest_commits);
	
	########################################
	#FORMAT GENERAL STATISTICS FOR ROLLBACKS
	########################################
	$average_day_rollbacks = $total_rollbacks / $total_days_tracked;
	$average_hour_rollbacks = $average_day_rollbacks / 24;
	$average_minute_rollbacks = $average_hour_rollbacks / 60;
	$average_second_rollbacks = $average_minute_rollbacks / 60;
	$average_day_rollbacks = number_format($average_day_rollbacks, 2, '.', ',');
	$average_hour_rollbacks = number_format($average_hour_rollbacks, 2, '.', ',');
	$average_minute_rollbacks = number_format($average_minute_rollbacks, 2, '.', ',');
	$average_second_rollbacks = number_format($average_second_rollbacks, 4, '.', ',');
	$total_rollbacks = number_format($total_rollbacks);
	$highest_rollbacks = number_format($highest_rollbacks);
	
	##########################################
	#FORMAT GENERAL STATISTICS FOR BLOCK READS
	##########################################
	$average_day_block_reads = $total_block_reads / $total_days_tracked;
	$average_hour_block_reads = $average_day_block_reads / 24;
	$average_minute_block_reads = $average_hour_block_reads / 60;
	$average_second_block_reads = $average_minute_block_reads / 60;
	$average_day_block_reads = number_format($average_day_block_reads, 2, '.', ',');
	$average_hour_block_reads = number_format($average_hour_block_reads, 2, '.', ',');
	$average_minute_block_reads = number_format($average_minute_block_reads, 2, '.', ',');
	$average_second_block_reads = number_format($average_second_block_reads, 4, '.', ',');
	$total_block_reads = number_format($total_block_reads);
	$highest_block_reads = number_format($highest_block_reads);
	
	##########################################
	#FORMAT GENERAL STATISTICS FOR BUFFER HITS
	##########################################
	$average_day_buffer_hits = $total_buffer_hits / $total_days_tracked;
	$average_hour_buffer_hits = $average_day_buffer_hits / 24;
	$average_minute_buffer_hits = $average_hour_buffer_hits / 60;
	$average_second_buffer_hits = $average_minute_buffer_hits / 60;
	$average_day_buffer_hits = number_format($average_day_buffer_hits, 2, '.', ',');
	$average_hour_buffer_hits = number_format($average_hour_buffer_hits, 2, '.', ',');
	$average_minute_buffer_hits = number_format($average_minute_buffer_hits, 2, '.', ',');
	$average_second_buffer_hits = number_format($average_second_buffer_hits, 4, '.', ',');
	$total_buffer_hits = number_format($total_buffer_hits);
	$highest_buffer_hits = number_format($highest_buffer_hits);
	
	######################################
	#FORMAT GENERAL STATISTICS FOR INSERTS
	######################################
	$average_day_inserts = $total_inserts / $total_days_tracked;
	$average_hour_inserts = $average_day_inserts / 24;
	$average_minute_inserts = $average_hour_inserts / 60;
	$average_second_inserts = $average_minute_inserts / 60;
	$average_day_inserts = number_format($average_day_inserts, 2, '.', ',');
	$average_hour_inserts = number_format($average_hour_inserts, 2, '.', ',');
	$average_minute_inserts = number_format($average_minute_inserts, 2, '.', ',');
	$average_second_inserts = number_format($average_second_inserts, 4, '.', ',');
	$total_inserts = number_format($total_inserts);
	$highest_inserts = number_format($highest_inserts);
	
	######################################
	#FORMAT GENERAL STATISTICS FOR UPDATES
	######################################
	$average_day_updates = $total_updates / $total_days_tracked;
	$average_hour_updates = $average_day_updates / 24;
	$average_minute_updates = $average_hour_updates / 60;
	$average_second_updates = $average_minute_updates / 60;
	$average_day_updates = number_format($average_day_updates, 2, '.', ',');
	$average_hour_updates = number_format($average_hour_updates, 2, '.', ',');
	$average_minute_updates = number_format($average_minute_updates, 2, '.', ',');
	$average_second_updates = number_format($average_second_updates, 4, '.', ',');
	$total_updates = number_format($total_updates);
	$highest_updates = number_format($highest_updates);
	
	######################################
	#FORMAT GENERAL STATISTICS FOR DELETES
	######################################
	$average_day_deletes = $total_deletes / $total_days_tracked;
	$average_hour_deletes = $average_day_deletes / 24;
	$average_minute_deletes = $average_hour_deletes / 60;
	$average_second_deletes = $average_minute_deletes / 60;
	$average_day_deletes = number_format($average_day_deletes, 2, '.', ',');
	$average_hour_deletes = number_format($average_hour_deletes, 2, '.', ',');
	$average_minute_deletes = number_format($average_minute_deletes, 2, '.', ',');
	$average_second_deletes = number_format($average_second_deletes, 4, '.', ',');
	$total_deletes = number_format($total_deletes);
	$highest_deletes = number_format($highest_deletes);
	
	#############################################
	#FORMAT GENERAL STATISTICS FOR SEQUENCE SCANS
	#############################################
	$average_day_seq_scans = $total_seq_scans / $total_days_tracked;
	$average_hour_seq_scans = $average_day_seq_scans / 24;
	$average_minute_seq_scans = $average_hour_seq_scans / 60;
	$average_second_seq_scans = $average_minute_seq_scans / 60;
	$average_day_seq_scans = number_format($average_day_seq_scans, 2, '.', ',');
	$average_hour_seq_scans = number_format($average_hour_seq_scans, 2, '.', ',');
	$average_minute_seq_scans = number_format($average_minute_seq_scans, 2, '.', ',');
	$average_second_seq_scans = number_format($average_second_seq_scans, 4, '.', ',');
	$total_seq_scans = number_format($total_seq_scans);
	$highest_seq_scans = number_format($highest_seq_scans);
	
	##########################################
	#FORMAT GENERAL STATISTICS FOR INDEX SCANS
	##########################################
	$average_day_idx_scans = $total_idx_scans / $total_days_tracked;
	$average_hour_idx_scans = $average_day_idx_scans / 24;
	$average_minute_idx_scans = $average_hour_idx_scans / 60;
	$average_second_idx_scans = $average_minute_idx_scans / 60;
	$average_day_idx_scans = number_format($average_day_idx_scans, 2, '.', ',');
	$average_hour_idx_scans = number_format($average_hour_idx_scans, 2, '.', ',');
	$average_minute_idx_scans = number_format($average_minute_idx_scans, 2, '.', ',');
	$average_second_idx_scans = number_format($average_second_idx_scans, 4, '.', ',');
	$total_idx_scans = number_format($total_idx_scans);
	$highest_idx_scans = number_format($highest_idx_scans);

	############################################
	#FORMAT GENERAL STATISTICS FOR SEQUENCE ROWS
	############################################
	$average_day_seq_rows = $total_seq_rows / $total_days_tracked;
	$average_hour_seq_rows = $average_day_seq_rows / 24;
	$average_minute_seq_rows = $average_hour_seq_rows / 60;
	$average_second_seq_rows = $average_minute_seq_rows / 60;
	$average_day_seq_rows = number_format($average_day_seq_rows, 2, '.', ',');
	$average_hour_seq_rows = number_format($average_hour_seq_rows, 2, '.', ',');
	$average_minute_seq_rows = number_format($average_minute_seq_rows, 2, '.', ',');
	$average_second_seq_rows = number_format($average_second_seq_rows, 4, '.', ',');
	$total_seq_rows = number_format($total_seq_rows);
	$highest_seq_rows = number_format($highest_seq_rows);
	
	#########################################
	#FORMAT GENERAL STATISTICS FOR INDEX ROWS
	#########################################
	$average_day_idx_rows = $total_idx_rows / $total_days_tracked;
	$average_hour_idx_rows = $average_day_idx_rows / 24;
	$average_minute_idx_rows = $average_hour_idx_rows / 60;
	$average_second_idx_rows = $average_minute_idx_rows / 60;
	$average_day_idx_rows = number_format($average_day_idx_rows, 2, '.', ',');
	$average_hour_idx_rows = number_format($average_hour_idx_rows, 2, '.', ',');
	$average_minute_idx_rows = number_format($average_minute_idx_rows, 2, '.', ',');
	$average_second_idx_rows = number_format($average_second_idx_rows, 4, '.', ',');
	$total_idx_rows = number_format($total_idx_rows);
	$highest_idx_rows = number_format($highest_idx_rows);

############################
#FINISH UP THE IUD AREA HTML
############################
$ajax_return .= <<<AJR
</td>
</tr>
</table>
AJR;

	###################################################################
	#FIRST PARSE OUR PEAK DATES SO WE CAN LINK TO INDI DB / DAILY STATS
	###################################################################
	$peak_month_commits = substr($highest_commits_date,0,2);
	$peak_day_commits = substr($highest_commits_date,3,2);
	$peak_year_commits = substr($highest_commits_date,6,4);
	
	$peak_month_rollbacks = substr($highest_rollbacks_date,0,2);
	$peak_day_rollbacks = substr($highest_rollbacks_date,3,2);
	$peak_year_rollbacks = substr($highest_rollbacks_date,6,4);
	
	$peak_month_block_reads = substr($highest_block_reads_date,0,2);
	$peak_day_block_reads = substr($highest_block_reads_date,3,2);
	$peak_year_block_reads = substr($highest_block_reads_date,6,4);
	
	$peak_month_buffer_hits = substr($highest_buffer_hits_date,0,2);
	$peak_day_buffer_hits = substr($highest_buffer_hits_date,3,2);
	$peak_year_buffer_hits = substr($highest_buffer_hits_date,6,4);
	
	$peak_month_inserts = substr($highest_inserts_date,0,2);
	$peak_day_inserts = substr($highest_inserts_date,3,2);
	$peak_year_inserts = substr($highest_inserts_date,6,4);
	
	$peak_month_updates = substr($highest_updates_date,0,2);
	$peak_day_updates = substr($highest_updates_date,3,2);
	$peak_year_updates = substr($highest_updates_date,6,4);
	
	$peak_month_deletes = substr($highest_deletes_date,0,2);
	$peak_day_deletes = substr($highest_deletes_date,3,2);
	$peak_year_deletes = substr($highest_deletes_date,6,4);
	
	$peak_month_seq_scans = substr($highest_seq_scans_date,0,2);
	$peak_day_seq_scans = substr($highest_seq_scans_date,3,2);
	$peak_year_seq_scans = substr($highest_seq_scans_date,6,4);
	
	$peak_month_idx_scans = substr($highest_idx_scans_date,0,2);
	$peak_day_idx_scans = substr($highest_idx_scans_date,3,2);
	$peak_year_idx_scans = substr($highest_idx_scans_date,6,4);
	
	$peak_month_seq_rows = substr($highest_seq_rows_date,0,2);
	$peak_day_seq_rows = substr($highest_seq_rows_date,3,2);
	$peak_year_seq_rows = substr($highest_seq_rows_date,6,4);
	
	$peak_month_idx_rows = substr($highest_idx_rows_date,0,2);
	$peak_day_idx_rows = substr($highest_idx_rows_date,3,2);
	$peak_year_idx_rows = substr($highest_idx_rows_date,6,4);

###############################################
#START UP OUR BOTTOM AREA OF GENERAL STATISTICS
###############################################
$ajax_return .= <<<AJR
<div class="stat_wrapper" style="padding:25px;">
<table class="ps_core_table body_secondary">
<tr>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Statistic</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Change/Total</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Peak Day</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Peak Date</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Avg/Day</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Avg/Hr</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Avg/Min</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Avg/Sec</td>
</tr>
AJR;

##################
#REPORT ON COMMITS
##################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Commits</td>
<td>$total_commits_change $total_commits</td>
<td>$highest_commits</td>
AJR;

if(!empty($highest_commits_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_commits','$peak_day_commits','$peak_year_commits')>$highest_commits_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day</td>
<td>$average_hour</td>
<td>$average_minute</td>
<td>$average_second</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

####################
#REPORT ON ROLLBACKS
####################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Rollbacks</td>
<td>$total_rollbacks_change $total_rollbacks</td>
<td>$highest_rollbacks</td>
AJR;

if(!empty($highest_rollbacks_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_rollbacks','$peak_day_rollbacks','$peak_year_rollbacks')>$highest_rollbacks_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_rollbacks</td>
<td>$average_hour_rollbacks</td>
<td>$average_minute_rollbacks</td>
<td>$average_second_rollbacks</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

######################
#REPORT ON BLOCK READS
######################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Block&nbsp;Reads</td>
<td>$total_block_reads_change $total_block_reads</td>
<td>$highest_block_reads</td>
AJR;

if(!empty($highest_block_reads_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_block_reads','$peak_day_block_reads','$peak_year_block_reads')>$highest_block_reads_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_block_reads</td>
<td>$average_hour_block_reads</td>
<td>$average_minute_block_reads</td>
<td>$average_second_block_reads</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

######################
#REPORT ON BUFFER HITS
######################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Buffer&nbsp;Hits</td>
<td>$total_buffer_hits_change $total_buffer_hits</td>
<td>$highest_buffer_hits</td>
AJR;

if(!empty($highest_buffer_hits_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_buffer_hits','$peak_day_buffer_hits','$peak_year_buffer_hits')>$highest_buffer_hits_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_buffer_hits</td>
<td>$average_hour_buffer_hits</td>
<td>$average_minute_buffer_hits</td>
<td>$average_second_buffer_hits</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

##################
#REPORT ON INSERTS
##################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Inserts</td>
<td>$total_inserts_change $total_inserts</td>
<td>$highest_inserts</td>
AJR;

if(!empty($highest_inserts_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_inserts','$peak_day_inserts','$peak_year_inserts')>$highest_inserts_date</a></span></td>"; 
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_inserts</td>
<td>$average_hour_inserts</td>
<td>$average_minute_inserts</td>
<td>$average_second_inserts</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

##################
#REPORT ON UPDATES
##################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Updates</td>
<td>$total_updates_change $total_updates</td>
<td>$highest_updates</td>
AJR;

if(!empty($highest_updates_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_updates','$peak_day_updates','$peak_year_updates')>$highest_updates_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_updates</td>
<td>$average_hour_updates</td>
<td>$average_minute_updates</td>
<td>$average_second_updates</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

##################
#REPORT ON DELETES
##################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Deletes</td>
<td>$total_deletes_change $total_deletes</td>
<td>$highest_deletes</td>
AJR;

if(!empty($highest_deletes_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_deletes','$peak_day_deletes','$peak_year_deletes')>$highest_deletes_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_deletes</td>
<td>$average_hour_deletes</td>
<td>$average_minute_deletes</td>
<td>$average_second_deletes</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

#########################
#REPORT ON SEQUENCE SCANS
#########################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">SEQ&nbsp;Scans</td>
<td>$total_seq_scans_change $total_seq_scans</td>
<td>$highest_seq_scans</td>
AJR;

if(!empty($highest_seq_scans_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_seq_scans','$peak_day_seq_scans','$peak_year_seq_scans')>$highest_seq_scans_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_seq_scans</td>
<td>$average_hour_seq_scans</td>
<td>$average_minute_seq_scans</td>
<td>$average_second_seq_scans</td>
</tr>
AJR;

########################
#REPORT ON SEQUENCE ROWS
########################
$ajax_return .= <<<AJR
<tr>
<td class="body_secondary" style="padding-right:10px;">Rows&nbsp;Used</td>
<td>$total_seq_rows_change $total_seq_rows</td>
<td>$highest_seq_rows</td>
AJR;

if(!empty($highest_seq_rows_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_seq_rows','$peak_day_seq_rows','$peak_year_seq_rows')>$highest_seq_rows_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_seq_rows</td>
<td>$average_hour_seq_rows</td>
<td>$average_minute_seq_rows</td>
<td>$average_second_seq_rows</td>
</tr>
<tr>
<td colspan="8" style="height:30px;"><hr size="1"/></td>
</tr>
AJR;

######################
#REPORT ON INDEX SCANS
######################
$ajax_return .= <<<AJR
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">IDX&nbsp;Scans</td>
<td>$total_idx_scans_change $total_idx_scans</td>
<td>$highest_idx_scans</td>
AJR;

if(!empty($highest_idx_scans_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_idx_scans','$peak_day_idx_scans','$peak_year_idx_scans')>$highest_idx_scans_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_idx_scans</td>
<td>$average_hour_idx_scans</td>
<td>$average_minute_idx_scans</td>
<td>$average_second_idx_scans</td>
</tr>
AJR;

#####################
#REPORT ON INDEX ROWS
#####################
$ajax_return .= <<<AJR
<tr>
<td class="body_secondary" style="padding-right:10px;">Rows&nbsp;Used</td>
<td>$total_idx_rows_change $total_idx_rows</td>
<td>$highest_idx_rows</td>
AJR;

if(!empty($highest_idx_rows_date)) $ajax_return .= "<td><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$peak_month_idx_rows','$peak_day_idx_rows','$peak_year_idx_rows')>$highest_idx_rows_date</a></span></td>";
else $ajax_return .= "<td>N/A</td>";

$ajax_return .= <<<AJR
<td>$average_day_idx_rows</td>
<td>$average_hour_idx_rows</td>
<td>$average_minute_idx_rows</td>
<td>$average_second_idx_rows</td>
</tr>
</table>
</div>
AJR;

/*
#############################
#GET TOTAL SCRIPT PEAK MEMORY
#############################
$script_mem = number_format(((memory_get_peak_usage(true)/1024)/1024),3) . "M";
$ajax_return .= <<<AJR
<div style="text-align:left; padding-top:10px;">Script Peak Memory: $script_mem</div>
AJR;
*/

	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_DAILY_DB_STATS
##########################################################################################################
##########################################################################################################
function retrieve_daily_db_stats($postgrestats_dir,$log_month,$log_day,$log_year,$graphics_src,$daily_chartting)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($log_month) || empty($log_day) || empty($log_year))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}

	##################################################################
	#DOUBLE CHECK THE CHARTTING VAR AND MAKE SURE IT HAS A LEGIT VALUE
	##################################################################
	if($daily_chartting != "commits" && $daily_chartting != "rollbacks" && $daily_chartting != "block_reads" && $daily_chartting != "buffer_hits" && $daily_chartting != "inserts" && $daily_chartting != "updates" && $daily_chartting != "deletes" && $daily_chartting != "seq_scans" && $daily_chartting != "idx_scans")
	{
		###########################################
		#DEFAULT THE CHART TO SHOWING COMMIT VALUES
		###########################################
		$daily_chartting = "commits";
	}

	#############################################################
	#PUT THE LOG FILE NAME TOGETHER AND MAKE SURE THE FILE EXISTS
	#############################################################
	$log_file_name = $log_month;

	###########################
	#MAKE DOUBLE SURE IT EXISTS
	###########################
	if(!file_exists("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name"))
	{
		$stat_msg = "Zero stats have been recorded for this day.";
		echo $stat_msg;
		exit;
	}

	#########################
	#ATTEMPT TO OPEN THE FILE
	#########################
	if(!$open_log = fopen("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name", "r"))
	{
		$stat_msg = "Unable to open this log file.";
		echo $stat_msg;
		exit;
	}

	###########################
	#INITIATE SOME GENERAL VARS
	###########################
	$bar_width = 250;
	$bar_height = 20;
	$stats_available = false;

	##########################################################################
	#CREATE ARRAY WHICH WILL STORE DB STATISTICS - EACH FIELD WILL BE AN ARRAY
	##########################################################################
	$db_array = array();

	############################################################
	#GO THROUGH THE LOG - LINE BY LINE AND RETRIEVE ALL THE DATA
	############################################################
	while(!feof($open_log))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log);

		###################################
		#MAKE SURE WE HAVE A LINE WITH DATA
		###################################
		if(!empty($db_line_single))
		{
			#############################################################
			#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE
			#############################################################

			################################################################
			#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
			#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
			################################################################
			if(substr($db_line_single,0,2) == $log_month && substr($db_line_single,6,4) == $log_year)
			{
				#############
				#FLAG THE DAY
				#############
				$day_flag = substr($db_line_single,3,2);

				###################################################################################
				#CHECK AND SEE IF IT'S OUR SELECTED DAY - IF SO, COLLECT SOME STAT TOTALS FOR LATER
				###################################################################################
				if($day_flag == $log_day)
				{
					#######################################
					#YES - STATS ARE AVAILABLE FOR THIS DAY
					#######################################
					$stats_available = true;
					$log_line = explode(",",$db_line_single);				
					if(!empty($log_line[3])) { $commits = $log_line[3]; } else { $commits = 0; }
					if(!empty($log_line[4])) { $rollbacks = $log_line[4]; } else { $rollbacks = 0; }
					if(!empty($log_line[5])) { $block_reads = $log_line[5]; } else { $block_reads = 0; }
					if(!empty($log_line[6])) { $buffer_hits = $log_line[6]; } else { $buffer_hits = 0; }
					if(!empty($log_line[7])) { $inserts = $log_line[7]; } else { $inserts = 0; }
					if(!empty($log_line[8])) { $updates = $log_line[8]; } else { $updates = 0; }
					if(!empty($log_line[9])) { $deletes = $log_line[9]; } else { $deletes = 0; }
					if(!empty($log_line[10])) { $seq_scans = $log_line[10]; } else { $seq_scans = 0; }
					if(!empty($log_line[11])) { $idx_scans = $log_line[11]; } else { $idx_scans = 0; }
					if(!empty($log_line[12])) { $seq_rows = $log_line[12]; } else { $seq_rows = 0; }
					if(!empty($log_line[13])) { $idx_rows = $log_line[13]; } else { $idx_rows = 0; }
				}
			}

			#################################################################################
			#IF IT'S NOT A FIRST LINE - CHECK TO SEE IF THE FLAG DAY IS OUR SELECTED STAT DAY
			#################################################################################
			elseif($day_flag == $log_day)
			{
				##############################################
				#DO NOT PROCESS LINES WITH \N NEWLINES IN THEM
				##############################################
				if($db_line_single != "\n")
				{
					######################################
					#EXPLODE THE LINE AND GET OUR DB STATS
					######################################
					$log_line = explode(",",$db_line_single);

					###############################################################
					#SAMPLE LINE FROM LOG FILE
					#19504,db_name_psql,11602,1,1072,1013126,0,0,0,10638,0,680219,0
					###############################################################

					##############################################################################
					#CREATE AN ARRAY WITH THIS DBS STATS - IT WILL BE STORED IN THE DB_ARRAY ARRAY
					##############################################################################
					$db_solo_array = array();

					##########################
					#PUSH THE SOLO DB STATS IN
					##########################
					if(empty($log_line[0])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[0]; }		#DATABASE ID
					if(empty($log_line[1])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[1]; } 	#DATABASE NAME
					if(empty($log_line[2])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[2]; } 	#DATABASE COMMITS
					if(empty($log_line[3])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[3]; } 	#DATABASE ROLL BACKS
					if(empty($log_line[4])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[4]; } 	#DATABASE BLOCK READS
					if(empty($log_line[5])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[5]; } 	#DATABASE BUFFER HITS
					if(empty($log_line[6])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[6]; } 	#DATABASE INSERTS
					if(empty($log_line[7])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[7]; } 	#DATABASE UPDATES
					if(empty($log_line[8])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[8]; } 	#DATABASE DELETES
					if(empty($log_line[9])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[9]; } 	#SEQUENCE SCANS
					if(empty($log_line[10])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[10]; }	#INDEX SCANS
					if(empty($log_line[11])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[11]; } 	#SEQUENCE ROWS USED
					if(empty($log_line[12])) { $db_solo_array[] = 0; } else { $db_solo_array[] = $log_line[12]; } 	#INDEX ROWS USED

					###########################################
					#PUSH THIS NEW SOLO ARRAY INTO THE DB_ARRAY
					###########################################
					$db_array[] = $db_solo_array;
				}
			}
		}	
	}

	#############################
	#CLOSE UP THE FILE - ALL DONE
	#############################
	fclose($open_log);

	#######################################################
	#FIRST CHECK TO SEE IF STATS FOR THIS DAY ARE AVAILABLE
	#######################################################
	if($stats_available == false)
	{
		$stat_msg = "Zero stats have been recorded for this day.";
		echo $stat_msg;
		exit;
	}

	###############################################################
	#RE-SORT OUR MULTI-DIMENSIONAL ARRAY BY THE CHARTTING VAR VALUE
	###############################################################
	foreach ($db_array as $key => $row)
	{
		$sort_db_id[$key]  = $row[0];
		$sort_db[$key]  = $row[1];
		$sort_commits[$key]  = $row[2];
		$sort_rollbacks[$key]  = $row[3];
		$sort_block_reads[$key]  = $row[4];
		$sort_buffer_hits[$key]  = $row[5];
		$sort_inserts[$key]  = $row[6];
		$sort_updates[$key]  = $row[7];
		$sort_deletes[$key]  = $row[8];
		$sort_seq_scans[$key]  = $row[9];
		$sort_idx_scans[$key]  = $row[10];
		$sort_seq_rows[$key]  = $row[11];
		$sort_idx_rows[$key]  = $row[12];
	}

	########
	#SORT IT
	########
	array_multisort(${'sort_' . $daily_chartting}, SORT_DESC, $db_array);

	##########################################################################
	#CHECK AND SEE IF STATS ARE AVAILABLE FOR THE PREVIOUS CYCLE
	#IF SO, WE'LL PROVIDE UP/DOWN ARROWS TO SHOW A QUICK A/B OF THE TWO CYCLES
	##########################################################################

	#################################################################################
	#FIRST GET THE PREVIOUS DAY SO WE CAN PUT TOGETHER THE PRECEDING LOG FILE NAME
	#EG: IF IT'S THE 1ST, PREVIOUS DAILY STATS WILL BE IN THE PREVIOUS LOG
	#################################################################################
	$prev_date_temp = mktime(0, 0, 0, $log_month, $log_day-1, $log_year);

	#########################
	#RETRIEVE OUR DATE VALUES
	#########################
	$prev_log_month = date("m", $prev_date_temp);
	$prev_log_day = date("d", $prev_date_temp);
	$prev_log_year = date("Y", $prev_date_temp);

	####################################
	#PUT THE PREV LOG FILE NAME TOGETHER
	####################################
	$prev_log_file_name = $prev_log_month;

	######################################
	#DEFAULT PREV_STATS_AVAILABLE TO FALSE
	######################################
	$prev_stats_available = false;

	###########################
	#INITIATE SOME GENERAL VARS
	###########################
	$stat_a1_src = "$graphics_src/stat_arrow_up.png";
	$stat_a2_src = "$graphics_src/stat_arrow_down.png";
	$stat_a3_src = "$graphics_src/stat_arrow_equal.png";
	$stat_a4_src = "$graphics_src/stat_arrow_no_data.png";

	#############################################
	#NOW LETS SEE IF THE PREVIOUS LOG FILE EXISTS
	#############################################
	if(file_exists("$postgrestats_dir/postgrestats/data/$prev_log_year/$prev_log_file_name"))
	{	
		#########################
		#ATTEMPT TO OPEN THE FILE
		#########################
		if(!$open_prev_log = fopen("$postgrestats_dir/postgrestats/data/$prev_log_year/$prev_log_file_name", "r"))
		{
			$stat_msg = "Unable to open previous stats log file.";
			echo $stat_msg;
			exit;
		}

		############################################################################################
		#CREATE ARRAY WHICH WILL STORE PREVIOUS STATS ON DB STATISTICS - EACH FIELD WILL BE AN ARRAY
		############################################################################################
		$prev_db_array = array();

		##########################################################
		#ALL IS GOOD, LET'S DIG IN AND GET OUR PREVIOUS STATISTICS
		##########################################################

		#################################
		#LET'S GO THROUGH IT LINE BY LINE
		#################################
		while(!feof($open_prev_log))
		{
			#############
			#GET ONE LINE
			#############
			$db_line_single = fgets($open_prev_log);

			###################################
			#MAKE SURE WE HAVE A LINE WITH DATA
			###################################
			if(!empty($db_line_single))
			{
				#############################################################
				#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE
				#############################################################

				################################################################
				#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
				#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
				################################################################
				if(substr($db_line_single,0,2) == $prev_log_month && substr($db_line_single,6,4) == $prev_log_year)
				{
					#############
					#FLAG THE DAY
					#############
					$day_flag = substr($db_line_single,3,2);

					###################################################################################
					#CHECK AND SEE IF IT'S OUR SELECTED DAY - IF SO, COLLECT SOME STAT TOTALS FOR LATER
					###################################################################################
					if($day_flag == $prev_log_day)
					{
						#######################################
						#YES - STATS ARE AVAILABLE FOR THIS DAY
						#######################################
						$prev_stats_available = true;
					}
				}

				#################################################################################
				#IF IT'S NOT A FIRST LINE - CHECK TO SEE IF THE FLAG DAY IS OUR SELECTED STAT DAY
				#################################################################################
				elseif($day_flag == $prev_log_day)
				{
					##############################################
					#DO NOT PROCESS LINES WITH \N NEWLINES IN THEM
					##############################################
					if($db_line_single != "\n")
					{
						######################################
						#EXPLODE THE LINE AND GET OUR DB STATS
						######################################
						$log_line = explode(",",$db_line_single);

						###############################################################
						#SAMPLE LINE FROM LOG FILE
						#19504,db_name_psql,11602,1,1072,1013126,0,0,0,10638,0,680219,0
						###############################################################

						##############################################################################
						#CREATE AN ARRAY WITH THIS DBS STATS - IT WILL BE STORED IN THE DB_ARRAY ARRAY
						##############################################################################
						$prev_db_solo_array = array();

						##########################
						#PUSH THE SOLO DB STATS IN
						##########################
						if(empty($log_line[0])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[0]; }		#DATABASE ID
						if(empty($log_line[1])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[1]; }		#DATABASE NAME
						if(empty($log_line[2])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[2]; }		#DATABASE COMMITS
						if(empty($log_line[3])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[3]; }		#DATABASE ROLL BACKS
						if(empty($log_line[4])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[4]; }		#DATABASE BLOCK READS
						if(empty($log_line[5])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[5]; }		#DATABASE BUFFER HITS
						if(empty($log_line[6])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[6]; }		#DATABASE INSERTS
						if(empty($log_line[7])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[7]; }		#DATABASE UPDATES
						if(empty($log_line[8])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[8]; }		#DATABASE DELETES
						if(empty($log_line[9])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[9]; }		#SEQUENCE SCANS
						if(empty($log_line[10])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[10]; }		#INDEX SCANS
						if(empty($log_line[11])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[11]; }		#SEQUENCE ROWS USED
						if(empty($log_line[12])) { $prev_db_solo_array[] = 0; } else { $prev_db_solo_array[] = $log_line[12]; }		#INDEX ROWS USED

						###########################################
						#PUSH THIS NEW SOLO ARRAY INTO THE DB_ARRAY
						###########################################
						$prev_db_array[] = $prev_db_solo_array;
					}
				}
			}	
		}

		#############################
		#CLOSE UP THE FILE - ALL DONE
		#############################
		fclose($open_prev_log);
	}

	#####################################
	#CREATE FRIENDLY NAME FOR TARGET STAT
	#####################################
	switch($daily_chartting)
	{
		case "commits": $target_friendly = "Commits"; break;
		case "rollbacks": $target_friendly = "Rollbacks"; break;
		case "block_reads": $target_friendly = "Block&nbsp;Reads"; break;
		case "buffer_hits": $target_friendly = "Buffer&nbsp;Hits"; break;
		case "inserts": $target_friendly = "Inserts"; break;
		case "updates": $target_friendly = "Updates"; break;
		case "deletes": $target_friendly = "Deletes"; break;
		case "seq_scans": $target_friendly = "SEQ&nbsp;Scans"; break;
		case "idx_scans": $target_friendly = "IDX&nbsp;Scans";
	}

########################################
#START PUTTING OUR INITIAL HTML TOGETHER
########################################
$ajax_return = <<<AJR
<div class="stat_wrapper" style="margin-top:10px;">
<table class="ps_core_table">
<tr>
<td class="body_primary" style="font-weight:bold;">Database</td>
<td class="body_primary" style="font-weight:bold;">$target_friendly</td>
<td class="body_primary" style="font-weight:bold;">Activity</td>
<td class="body_primary" style="font-weight:bold;">Extended&nbsp;Statistics</td>
</tr>
AJR;

	###########################
	#INITIATE SOME DEFAULT VARS
	###########################
	$bar_alt_color = "#FFFFFF";
	$bar_border_color = "#444444";
	$peak_stat = 0;
	$i_peak_flag = 0;
	$target_stat = 0;

	#################################
	#GET A COUNT ON OUR MAIN DB ARRAY
	#################################
	$db_count = count($db_array);

	################################
	#FLAG OUR TARGET ITERATION VALUE
	################################
	switch($daily_chartting)
	{
		case "commits": $i_peak_flag = 2; break;
		case "rollbacks": $i_peak_flag = 3; break;
		case "block_reads": $i_peak_flag = 4; break;
		case "buffer_hits": $i_peak_flag = 5; break;
		case "inserts": $i_peak_flag = 6; break;
		case "updates": $i_peak_flag = 7; break;
		case "deletes": $i_peak_flag = 8; break;
		case "seq_scans": $i_peak_flag = 9; break;
		case "idx_scans": $i_peak_flag = 10;
	}

	##########################################
	#WE NEED TO FIRST FIND THE PEAK STAT LEVEL
	##########################################
	for($i=0; $i<$db_count; $i++)
	{
		$target_stat = $db_array[$i][$i_peak_flag];
		if($target_stat > $peak_stat) $peak_stat = $target_stat;
	}

	##########################################################
	#NOW ITERATE THROUGH OUR SOLO DB ARRAYS AND GET OUR VALUES
	##########################################################
	for($i=0; $i<$db_count; $i++)
	{
		##########################
		#SET ALL PREV VARS TO ZERO
		##########################
		$prev_db_commits = 0;
		$prev_db_rollbacks = 0;
		$prev_db_block_reads = 0;
		$prev_db_buffer_hits = 0;
		$prev_db_inserts = 0;
		$prev_db_updates = 0;
		$prev_db_deletes = 0;
		$prev_db_seq_scans = 0;
		$prev_db_idx_scans = 0;
		$prev_db_seq_rows = 0;
		$prev_db_idx_rows = 0;

		############################
		#STATS FOR THE CURRENT CYCLE
		############################
		$db_id = $db_array[$i][0];
		$db_name = $db_array[$i][1];
		$db_commits = $db_array[$i][2];
		$db_rollbacks = $db_array[$i][3];
		$db_block_reads = $db_array[$i][4];
		$db_buffer_hits = $db_array[$i][5];
		$db_inserts = $db_array[$i][6];
		$db_updates = $db_array[$i][7];
		$db_deletes = $db_array[$i][8];
		$db_seq_scans = $db_array[$i][9];
		$db_idx_scans = $db_array[$i][10];
		$db_seq_rows = $db_array[$i][11];
		$db_idx_rows = $db_array[$i][12];

		###########################
		#FORCE DATA TYPE TO INTEGER
		###########################
		$db_commits+=0;
		$db_rollbacks+=0;
		$db_block_reads+=0;
		$db_buffer_hits+=0;
		$db_inserts+=0;
		$db_updates+=0;
		$db_deletes+=0;
		$db_seq_scans+=0;
		$db_idx_scans+=0;
		$db_seq_rows+=0;
		$db_idx_rows+=0;

		###########################################
		#STATS FOR THE PREVIOUS CYCLE IF APPLICABLE
		###########################################
		if($prev_stats_available == true)
		{
			##############################################
			#LOOP THROUGH THE PREV ARRAYS AND FIND THIS DB
			#TO OBTAIN PREVIOUS STATS
			##############################################
			for($ip=0; $ip<count($prev_db_array); $ip++)
			{
				########################
				#CHECK FOR DB NAME MATCH
				########################
				if($db_name == $prev_db_array[$ip][1])
				{
					##################
					#GRAB WHAT WE NEED
					##################
					$prev_db_id = $prev_db_array[$ip][0];
					$prev_db_name = $prev_db_array[$ip][1];
					$prev_db_commits = $prev_db_array[$ip][2];
					$prev_db_rollbacks = $prev_db_array[$ip][3];
					$prev_db_block_reads = $prev_db_array[$ip][4];
					$prev_db_buffer_hits = $prev_db_array[$ip][5];
					$prev_db_inserts = $prev_db_array[$ip][6];
					$prev_db_updates = $prev_db_array[$ip][7];
					$prev_db_deletes = $prev_db_array[$ip][8];
					$prev_db_seq_scans = $prev_db_array[$ip][9];
					$prev_db_idx_scans = $prev_db_array[$ip][10];
					$prev_db_seq_rows = $prev_db_array[$ip][11];
					$prev_db_idx_rows = $prev_db_array[$ip][12];

					###########################
					#FORCE DATA TYPE TO INTEGER
					###########################
					$prev_db_commits+=0;
					$prev_db_rollbacks+=0;
					$prev_db_block_reads+=0;
					$prev_db_buffer_hits+=0;
					$prev_db_inserts+=0;
					$prev_db_updates+=0;
					$prev_db_deletes+=0;
					$prev_db_seq_scans+=0;
					$prev_db_idx_scans+=0;
					$prev_db_seq_rows+=0;
					$prev_db_idx_rows+=0;
				}
			}
		}

		##########################################################################
		#HANDLE THE PEAK ORANGE GRAPHIC AND SHOW IT OR THE STANDARD COLOR DATA BAR
		##########################################################################
		if(${'db_' . $daily_chartting} == $peak_stat) $bar_color = "#FF6633";
		else $bar_color = "#7B7B7B";

		#######################################
		#CREATE THE PERCENT VARS AND BAR VALUES
		#######################################
		if(!empty(${$daily_chartting})) { $stat_percent = (${'db_' . $daily_chartting} / ${$daily_chartting}); } else { $stat_percent = 0; }
		$stat_percent = (number_format($stat_percent, 4, '.', '') * 100);
		if(!empty($peak_stat)) { $commit_bar_percent = (${'db_' . $daily_chartting} / $peak_stat); } else { $commit_bar_percent = 0; }
		$bar_width_percent = $commit_bar_percent;
		$bar_width_final = $bar_width * $bar_width_percent;
		$bar_width_final = number_format($bar_width_final, 0, '.', '');

		############################
		#FORCE BAR LENGTH - FOR SAFE
		############################
		if($bar_width_final > $bar_width) $bar_width_final = $bar_width;

		#####################################
		#GET THE ALT (LIGHT GREY) IMAGE WIDTH
		#####################################
		$bar_width_final_alt = $bar_width - $bar_width_final;

		###################################################################################################
		#CHECK TO SEE IF WE HAVE STATS FOR PREVIOUS CYCLE
		#IF SO, GO THROUGH EACH STAT AND CREATE THE IMAGE SOURCE CODE WHICH WILL DISPLAY THE +/-/= GRAPHICS
		###################################################################################################
		$stat_type_array = array('commits','rollbacks','block_reads','buffer_hits','inserts','updates','deletes','seq_scans','seq_rows','idx_scans','idx_rows');

		####################
		#START OUR ITERATION
		####################
		foreach($stat_type_array as &$stat)
		{
			${'indi_compare_' . $stat} = "";
	
			if($prev_stats_available == true)
			{	
				###########################
				#CHECK FOR STATS INCREASING
				###########################
				if(${'db_' . $stat} > ${'prev_db_' . $stat}) ${'indi_compare_' . $stat} = "<img src=\"$stat_a1_src\" width=\"10\" height=\"8\"/>";
				
				###########################
				#CHECK FOR STATS DECREASING
				###########################
				elseif(${'prev_db_' . $stat} > ${'db_' . $stat}) ${'indi_compare_' . $stat} = "<img src=\"$stat_a2_src\" width=\"10\" height=\"8\"/>";
				
				#################################
				#CHECK FOR STATS EXACTLY THE SAME
				#################################
				elseif(${'prev_db_' . $stat} == ${'db_' . $stat}) ${'indi_compare_' . $stat} = "<img src=\"$stat_a3_src\" width=\"10\" height=\"8\"/>";
			}
			else
			{
				################################
				#FORCE THE NO-DATA ICON FOR SAFE
				################################
				${'indi_compare_' . $stat} = "<img src=\"$stat_a4_src\" width=\"10\" height=\"8\"/>";
			}

			########################################
			#FORMAT THE VAR WHILE WE'RE IN THIS LOOP
			########################################
			${'db_' . $stat} = number_format(${'db_' . $stat});
		}

		###################################
		#CREATE A TOOLTIP FOR THE NAME/LINK
		###################################
		$tooltip = "Database ID: $db_id<br>";	
		$tooltip .= "Click&nbsp;for&nbsp;Monthly&nbsp;Stats<br>";

$ajax_return .= <<<AJR
<tr>
<td style="font-weight:bold;"><span onMouseOver="tooltip.show('$tooltip')" onMouseOut='tooltip.hide()'><a href="javascript:retrieve_indi_db_stats('$db_name','$log_month','$log_day','$log_year')">$db_name</a></span></td>
<td class="body_secondary">${'indi_compare_' . $daily_chartting} ${'db_' . $daily_chartting}</td>
<td>
AJR;

		###########################
		#PREP FINAL CELL DIMENSIONS
		###########################
		$bar_width_px = $bar_width . "px";
		$bar_height_px = $bar_height . "px";

		#######################################################
		#CHECK THE LENGTH VALUES AND PUT THE BARS TOGETHER
		#START UP THE TABLEWORK WHICH WILL HOUSE THE BAR ITSELF
		#######################################################
		$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$bar_width_px; empty-cells:show;\"><tr>";

		############################
		#DETERMINE BORDER STYLES NOW
		############################
		$bar_border = "border:1px solid $bar_border_color;";

		if($bar_width_final <= 0) $bar_alt_border = "border:1px solid $bar_border_color;";
		else $bar_alt_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";

		#####################################################################################
		#HANDLE THE ALT COLOR - WILL EITHER BE THE ENTIRE BAR (NO DATA) OR THE TOP OF THE BAR
		#####################################################################################
		if($bar_width_final > 0)
		{
			$tooltip = "$stat_percent%";
			$bar_width_final_px = $bar_width_final . "px";
			$ajax_return .= "<td style=\"width:$bar_width_final_px; height:$bar_height_px; background-color:$bar_color; $bar_border\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'></td>";
		}

		################################################
		#HANDLE THE BOTTOM OF THE BAR WHERE WE HAVE DATA
		################################################
		if($bar_width_final_alt > 0)
		{
			if($stat_percent > 0) { $tooltip = "$stat_percent%"; } else { $tooltip = "No Data"; }
			$bar_width_final_alt_px = $bar_width_final_alt . "px";
			$ajax_return .= "<td style=\"width:$bar_width_final_alt_px; height:$bar_height_px; background-color:$bar_alt_color; $bar_alt_border\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'></td>";
		}

		##############
		#COMPLETE CELL
		##############
		$ajax_return .= "</tr></table>";

		#####################################################################
		#CREATE THE DYNAMIC DIV NAME TOGETHER FOR EACH DBS INDI STATS SECTION
		#####################################################################
		$div_data = $db_name . "_data";

		##############################
		#CREATE THE EXT STATS ROW NAME
		##############################
		$ext_row = $db_name . "_ext";

#######################
#CONTINUE OUR TABLEWORK
#######################
$ajax_return .= <<<AJR
</td>
<td class="body_secondary"><a href="javascript:retrieve_indi_extended('$db_name')">Daily&nbsp;Extended</a></td>
</tr>
AJR;

##########################################
#CREATE OUR EXTENDED STATS ROW AND HIDE IT
##########################################
$ajax_return .= <<<AJR
<tr id="$ext_row" style="display:none;">
<td colspan="4" class="body_secondary">
AJR;

	###################################################################
	#INCLUDE OUR BLOCK READS, BUFFER HITS AND SEQ / IDX STATISTICS HERE
	###################################################################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"margin:10px 10px 20px 0; padding:25px;\">";
	$ajax_return .= "<div class=\"title_highlight\" style=\"margin-bottom:10px; font-weight:bold;\">Daily&nbsp;Statistics&nbsp;For:&nbsp;<span class=\"title_primary\" style=\"font-weight:bold;\">$db_name</span></div>";

############################################################
#CREATE SUB TABLE WHICH WILL HOUSE EXTENDED STATS THEMSELVES
############################################################
$ajax_return .= <<<AJR
<table class="stat_recap_table ps_core_table">
<tr>
<td class="body_secondary" style="width:75px;">Commits:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_commits&nbsp;&nbsp;$db_commits</td>
</tr>
<tr>
<td class="body_secondary">Rollbacks:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_rollbacks&nbsp;&nbsp;$db_rollbacks</td>
</tr>
<tr>
<td class="body_secondary">Block&nbsp;Reads:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_block_reads&nbsp;&nbsp;$db_block_reads</td>
</tr>
<tr>
<td class="body_secondary">Buffer&nbsp;Hits:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_buffer_hits&nbsp;&nbsp;$db_buffer_hits</td>
</tr>
<tr>
<td class="body_secondary">User&nbsp;Inserts:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_inserts&nbsp;&nbsp;$db_inserts</td>
</tr>
<tr>
<td class="body_secondary">User&nbsp;Updates:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_updates&nbsp;&nbsp;$db_updates</td>
</tr>
<tr>
<td class="body_secondary">User&nbsp;Deletes:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_deletes&nbsp;&nbsp;$db_deletes</td>
</tr>
<tr>
<td class="body_secondary">SEQ&nbsp;Scans:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_seq_scans&nbsp;&nbsp;$db_seq_scans</td>
</tr>
<tr>
<td class="body_secondary">SEQ&nbsp;Rows&nbsp;Used:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_seq_rows&nbsp;&nbsp;$db_seq_rows</td>
</tr>
<tr>
<td class="body_secondary">IDX&nbsp;Scans:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_idx_scans&nbsp;&nbsp;$db_idx_scans</td>
</tr>
<tr>
<td class="body_secondary">IDX&nbsp;Rows&nbsp;Used:</td>
<td class="body_primary" style="font-weight:bold;">$indi_compare_idx_rows&nbsp;&nbsp;$db_idx_rows</td>
</tr>
</table>
AJR;

	##############################
	#COMPLETE DIV, ROW AND MOVE ON
	##############################
	$ajax_return .= "</div></td></tr>";

########################################
#CREATE OUR DB STAT INDI ROW AND HIDE IT
########################################
$ajax_return .= <<<AJR
<tr id="$db_name" style="display:none;">
<td colspan="4" class="body_secondary"><div id="$div_data" style="display:none;"></div></td>
</tr>
AJR;
	}

	######################################
	#SOME GENERAL FORMATTING ON THE TOTALS
	######################################
	$stat_type_array = array('commits','rollbacks','block_reads','buffer_hits','inserts','updates','deletes','seq_scans','seq_rows','idx_scans','idx_rows');

	####################
	#START OUR ITERATION
	####################
	foreach($stat_type_array as &$stat)
	{
		${$stat} += 0;
		${$stat} = number_format(${$stat});
	}

#######################
#DISPLAY TOTALS ROW NOW
#######################
$ajax_return .= <<<AJR
<tr>
<td><strong class="body_primary">Total:</strong></td>
<td class="body_highlight" style="font-weight:bold;">${$daily_chartting}</td>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>
</table>
</div>
AJR;

/*
#############################
#GET TOTAL SCRIPT PEAK MEMORY
#############################
$script_mem = number_format(((memory_get_peak_usage(true)/1024)/1024),3) . "M";
$ajax_return .= <<<AJR
<div style="text-align:left; padding-top:10px;">Script Peak Memory: $script_mem</div>
AJR;
*/

	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_INDI_DB_STATS
##########################################################################################################
##########################################################################################################
function retrieve_indi_db_stats($postgrestats_dir,$db_name,$log_month,$log_day,$log_year,$graphics_src)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($db_name) || empty($log_month) || empty($log_day) || empty($log_year))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}

	#############################################################
	#PUT THE LOG FILE NAME TOGETHER AND MAKE SURE THE FILE EXISTS
	#############################################################
	$log_file_name = "$postgrestats_dir/postgrestats/data/$log_year/$log_month";
	if(!file_exists($log_file_name))
	{
		$stat_msg = "This log file does not exist.";
		echo $stat_msg;
		exit;
	}

	######################
	#ATTEMPT TO OPEN IT UP
	######################
	if(!$open_log = fopen($log_file_name, "r"))
	{	
		$stat_msg = "Error opening log file for read.";
		echo $stat_msg;
		exit;
	}

	##################################################################
	#GO THROUGH THE LOG FILE - SEARCH FOR THIS MONTH AND THIS DBS DATA
	##################################################################
	while(!feof($open_log))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log);

		##############################################
		#SEARCH FOR DATE LINE, MATCHING MONTH AND YEAR
		##############################################
		if(substr($db_line_single,0,2) == $log_month && substr($db_line_single,6,4) == $log_year)
		{
			$day_flag = substr($db_line_single,3,2);
		}

		#############################
		#SEARCH FOR DATABASE NAME NOW
		#############################
		if(strpos($db_line_single, "," . $db_name . ","))
		{
			####################################################################################################
			#EXPLODE LINE AND GET THE COMMITS, ROLLBACKS, BLOCK READS, BUFFER HITS, INSERTS, UPDATES AND DELETES
			#16395,db_name_psql,3116,1,1711,427178,0,0,0
			####################################################################################################
			$log_line = explode(",",$db_line_single);
			if(!empty($log_line[2])) { ${'indi_' . $db_name . '_' . $day_flag} = $log_line[2]; } else { ${'indi_' . $db_name . '_' . $day_flag} = 0; }
			if(!empty($log_line[3])) { ${'indi_' . $db_name . '_rol_' . $day_flag} = $log_line[3]; } else { ${'indi_' . $db_name . '_rol_' . $day_flag} = 0; }
			if(!empty($log_line[4])) { ${'indi_' . $db_name . '_blk_' . $day_flag} = $log_line[4]; } else { ${'indi_' . $db_name . '_blk_' . $day_flag} = 0; }
			if(!empty($log_line[5])) { ${'indi_' . $db_name . '_buf_' . $day_flag} = $log_line[5]; } else { ${'indi_' . $db_name . '_buf_' . $day_flag} = 0; }
			if(!empty($log_line[6])) { ${'indi_' . $db_name . '_ins_' . $day_flag} = $log_line[6]; } else { ${'indi_' . $db_name . '_ins_' . $day_flag} = 0; }
			if(!empty($log_line[7])) { ${'indi_' . $db_name . '_upd_' . $day_flag} = $log_line[7]; } else { ${'indi_' . $db_name . '_upd_' . $day_flag} = 0; }
			if(!empty($log_line[8])) { ${'indi_' . $db_name . '_del_' . $day_flag} = $log_line[8]; } else { ${'indi_' . $db_name . '_del_' . $day_flag} = 0; }
			if(!empty($log_line[9])) { ${'indi_' . $db_name . '_seq_scan_' . $day_flag} = $log_line[9]; } else { ${'indi_' . $db_name . '_seq_scan_' . $day_flag} = 0; }
			if(!empty($log_line[10])) { ${'indi_' . $db_name . '_idx_scan_' . $day_flag} = $log_line[10]; } else { ${'indi_' . $db_name . '_idx_scan_' . $day_flag} = 0; }
			if(!empty($log_line[11])) { ${'indi_' . $db_name . '_seq_row_' . $day_flag} = $log_line[11]; } else { ${'indi_' . $db_name . '_seq_row_' . $day_flag} = 0; }
			if(!empty($log_line[12])) { ${'indi_' . $db_name . '_idx_row_' . $day_flag} = $log_line[12]; } else { ${'indi_' . $db_name . '_idx_row_' . $day_flag} = 0; }
		}
	}

	#####################
	#ALL DONE CLOSE IT UP
	#####################
	fclose($open_log);

	#########################
	#SET UP SOME GENERAL VARS
	#########################
	$bar_height_vert = 150;
	$bar_width = 20;

	#####################
	#BAR CHART COLOR VARS
	#####################
	$bar_vert_alt_color = "#FFFFFF";
	$bar_vert_peak_color = "#FF6633";
	$bar_vert_color = "#7B7B7B";
	$bar_border_color = "#444444";

	############################
	#VARS TO HOUSE PEAK DAY DATE
	############################
	$peak_db_day_date = "";

	###############
	#IUD BAR COLORS
	###############
	$iud_insert_color = "#7B7B7B";
	$iud_update_color = "#C8C8C8";
	$iud_delete_color = "#F1F1F1";
	$iud_nodata_color = "#000000";

	######################################
	#VARS TO HOUSE THE TOOLTIP STAT VALUES
	######################################
	$db_stat = 0;
	$tt_commits = 0;
	$tt_rollbacks = 0;
	$tt_block_reads = 0;
	$tt_buffer_hits = 0;
	$tt_inserts = 0;
	$tt_updates = 0;
	$tt_deletes = 0;
	$tt_seq_scans = 0;
	$tt_idx_scans = 0;

###########################################################################
#START PUT TOGETHER OUR RETURNING HTML TABLE WORK FOR THE AJAX RETURN VALUE
###########################################################################
$ajax_return = <<<AJR
<div class="stat_sub_wrapper" style="margin:10px 0 20px 0;">
<div class="body_highlight" style="margin-bottom:15px; font-size:14px; font-weight:bold;">Monthly&nbsp;Statistics&nbsp;For:&nbsp;<span class="title_primary">$db_name</span></div>
<table class="stat_chart_table" style="width:auto;">
<tr>
AJR;

	########################################
	#INITIATE SOME GENERAT VARS FOR STATTING
	#FIND THE HIGHEST DAY FOR THIS DB
	########################################
	$peak_db_day = 0;
	$total_days_tracked = 0;
	$total_commits_indi = 0;
	$total_rollbacks_indi = 0;
	$total_block_reads_indi = 0;
	$total_buffer_hits_indi = 0;
	$total_inserts_indi = 0;
	$total_updates_indi = 0;
	$total_deletes_indi = 0;
	$total_seq_scans_indi = 0;
	$total_idx_scans_indi = 0;
	$total_seq_rows_indi = 0;
	$total_idx_rows_indi = 0;

	for($ix=1; $ix<32; $ix++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($ix) == 1) $ix = "0$ix";

		##################
		#CHECK ON PEAK DAY
		##################
		if(isset(${'indi_' . $db_name . '_' . $ix}))
		{
			if(${'indi_' . $db_name . '_' . $ix} > $peak_db_day)
			{
				$peak_db_day = ${'indi_' . $db_name . '_' . $ix};
				$peak_db_day_date = $ix;
			}

			###################################
			#TALLY TOTAL DAYS TRACKED FOR STATS
			###################################
			if(${'indi_' . $db_name . '_' . $ix} > 0) $total_days_tracked++;

			#######################
			#TALLY UP TOTAL COMMITS
			#######################
			if(!empty(${'indi_' . $db_name . '_' . $ix})) $total_commits_indi += ${'indi_' . $db_name . '_' . $ix};
		}

		##########################################################################################################
		#TALLY UP ROLLBACKS, BLOCK READS, BUFFER HITS, INSERTS, UPDATES AND DELETES IF VALUES EXIST FOR THESE DAYS
		##########################################################################################################
		if(!empty(${'indi_' . $db_name . '_rol_' . $ix})) $total_rollbacks_indi += ${'indi_' . $db_name . '_rol_' . $ix};
		if(!empty(${'indi_' . $db_name . '_blk_' . $ix})) $total_block_reads_indi += ${'indi_' . $db_name . '_blk_' . $ix};
		if(!empty(${'indi_' . $db_name . '_buf_' . $ix})) $total_buffer_hits_indi += ${'indi_' . $db_name . '_buf_' . $ix};
		if(!empty(${'indi_' . $db_name . '_ins_' . $ix})) $total_inserts_indi += ${'indi_' . $db_name . '_ins_' . $ix};
		if(!empty(${'indi_' . $db_name . '_upd_' . $ix})) $total_updates_indi += ${'indi_' . $db_name . '_upd_' . $ix};
		if(!empty(${'indi_' . $db_name . '_del_' . $ix})) $total_deletes_indi += ${'indi_' . $db_name . '_del_' . $ix};
		if(!empty(${'indi_' . $db_name . '_seq_scan_' . $ix})) $total_seq_scans_indi += ${'indi_' . $db_name . '_seq_scan_' . $ix};
		if(!empty(${'indi_' . $db_name . '_idx_scan_' . $ix})) $total_idx_scans_indi += ${'indi_' . $db_name . '_idx_scan_' . $ix};
		if(!empty(${'indi_' . $db_name . '_seq_row_' . $ix})) $total_seq_rows_indi += ${'indi_' . $db_name . '_seq_row_' . $ix};
		if(!empty(${'indi_' . $db_name . '_idx_row_' . $ix})) $total_idx_rows_indi += ${'indi_' . $db_name . '_idx_row_' . $ix};
	}
	
	###########################
	#PUT THE PEAK DATE TOGETHER
	###########################
	$peak_db_day_date = "($log_month-$peak_db_day_date-$log_year)";
	
	#############################################################################
	#PUT TOGETHER THE MONTH TABLEWORK - USING OUR BAR HEIGHT VARS AS GRAPH VALUES
	#############################################################################
	for($i=1; $i<32; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";
		
		##################################
		#FIRST PREP OUR COMMIT TOOLTIP VAR
		##################################
		if(isset(${'indi_' . $db_name . '_' . $i}))
		{
			$tt_commits = ${'indi_' . $db_name . '_' . $i};
			$db_stat = $tt_commits;
			if($tt_commits == 0 || empty($tt_commits)) $tt_commits = "0";
			else { $tt_commits += 0; $tt_commits = number_format($tt_commits); }
		}

		##############################
		#PREP OUR ROLLBACK TOOLTIP VAR
		##############################
		if(isset(${'indi_' . $db_name . '_rol_' . $i}))
		{
			$tt_rollbacks = ${'indi_' . $db_name . '_rol_' . $i};
			if($tt_rollbacks == 0 || empty($tt_rollbacks)) $tt_rollbacks = "0";
			else { $tt_rollbacks += 0; $tt_rollbacks = number_format($tt_rollbacks); }
		}
		#################################
		#PREP OUR BLOCK READS TOOLTIP VAR
		#################################
		if(isset(${'indi_' . $db_name . '_blk_' . $i}))
		{
			$tt_block_reads = ${'indi_' . $db_name . '_blk_' . $i};
			if($tt_block_reads == 0 || empty($tt_block_reads)) $tt_block_reads = "0";
			else { $tt_block_reads += 0; $tt_block_reads = number_format($tt_block_reads); }
		}
		
		#################################
		#PREP OUR BUFFER HITS TOOLTIP VAR
		#################################
		if(isset(${'indi_' . $db_name . '_buf_' . $i}))
		{
			$tt_buffer_hits = ${'indi_' . $db_name . '_buf_' . $i};
			if($tt_buffer_hits == 0 || empty($tt_buffer_hits)) $tt_buffer_hits = "0";
			else { $tt_buffer_hits += 0; $tt_buffer_hits = number_format($tt_buffer_hits); }
		}
		
		############################
		#PREP OUR INSERT TOOLTIP VAR
		############################
		if(isset(${'indi_' . $db_name . '_ins_' . $i}))
		{
			$tt_inserts = ${'indi_' . $db_name . '_ins_' . $i};
			if($tt_inserts == 0 || empty($tt_inserts)) $tt_inserts = "0";
			else { $tt_inserts += 0; $tt_inserts = number_format($tt_inserts); }
		}
		
		############################
		#PREP OUR UPDATE TOOLTIP VAR
		############################
		if(isset(${'indi_' . $db_name . '_upd_' . $i}))
		{
			$tt_updates = ${'indi_' . $db_name . '_upd_' . $i};
			if($tt_updates == 0 || empty($tt_updates)) $tt_updates = "0";
			else { $tt_updates += 0; $tt_updates = number_format($tt_updates); }
		}
		
		############################
		#PREP OUR DELETE TOOLTIP VAR
		############################
		if(isset(${'indi_' . $db_name . '_del_' . $i}))
		{
			$tt_deletes = ${'indi_' . $db_name . '_del_' . $i};
			if($tt_deletes == 0 || empty($tt_deletes)) $tt_deletes = "0";
			else { $tt_deletes += 0; $tt_deletes = number_format($tt_deletes); }
		}
		
		##############################
		#PREP OUR SEQ SCAN TOOLTIP VAR
		##############################
		if(isset(${'indi_' . $db_name . '_seq_scan_' . $i}))
		{
			$tt_seq_scans = ${'indi_' . $db_name . '_seq_scan_' . $i};
			if($tt_seq_scans == 0 || empty($tt_seq_scans)) $tt_seq_scans = "0";
			else { $tt_seq_scans += 0; $tt_seq_scans = number_format($tt_seq_scans); }
		}
		
		##############################
		#PREP OUR IDX SCAN TOOLTIP VAR
		##############################
		if(isset(${'indi_' . $db_name . '_idx_scan_' . $i}))
		{
			$tt_idx_scans = ${'indi_' . $db_name . '_idx_scan_' . $i};
			if($tt_idx_scans == 0 || empty($tt_idx_scans)) $tt_idx_scans = "0";
			else { $tt_idx_scans += 0; $tt_idx_scans = number_format($tt_idx_scans); }
		}

		################
		#PUT IT TOGETHER
		################
		$tooltip  = "<div style=line-height:17px;>";
		$tooltip .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits . "</strong></span><br>";
		$tooltip .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks . "</strong><br>";
		$tooltip .= "Block Reads:&nbsp;<strong>" . $tt_block_reads . "</strong><br>";
		$tooltip .= "Buffer Hits:&nbsp;<strong>" . $tt_buffer_hits . "</strong><br>";
		$tooltip .= "Inserts:&nbsp;<strong>" . $tt_inserts . "</strong><br>";
		$tooltip .= "Updates:&nbsp;<strong>" . $tt_updates . "</strong><br>";
		$tooltip .= "Deletes:&nbsp;<strong>" . $tt_deletes . "</strong><br>";
		$tooltip .= "Sequence Scans:&nbsp;<strong>" . $tt_seq_scans . "</strong><br>";
		$tooltip .= "Index Scans:&nbsp;<strong>" . $tt_idx_scans . "</strong>";
		$tooltip .= "</div>";

		##############################################################################
		#CALUCLATE BAR HEIGHT IF STATS EXIST FOR THIS DAY - OTHERWISE, SET TO DEFAULTS
		##############################################################################
		if(isset(${'indi_' . $db_name . '_' . $i}))
		{	
			#######################
			#GET THE COMMIT PERCENT
			#######################
			if(!empty($peak_db_day)) { $commit_percent = ($db_stat / $peak_db_day); } else { $commit_percent = 0; }
			$bar_height_percent = $commit_percent;
			$commit_percent = (number_format($commit_percent, 4, '.', '') * 100);

			#####################
			#CALCULATE BAR HEIGHT
			#####################
			$bar_height_final = $bar_height_vert * $bar_height_percent;
			$bar_height = number_format($bar_height_final, 0, '.', '');

			############################
			#FORCE BAR HEIGHT - FOR SAFE
			############################
			if($bar_height > $bar_height_vert) $bar_height = $bar_height_vert;

			#########################################################################################
			#CREATE THE ALT BAR (LIGHT GREY) WHICH WILL BE DISPLAYED ABOVE THE DARK GREY VERTICAL BAR
			#########################################################################################
			$bar_height_alt = $bar_height_vert - $bar_height;

			##########################
			#HANDLE THE PEAK IMAGE SRC
			##########################
			if($peak_db_day == $db_stat) $bar_vert_color_final = $bar_vert_peak_color;
			else $bar_vert_color_final = $bar_vert_color;
		}
		else
		{
			$bar_height = 0;
			$bar_height_alt = $bar_height_vert;
		}

		##################################
		#PUT SOME MORE TABLE WORK TOGETHER
		##################################
		$ajax_return .= "<td valign=\"bottom\" style=\"text-align:center; padding-right:5px;\">";

		###########################
		#PREP FINAL CELL DIMENSIONS
		###########################
		$bar_width_px = $bar_width . "px";
		$bar_height_px = $bar_height . "px";

		#######################################################
		#START UP THE TABLEWORK WHICH WILL HOUSE THE BAR ITSELF
		#######################################################
		$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$bar_width_px; empty-cells:show;\">";

		############################
		#DETERMINE BORDER STYLES NOW
		############################
		$bar_border = "border:1px solid $bar_border_color;";
		if($bar_height <= 0) $bar_alt_border = "border:1px solid $bar_border_color;";
		else $bar_alt_border = "border-color: $bar_border_color; border-width: 1px 1px 0px 1px; border-style: solid;";

		#####################################################################################
		#HANDLE THE ALT COLOR - WILL EITHER BE THE ENTIRE BAR (NO DATA) OR THE TOP OF THE BAR
		#####################################################################################
		if($bar_height_alt > 0)
		{
			$bar_height_alt_px = $bar_height_alt . "px";
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_px; background-color:$bar_vert_alt_color; $bar_alt_border\"></td></tr>";
		}

		################################################
		#HANDLE THE BOTTOM OF THE BAR WHERE WE HAVE DATA
		################################################
		if($bar_height > 0)
		{
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_px; background-color:$bar_vert_color_final; $bar_border\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'></td></tr>";
		}

		###################
		#COMPLETE TABLEWORK
		###################
		$ajax_return .= "</table>";

		################################################
		#FINISHED WITH THE BAR TABLE - CONTINUE OUR HTML
		################################################
	}

############
#CONTINUE ON
############
$ajax_return .= <<<AJR
<tr>
AJR;

	############################################################
	#PUT TOGETHER THE MONTH TABLEWORK - SHOWING EACH DAY'S TITLE
	############################################################
	for($i=1; $i<32; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";

		##################
		#PREPARE A TOOLTIP
		##################
		$tooltip = "Retieve&nbsp;Stats";

		#########################################################################################################################
		#SHOW EITHER THE DAY OF THE MONTH WITH A LINK TO THE DAILY/INDI VIEW OR JUST THE DAY - DEPENDING ON WHETHER THERE IS DATA
		#########################################################################################################################
		if(isset(${'indi_' . $db_name . '_' . $i}))
		{
			###########
			#LINK IT UP
			###########
			$ajax_return .= "<td class=\"body_secondary\" style=\"text-align:center; padding:5px 5px 0 0;\"><span onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$log_month','$i','$log_year')>$i</a></span></td>";
		}
		else
		{
			#####################
			#SHOW ONLY DAY NUMBER
			#####################
			$ajax_return .= "<td class=\"body_secondary\" style=\"text-align:center; padding:5px 5px 0 0;\">$i</td>";
		}
	}

##########################################
#PUT THE LAST OF OUR HTML TOGETHER
#AND START UP THE IUD STATS IMPLEMENTATION
##########################################
$ajax_return .= <<<AJR
</tr>
</table>
AJR;

###############################################################
#START UP THE TABLE WORK WHICH WILL HOUSE OUR IUD BAR AND STATS
###############################################################
$ajax_return .= <<<AJR
<div class="body_secondary" style="margin:25px 0 5px 0;">
<div class="body_highlight" style="margin-right:10px; float:left; font-weight:bold;">Insert&nbsp;/&nbsp;Update&nbsp;/&nbsp;Delete&nbsp;Quick&nbsp;View:&nbsp;</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_insert_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Inserts</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_update_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Updates</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_delete_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Deletes</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_nodata_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">No&nbsp;Data</div>
<div style="clear:both;"></div>
</div>
AJR;

	##############################################################
	#PUT TOGETHER OUR IUD STATS - FIRST INITIATE SOME GENERAL VARS
	##############################################################
	$iud_bar_width = 770;
	$iud_bar_height = 20;
	$total_iud = $total_inserts_indi + $total_updates_indi + $total_deletes_indi;
	
	##############################################
	#GET THE PERCENTAGES OF EACH AGAINST THE TOTAL
	##############################################
	if(!empty($total_iud)) { $iud_ins_percent = number_format($total_inserts_indi / $total_iud, 2, '.', ','); } else { $iud_ins_percent = 0; }
	if(!empty($total_iud)) { $iud_upd_percent = number_format($total_updates_indi / $total_iud, 2, '.', ','); } else { $iud_upd_percent = 0; }
	if(!empty($total_iud)) { $iud_del_percent = number_format($total_deletes_indi / $total_iud, 2, '.', ','); } else { $iud_del_percent = 0; }

	#################################################
	#PREPARE THE BAR WIDTHS AND ROUND THE NUMBERS OUT
	#################################################
	$iud_ins_bar_w = number_format($iud_bar_width * $iud_ins_percent);
	$iud_upd_bar_w = number_format($iud_bar_width * $iud_upd_percent);
	$iud_del_bar_w = number_format($iud_bar_width * $iud_del_percent);
	
	#######################################################################################################
	#TALLY UP THE WIDTHS AND TAKE ANY NOMINAL REMAINDER +/- IUD_BAR_WIDTH AND +/- IT FROM THE LARGEST SCORE
	#######################################################################################################
	$iud_bar_total = $iud_ins_bar_w + $iud_upd_bar_w + $iud_del_bar_w;
	
	if($iud_bar_total < $iud_bar_width && $iud_bar_total > 0) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_width-$iud_bar_total;
		
		############################
		#ADD IT TO HIGHEST IUD SCORE
		############################
		if($total_inserts_indi > $total_updates_indi && $total_inserts_indi > $total_deletes_indi) $iud_ins_bar_w += $iud_remain;
		elseif($total_updates_indi > $total_inserts_indi && $total_updates_indi > $total_deletes_indi) $iud_upd_bar_w += $iud_remain;
		else $iud_del_bar_w += $iud_remain;
	}
	elseif($iud_bar_total > $iud_bar_width) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_total-$iud_bar_width;
		
		###################################
		#SUBTRACT IT FROM HIGHEST IUD SCORE
		###################################
		if($total_inserts_indi > $total_updates_indi && $total_inserts_indi > $total_deletes_indi) $iud_ins_bar_w -= $iud_remain;
		elseif($total_updates_indi > $total_inserts_indi && $total_updates_indi > $total_deletes_indi) $iud_upd_bar_w -= $iud_remain;
		else $iud_del_bar_w -= $iud_remain;
	}
	
	##################################
	#PUT TOGETHER OUR IUD TOOLTIP VARS
	##################################
	$iud_tt_ins = "<div style=line-height:12px;>";
	$iud_tt_ins .= "Total Inserts:&nbsp;<strong class=body_highlight>" . number_format($total_inserts_indi) . "</strong>";
	$iud_tt_ins .= "</div>";
	
	$iud_tt_upd = "<div style=line-height:12px;>";
	$iud_tt_upd .= "Total Updates:&nbsp;<strong class=body_highlight>" . number_format($total_updates_indi) . "</strong>";
	$iud_tt_upd .= "</div>";
	
	$iud_tt_del = "<div style=line-height:12px;>";
	$iud_tt_del .= "Total Deletes:&nbsp;<strong class=body_highlight>" . number_format($total_deletes_indi) . "</strong>";
	$iud_tt_del .= "</div>";
	
	###########################################################################################
	#SHOW EITHER THE THREE IUD BARS TOGETHER OR THE DEFAULT NO-DATA BAR IF THERE IS NO IUD DATA
	###########################################################################################

	################################
	#PREP SOME FINAL CELL DIMENSIONS
	################################
	$iud_bar_width_px = $iud_bar_width . "px";
	$iud_bar_height_px = $iud_bar_height . "px";

	####################################
	#START WITH OUR BEGINNING TABLE WORK
	####################################
	$ajax_return .= "<div style=\"margin-top:10px;\">";
	$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$iud_bar_width_px; empty-cells:show;\">";
	$ajax_return .= "<tr>";
	
	if($total_iud > 0)
	{
		###########
		#INSERT BAR
		###########
		if($iud_ins_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			$ins_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_ins_bar_w_px = $iud_ins_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_ins_bar_w_px; height:$iud_bar_height_px; background-color:#7B7B7B; $ins_border\" onMouseOver=\"tooltip.show('$iud_tt_ins')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#UPDATE BAR
		###########
		if($iud_upd_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			if($iud_ins_bar_w > 0) $upd_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $upd_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_upd_bar_w_px = $iud_upd_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_upd_bar_w_px; height:$iud_bar_height_px; background-color:#C9C9C9; $upd_border\" onMouseOver=\"tooltip.show('$iud_tt_upd')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#DELETE BAR
		###########
		if($iud_del_bar_w > 0) 
		{
			############
			#PREP BORDER
			############
			if($iud_upd_bar_w > 0 || $iud_ins_bar_w > 0) $del_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $del_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_del_bar_w_px = $iud_del_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_del_bar_w_px; height:$iud_bar_height_px; background-color:#F1F1F1; $del_border\" onMouseOver=\"tooltip.show('$iud_tt_del')\" onMouseOut='tooltip.hide()'></td>";
		}
	}
	else
	{
		##################################
		#DEFAULT THE NO-DATA BAR AND WIDTH
		##################################
		$ajax_return .= "<td style=\"width:$iud_bar_width_px; height:$iud_bar_height_px; background-color:#000000; border:1px solid $bar_border_color;\" onMouseOver=\"tooltip.show('No data to report.')\" onMouseOut='tooltip.hide()'></td>";
	}

	##############################################
	#COMPLETE OUR HTML TABLE WORK FOR THE IUD AREA
	##############################################
	$ajax_return .= "</tr>";
	$ajax_return .= "</table></div>";

	###################################
	#FORMAT RIGHT-SIDE RECAP STATISTICS
	###################################
	if(!empty($total_days_tracked)) { $average_day_indi = $total_commits_indi / $total_days_tracked; } else { $average_day_indi = 0; } 
	$average_hour_indi = $average_day_indi / 24;
	$average_minute_indi = $average_hour_indi / 60;
	$average_second_indi = $average_minute_indi / 60;
	$average_day_indi = number_format($average_day_indi, 2, '.', ',');
	$average_hour_indi = number_format($average_hour_indi, 2, '.', ',');
	$average_minute_indi = number_format($average_minute_indi, 2, '.', ',');
	$average_second_indi = number_format($average_second_indi, 4, '.', ',');
	$total_commits_indi = number_format($total_commits_indi, 0, '', ',');
	$total_rollbacks_indi = number_format($total_rollbacks_indi);
	$total_block_reads_indi = number_format($total_block_reads_indi);
	$total_buffer_hits_indi = number_format($total_buffer_hits_indi);
	$total_inserts_indi = number_format($total_inserts_indi);
	$total_updates_indi = number_format($total_updates_indi);
	$total_deletes_indi = number_format($total_deletes_indi);
	$total_seq_scans_indi = number_format($total_seq_scans_indi, 0, '', ',');
	$total_idx_scans_indi = number_format($total_idx_scans_indi, 0, '', ',');
	$total_seq_rows_indi = number_format($total_seq_rows_indi, 0, '', ',');
	$total_idx_rows_indi = number_format($total_idx_rows_indi, 0, '', ',');
	$peak_db_day = number_format($peak_db_day, 0, '', ',');

	######################
	#DISPLAY TOTAL COMMITS
	######################
	$ajax_return .= "<div class=\"stat_wrapper body_secondary\" style=\"float:left; margin:25px 15px 0 0; padding:10px;\">";
	$ajax_return .= "<span class=\"body_highlight\" style=\"font-weight:bold;\">Commits:</span> <span class=\"body_primary\" style=\"font-weight:bold;\">$total_commits_indi</span>";
	$ajax_return .= "</div>";

	######################
	#DISPLAY PEAK AND DATE
	######################
	$ajax_return .= "<div class=\"stat_wrapper body_secondary\" style=\"float:left; margin:25px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Peak: <span class=\"body_primary\" style=\"font-weight:bold;\">$peak_db_day</span>&nbsp;";
	if($peak_db_day > 0) $ajax_return .= "$peak_db_day_date";
	else $ajax_return .= "N/A";
	$ajax_return .= "</div>";

	######################
	#DISPLAY AVERAGE / DAY
	######################
	$ajax_return .= "<div class=\"stat_wrapper body_secondary\" style=\"float:left; margin:25px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Avg/Day: <span class=\"body_primary\" style=\"font-weight:bold;\">$average_day_indi</span>";
	$ajax_return .= "</div>";

	#######################
	#DISPLAY AVERAGE / HOUR
	#######################
	$ajax_return .= "<div class=\"stat_wrapper body_secondary\" style=\"float:left; margin:25px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Avg/Hr: <span class=\"body_primary\" style=\"font-weight:bold;\">$average_hour_indi</span>";
	$ajax_return .= "</div>";

	######################
	#DISPLAY AVERAGE / MIN
	######################
	$ajax_return .= "<div class=\"stat_wrapper body_secondary\" style=\"float:left; margin:25px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Avg/Min: <span class=\"body_primary\" style=\"font-weight:bold;\">$average_minute_indi</span>";
	$ajax_return .= "</div>";

	##########################
	#CLEAR FLOATS AND CONTINUE
	##########################
	$ajax_return .= "<div style=\"clear:both;\"></div>";

###############################
#START UP THE RECAP DIV / TABLE
###############################
$ajax_return .= <<<AJR
<div style="margin-top:25px;">
<table class="stat_recap_table ps_core_table">
<tr>
<td class="body_secondary" style="width:75px;">Total&nbsp;Rollbacks:</td>
<td class="body_primary" style="font-weight:bold;">$total_rollbacks_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;Block&nbsp;Reads:</td>
<td class="body_primary" style="font-weight:bold;">$total_block_reads_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;Buffer&nbsp;Hits:</td>
<td class="body_primary" style="font-weight:bold;">$total_buffer_hits_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;User&nbsp;Inserts:</td>
<td class="body_primary" style="font-weight:bold;">$total_inserts_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;User&nbsp;Updates:</td>
<td class="body_primary" style="font-weight:bold;">$total_updates_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;User&nbsp;Deletes:</td>
<td class="body_primary" style="font-weight:bold;">$total_deletes_indi</td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;SEQ&nbsp;Scans:</td>
<td class="body_primary" style="font-weight:bold;">$total_seq_scans_indi&nbsp;<span class="body_secondary" style="font-weight:normal;">(Rows&nbsp;Used:&nbsp;$total_seq_rows_indi)</span></td>
</tr>
<tr>
<td class="body_secondary">Total&nbsp;IDX&nbsp;Scans</td>
<td class="body_primary" style="font-weight:bold;">$total_idx_scans_indi&nbsp;<span class="body_secondary" style="font-weight:normal;">(Rows&nbsp;Used:&nbsp;$total_idx_rows_indi)</span></td>
</tr>
</table>
</td>
</tr>
</table>
</div>
</div>
AJR;

	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_MVM_DB_STATS
##########################################################################################################
##########################################################################################################
function retrieve_mvm_db_stats($postgrestats_dir,$month_a,$year_a,$month_b,$year_b,$graphics_src,$vs_chartting)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($month_a) || empty($year_a) || empty($month_b) || empty($year_b))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}
	
	##############################################################
	#FIRST MAKE SURE THEY ARE NOT THE SAME MONTHS WE ARE COMPARING
	##############################################################
	if($month_a == $month_b && $year_a == $year_b)
	{	
		$stat_msg = "Please select two different months.";
		echo $stat_msg;
		exit;
	}
	
	##################################################################
	#DOUBLE CHECK THE CHARTTING VAR AND MAKE SURE IT HAS A LEGIT VALUE
	##################################################################
	if($vs_chartting != "commits" && $vs_chartting != "rollbacks" && $vs_chartting != "block_reads" && $vs_chartting != "buffer_hits" && $vs_chartting != "inserts" && $vs_chartting != "updates" && $vs_chartting != "deletes" && $vs_chartting != "seq_scans" && $vs_chartting != "idx_scans")
	{
		###########################################
		#DEFAULT THE CHART TO SHOWING COMMIT VALUES
		###########################################
		$vs_chartting = "commits";
	}
	
	#########################################
	#SWITCH STATEMENT TO OBTAIN A MONTH NAMES
	#########################################
	switch($month_a)
	{
		case "01": $month_a_str = "January"; break;
		case "02": $month_a_str = "February"; break;
		case "03": $month_a_str = "March"; break;
		case "04": $month_a_str = "April"; break;
		case "05": $month_a_str = "May"; break;
		case "06": $month_a_str = "June"; break;
		case "07": $month_a_str = "July"; break;
		case "08": $month_a_str = "August"; break;
		case "09": $month_a_str = "September"; break;
		case "10": $month_a_str = "October"; break;
		case "11": $month_a_str = "November"; break;
		case "12": $month_a_str = "December"; break;
	}
	switch($month_b)
	{
		case "01": $month_b_str = "January"; break;
		case "02": $month_b_str = "February"; break;
		case "03": $month_b_str = "March"; break;
		case "04": $month_b_str = "April"; break;
		case "05": $month_b_str = "May"; break;
		case "06": $month_b_str = "June"; break;
		case "07": $month_b_str = "July"; break;
		case "08": $month_b_str = "August"; break;
		case "09": $month_b_str = "September"; break;
		case "10": $month_b_str = "October"; break;
		case "11": $month_b_str = "November"; break;
		case "12": $month_b_str = "December"; break;
	}
	
	###############################################################
	#PUT THE LOG FILE NAMES TOGETHER AND MAKE SURE THE FILES EXISTS
	###############################################################
	$log_file_name_a = "$postgrestats_dir/postgrestats/data/$year_a/$month_a";
	$log_file_name_b = "$postgrestats_dir/postgrestats/data/$year_b/$month_b";
	
	if(!file_exists($log_file_name_a)) $stat_msg = "Zero statistics have been collected for month: $month_a/$year_a";
	if(!file_exists($log_file_name_b))
	{
		if(!empty($stat_msg)) $stat_msg .= "<br>";
		$stat_msg .= "Zero statistics have been collected for month: $month_b/$year_b";
	}
	
	###################
	#RETURN WITH ERRORS
	###################
	if(!empty($stat_msg))
	{
		echo $stat_msg;
		exit;
	}
	
	##################################
	#ATTEMPT TO OPEN UP BOTH LOG FILES
	##################################
	if(!$open_log_a = fopen($log_file_name_a, "r")) $stat_msg = "Error opening up log file for month: $month_a/$year_a";
	if(!$open_log_b = fopen($log_file_name_b, "r"))
	{
		if(!empty($stat_msg)) $stat_msg .= "<br>";
		$stat_msg .= "Error opening up log file for month: $month_b/$year_b";
	}
	
	###################
	#RETURN WITH ERRORS
	###################
	if(!empty($stat_msg))
	{
		echo $stat_msg;
		exit;
	}
	
	#####################################
	#VARS FOR TRACKING TOTAL DAYS STATTED
	#####################################
	$total_days_tracked_month_a = 0;
	$total_days_tracked_month_b = 0;
	
	###############################################
	#VARS FOR TRACKING THE PEAK LEVEL FOR EACH STAT
	###############################################
	$highest_commits_month_a = 0;
	$highest_commits_month_b = 0;
	$highest_rollbacks_month_a = 0;
	$highest_rollbacks_month_b = 0;
	$highest_block_reads_month_a = 0;
	$highest_block_reads_month_b = 0;
	$highest_buffer_hits_month_a = 0;
	$highest_buffer_hits_month_b = 0;
	$highest_inserts_month_a = 0;
	$highest_inserts_month_b = 0;
	$highest_updates_month_a = 0;
	$highest_updates_month_b = 0;
	$highest_deletes_month_a = 0;
	$highest_deletes_month_b = 0;
	$highest_seq_scans_month_a = 0;
	$highest_seq_scans_month_b = 0;
	$highest_seq_rows_month_a = 0;
	$highest_seq_rows_month_b = 0;
	$highest_idx_scans_month_a = 0;
	$highest_idx_scans_month_b = 0;
	$highest_idx_rows_month_a = 0;
	$highest_idx_rows_month_b = 0;
	
	#####################
	#VARS FOR TOTAL STATS
	#####################
	$total_commits_month_a = 0;
	$total_commits_month_b = 0;
	$total_rollbacks_month_a = 0;
	$total_rollbacks_month_b = 0;
	$total_block_reads_month_a = 0;
	$total_block_reads_month_b = 0;
	$total_buffer_hits_month_a = 0;
	$total_buffer_hits_month_b = 0;
	$total_inserts_month_a = 0;
	$total_inserts_month_b = 0;
	$total_updates_month_a = 0;
	$total_updates_month_b = 0;
	$total_deletes_month_a = 0;
	$total_deletes_month_b = 0;
	$total_seq_scans_month_a = 0;
	$total_seq_scans_month_b = 0;
	$total_seq_rows_month_a = 0;
	$total_seq_rows_month_b = 0;
	$total_idx_scans_month_a = 0;
	$total_idx_scans_month_b = 0;
	$total_idx_rows_month_a = 0;
	$total_idx_rows_month_b = 0;

	#VARS TO HOUSE TOOLTIP VALUES
	$tt_commits_a = 0;
	$tt_rollbacks_a = 0;
	$tt_block_reads_a = 0;
	$tt_buffer_hits_a = 0;
	$tt_inserts_a = 0;
	$tt_updates_a = 0;
	$tt_deletes_a = 0;
	$tt_seq_scans_a = 0;
	$tt_idx_scans_a = 0;
	$tt_commits_b = 0;
	$tt_rollbacks_b = 0;
	$tt_block_reads_b = 0;
	$tt_buffer_hits_b = 0;
	$tt_inserts_b = 0;
	$tt_updates_b = 0;
	$tt_deletes_b = 0;
	$tt_seq_scans_b = 0;
	$tt_idx_scans_b = 0;

	###################
	#BAR AND TABLE VARS
	###################
	$bar_height_vert = 150;
	$bar_width = 13;
	$table_width = $bar_width * 2;

	################################################
	#RESET BAR WIDTH NOW ACCOUNTING FOR BORDER STYLE
	################################################
	$bar_width = $bar_width -2;

	################################################################################
	#VARS FOR BAR COLORS - CHECK INTERFACE VAR FIRST AND APPLY THE APPROPRIATE COLOR
	################################################################################
	$bar_vert_alt_color = "#FFFFFF";
	$bar_vert_color_a = "#FF6633";
	$bar_vert_color_b = "#7B7B7B";
	$bar_border_color = "#444444";

	###################################################################
	#GO THROUGH LOG FILE A - SEARCH FOR THE DATE LINE - GET THE COMMITS
	###################################################################
	while(!feof($open_log_a))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log_a);

		###############################################################
		#SEARCH FOR A MONTH AND YEAR IN EACH LINE - LOOKING FOR MATCHES
		###############################################################
		if(substr($db_line_single,0,2) == $month_a && substr($db_line_single,6,4) == $year_a)
		{
			$day_flag = substr($db_line_single,3,2);		

			#############
			#EXPLODE LINE
			#############
			$log_line = explode(",",$db_line_single);

			###############################################################################
			#GET COMMITS, ROLLBACKS, BLOCK READS, BUFFER HITS, INSERTS, UPDATES AND DELETES
			###############################################################################
			if(!empty($log_line[3])) { ${'db_commits_a_' . $day_flag} = $log_line[3]; } else { ${'db_commits_a_' . $day_flag} = 0; }
			if(!empty($log_line[4])) { ${'db_rollbacks_a_' . $day_flag} = $log_line[4]; } else { ${'db_rollbacks_a_' . $day_flag} = 0; }
			if(!empty($log_line[5])) { ${'db_block_reads_a_' . $day_flag} = $log_line[5]; } else { ${'db_block_reads_a_' . $day_flag} = 0; }
			if(!empty($log_line[6])) { ${'db_buffer_hits_a_' . $day_flag} = $log_line[6]; } else { ${'db_buffer_hits_a_' . $day_flag} = 0; }
			if(!empty($log_line[7])) { ${'db_inserts_a_' . $day_flag} = $log_line[7]; } else { ${'db_inserts_a_' . $day_flag} = 0; }
			if(!empty($log_line[8])) { ${'db_updates_a_' . $day_flag} = $log_line[8]; } else { ${'db_updates_a_' . $day_flag} = 0; }
			if(!empty($log_line[9])) { ${'db_deletes_a_' . $day_flag} = $log_line[9]; } else { ${'db_deletes_a_' . $day_flag} = 0; }
			if(!empty($log_line[10])) { ${'db_seq_scans_a_' . $day_flag} = $log_line[10]; } else { ${'db_seq_scans_a_' . $day_flag} = 0; }
			if(!empty($log_line[11])) { ${'db_idx_scans_a_' . $day_flag} = $log_line[11]; } else { ${'db_idx_scans_a_' . $day_flag} = 0; }
			if(!empty($log_line[12])) { ${'db_seq_rows_a_' . $day_flag} = $log_line[12]; } else { ${'db_seq_rows_a_' . $day_flag} = 0; }
			if(!empty($log_line[13])) { ${'db_idx_rows_a_' . $day_flag} = $log_line[13]; } else { ${'db_idx_rows_a_' . $day_flag} = 0; }

			####################################
			#FORCE TYPE CAST TO INTEGER FOR SAFE
			####################################
			${'db_commits_a_' . $day_flag} += 0;
			${'db_rollbacks_a_' . $day_flag} += 0;
			${'db_block_reads_a_' . $day_flag} += 0;
			${'db_buffer_hits_a_' . $day_flag} += 0;
			${'db_inserts_a_' . $day_flag} += 0;
			${'db_updates_a_' . $day_flag} += 0;
			${'db_deletes_a_' . $day_flag} += 0;
			${'db_seq_scans_a_' . $day_flag} += 0;
			${'db_idx_scans_a_' . $day_flag} += 0;
			${'db_seq_rows_a_' . $day_flag} += 0;
			${'db_idx_rows_a_' . $day_flag} += 0;

			############################
			#TALLY UP OUR "HIGHEST" VARS
			############################
			if(${'db_commits_a_' . $day_flag} >= $highest_commits_month_a)
			{
				$highest_commits_month_a = ${'db_commits_a_' . $day_flag};
				$highest_commits_day_month_a = $day_flag;
				$highest_commits_date_month_a = "$month_a-$day_flag-$year_a";	
			}
			if(${'db_rollbacks_a_' . $day_flag} >= $highest_rollbacks_month_a)
			{
				$highest_rollbacks_month_a = ${'db_rollbacks_a_' . $day_flag};
				$highest_rollbacks_day_month_a = $day_flag;
				$highest_rollbacks_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_block_reads_a_' . $day_flag} >= $highest_block_reads_month_a)
			{
				$highest_block_reads_month_a = ${'db_block_reads_a_' . $day_flag};
				$highest_block_reads_day_month_a = $day_flag;
				$highest_block_reads_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_buffer_hits_a_' . $day_flag} >= $highest_buffer_hits_month_a)
			{
				$highest_buffer_hits_month_a = ${'db_buffer_hits_a_' . $day_flag};
				$highest_buffer_hits_day_month_a = $day_flag;
				$highest_buffer_hits_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_inserts_a_' . $day_flag} >= $highest_inserts_month_a)
			{
				$highest_inserts_month_a = ${'db_inserts_a_' . $day_flag};
				$highest_inserts_day_month_a = $day_flag;
				$highest_inserts_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_updates_a_' . $day_flag} >= $highest_updates_month_a)
			{
				$highest_updates_month_a = ${'db_updates_a_' . $day_flag};
				$highest_updates_day_month_a = $day_flag;
				$highest_updates_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_deletes_a_' . $day_flag} >= $highest_deletes_month_a)
			{
				$highest_deletes_month_a = ${'db_deletes_a_' . $day_flag};
				$highest_deletes_day_month_a = $day_flag;
				$highest_deletes_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_seq_scans_a_' . $day_flag} >= $highest_seq_scans_month_a)
			{
				$highest_seq_scans_month_a = ${'db_seq_scans_a_' . $day_flag};
				$highest_seq_scans_day_month_a = $day_flag;
				$highest_seq_scans_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_idx_scans_a_' . $day_flag} >= $highest_idx_scans_month_a)
			{
				$highest_idx_scans_month_a = ${'db_idx_scans_a_' . $day_flag};
				$highest_idx_scans_day_month_a = $day_flag;
				$highest_idx_scans_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_seq_rows_a_' . $day_flag} >= $highest_seq_rows_month_a)
			{
				$highest_seq_rows_month_a = ${'db_seq_rows_a_' . $day_flag};
				$highest_seq_rows_day_month_a = $day_flag;
				$highest_seq_rows_date_month_a = "$month_a-$day_flag-$year_a";
			}
			if(${'db_idx_rows_a_' . $day_flag} >= $highest_idx_rows_month_a)
			{
				$highest_idx_rows_month_a = ${'db_idx_rows_a_' . $day_flag};
				$highest_idx_rows_day_month_a = $day_flag;
				$highest_idx_rows_date_month_a = "$month_a-$day_flag-$year_a";
			}
			
			#########################
			#TALLY UP ALL STAT TOTALS
			#########################
			$total_commits_month_a += ${'db_commits_a_' . $day_flag};
			$total_rollbacks_month_a += ${'db_rollbacks_a_' . $day_flag};
			$total_block_reads_month_a += ${'db_block_reads_a_' . $day_flag};
			$total_buffer_hits_month_a += ${'db_buffer_hits_a_' . $day_flag};
			$total_inserts_month_a += ${'db_inserts_a_' . $day_flag};
			$total_updates_month_a += ${'db_updates_a_' . $day_flag};
			$total_deletes_month_a += ${'db_deletes_a_' . $day_flag};		
			$total_seq_scans_month_a += ${'db_seq_scans_a_' . $day_flag};
			$total_seq_rows_month_a += ${'db_seq_rows_a_' . $day_flag};
			$total_idx_scans_month_a += ${'db_idx_scans_a_' . $day_flag};
			$total_idx_rows_month_a += ${'db_idx_rows_a_' . $day_flag};
			
			##########################
			#TALLY UP THE DAYS TRACKED
			##########################
			if(isset(${'db_commits_a_' . $day_flag})) $total_days_tracked_month_a++;
		}
	}

	############
	#CLOSE IT UP
	############
	fclose($open_log_a);
	
	###################################################################
	#GO THROUGH LOG FILE B - SEARCH FOR THE DATE LINE - GET THE COMMITS
	###################################################################
	while(!feof($open_log_b))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log_b);
		
		###############################################################
		#SEARCH FOR A MONTH AND YEAR IN EACH LINE - LOOKING FOR MATCHES
		###############################################################
		if(substr($db_line_single,0,2) == $month_b && substr($db_line_single,6,4) == $year_b)
		{
			$day_flag = substr($db_line_single,3,2);
			
			#############
			#EXPLODE LINE
			#############
			$log_line = explode(",",$db_line_single);
			
			###############################################################################
			#GET COMMITS, ROLLBACKS, BLOCK READS, BUFFER HITS, INSERTS, UPDATES AND DELETES
			###############################################################################
			if(!empty($log_line[3])) { ${'db_commits_b_' . $day_flag} = $log_line[3]; } else { ${'db_commits_b_' . $day_flag} = 0; }
			if(!empty($log_line[4])) { ${'db_rollbacks_b_' . $day_flag} = $log_line[4]; } else { ${'db_rollbacks_b_' . $day_flag} = 0; }
			if(!empty($log_line[5])) { ${'db_block_reads_b_' . $day_flag} = $log_line[5]; } else { ${'db_block_reads_b_' . $day_flag} = 0; }
			if(!empty($log_line[6])) { ${'db_buffer_hits_b_' . $day_flag} = $log_line[6]; } else { ${'db_buffer_hits_b_' . $day_flag} = 0; }
			if(!empty($log_line[7])) { ${'db_inserts_b_' . $day_flag} = $log_line[7]; } else { ${'db_inserts_b_' . $day_flag} = 0; }
			if(!empty($log_line[8])) { ${'db_updates_b_' . $day_flag} = $log_line[8]; } else { ${'db_updates_b_' . $day_flag} = 0; }
			if(!empty($log_line[9])) { ${'db_deletes_b_' . $day_flag} = $log_line[9]; } else { ${'db_deletes_b_' . $day_flag} = 0; }
			if(!empty($log_line[10])) { ${'db_seq_scans_b_' . $day_flag} = $log_line[10]; } else { ${'db_seq_scans_b_' . $day_flag} = 0; }
			if(!empty($log_line[11])) { ${'db_idx_scans_b_' . $day_flag} = $log_line[11]; } else { ${'db_idx_scans_b_' . $day_flag} = 0; }
			if(!empty($log_line[12])) { ${'db_seq_rows_b_' . $day_flag} = $log_line[12]; } else { ${'db_seq_rows_b_' . $day_flag} = 0; }
			if(!empty($log_line[13])) { ${'db_idx_rows_b_' . $day_flag} = $log_line[13]; } else { ${'db_idx_rows_b_' . $day_flag} = 0; }

			####################################
			#FORCE TYPE CAST TO INTEGER FOR SAFE
			####################################
			${'db_commits_b_' . $day_flag} += 0;
			${'db_rollbacks_b_' . $day_flag} += 0;
			${'db_block_reads_b_' . $day_flag} += 0;
			${'db_buffer_hits_b_' . $day_flag} += 0;
			${'db_inserts_b_' . $day_flag} += 0;
			${'db_updates_b_' . $day_flag} += 0;
			${'db_deletes_b_' . $day_flag} += 0;
			${'db_seq_scans_b_' . $day_flag} += 0;
			${'db_idx_scans_b_' . $day_flag} += 0;
			${'db_seq_rows_b_' . $day_flag} += 0;
			${'db_idx_rows_b_' . $day_flag} += 0;

			############################
			#TALLY UP OUR "HIGHEST" VARS
			############################
			if(${'db_commits_b_' . $day_flag} >= $highest_commits_month_b)
			{
				$highest_commits_month_b = ${'db_commits_b_' . $day_flag};
				$highest_commits_day_month_b = $day_flag;
				$highest_commits_date_month_b = "$month_b-$day_flag-$year_b";	
			}
			if(${'db_rollbacks_b_' . $day_flag} >= $highest_rollbacks_month_b)
			{
				$highest_rollbacks_month_b = ${'db_rollbacks_b_' . $day_flag};
				$highest_rollbacks_day_month_b = $day_flag;
				$highest_rollbacks_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_block_reads_b_' . $day_flag} >= $highest_block_reads_month_b)
			{
				$highest_block_reads_month_b = ${'db_block_reads_b_' . $day_flag};
				$highest_block_reads_day_month_b = $day_flag;
				$highest_block_reads_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_buffer_hits_b_' . $day_flag} >= $highest_buffer_hits_month_b)
			{
				$highest_buffer_hits_month_b = ${'db_buffer_hits_b_' . $day_flag};
				$highest_buffer_hits_day_month_b = $day_flag;
				$highest_buffer_hits_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_inserts_b_' . $day_flag} >= $highest_inserts_month_b)
			{
				$highest_inserts_month_b = ${'db_inserts_b_' . $day_flag};
				$highest_inserts_day_month_b = $day_flag;
				$highest_inserts_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_updates_b_' . $day_flag} >= $highest_updates_month_b)
			{
				$highest_updates_month_b = ${'db_updates_b_' . $day_flag};
				$highest_updates_day_month_b = $day_flag;
				$highest_updates_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_deletes_b_' . $day_flag} >= $highest_deletes_month_b)
			{
				$highest_deletes_month_b = ${'db_deletes_b_' . $day_flag};
				$highest_deletes_day_month_b = $day_flag;
				$highest_deletes_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_seq_scans_b_' . $day_flag} >= $highest_seq_scans_month_b)
			{
				$highest_seq_scans_month_b = ${'db_seq_scans_b_' . $day_flag};
				$highest_seq_scans_day_month_b = $day_flag;
				$highest_seq_scans_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_idx_scans_b_' . $day_flag} >= $highest_idx_scans_month_b)
			{
				$highest_idx_scans_month_b = ${'db_idx_scans_b_' . $day_flag};
				$highest_idx_scans_day_month_b = $day_flag;
				$highest_idx_scans_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_seq_rows_b_' . $day_flag} >= $highest_seq_rows_month_b)
			{
				$highest_seq_rows_month_b = ${'db_seq_rows_b_' . $day_flag};
				$highest_seq_rows_day_month_b = $day_flag;
				$highest_seq_rows_date_month_b = "$month_b-$day_flag-$year_b";
			}
			if(${'db_idx_rows_b_' . $day_flag} >= $highest_idx_rows_month_b)
			{
				$highest_idx_rows_month_b = ${'db_idx_rows_b_' . $day_flag};
				$highest_idx_rows_day_month_b = $day_flag;
				$highest_idx_rows_date_month_b = "$month_b-$day_flag-$year_b";
			}

			#########################
			#TALLY UP ALL STAT TOTALS
			#########################
			$total_commits_month_b += ${'db_commits_b_' . $day_flag};
			$total_rollbacks_month_b += ${'db_rollbacks_b_' . $day_flag};
			$total_block_reads_month_b += ${'db_block_reads_b_' . $day_flag};
			$total_buffer_hits_month_b += ${'db_buffer_hits_b_' . $day_flag};
			$total_inserts_month_b += ${'db_inserts_b_' . $day_flag};
			$total_updates_month_b += ${'db_updates_b_' . $day_flag};
			$total_deletes_month_b += ${'db_deletes_b_' . $day_flag};
			$total_seq_scans_month_b += ${'db_seq_scans_b_' . $day_flag};
			$total_seq_rows_month_b += ${'db_seq_rows_b_' . $day_flag};
			$total_idx_scans_month_b += ${'db_idx_scans_b_' . $day_flag};
			$total_idx_rows_month_b += ${'db_idx_rows_b_' . $day_flag};

			##########################
			#TALLY UP THE DAYS TRACKED
			##########################
			if(isset(${'db_commits_b_' . $day_flag})) $total_days_tracked_month_b++;
		}
	}

	############
	#CLOSE IT UP
	############
	fclose($open_log_b);

###########################################################################
#START PUT TOGETHER OUR RETURNING HTML TABLE WORK FOR THE AJAX RETURN VALUE
###########################################################################
$ajax_return = <<<AJR
<div class="stat_wrapper" style="margin:10px 0 25px 0;">
<table class="stat_chart_table" style="width:auto;">
<tr>
AJR;

	############################################################################
	#FIND OUT WHICH PEAK IS HIGHER - WE'LL USE THIS TO DETERMINE ALL BAR HEIGHTS
	############################################################################
	if($highest_commits_month_a > $highest_commits_month_b) $peak_db_day = $highest_commits_month_a;
	else $peak_db_day = $highest_commits_month_b;

	#############################################################################
	#PUT TOGETHER THE MONTH TABLEWORK - USING OUR BAR HEIGHT VARS AS GRAPH VALUES
	#############################################################################
	for($i=1; $i<32; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";

		##################################
		#PUT TOGETHER TOOLTIPS FOR MONTH A
		#FIRST PREP OUR COMMIT TOOLTIP VAR
		##################################
		if(!empty(${'db_commits_a_' . $i}))
		{
			$tt_commits_a = ${'db_commits_a_' . $i};
			if($tt_commits_a == 0 || empty($tt_commits_a)) $tt_commits_a = "0";
			else $tt_commits_a = number_format($tt_commits_a);
		}

		################################
		#PREP OUR ROLL BACKS TOOLTIP VAR
		################################
		if(!empty(${'db_rollbacks_a_' . $i}))
		{
			$tt_rollbacks_a = ${'db_rollbacks_a_' . $i};
			if($tt_rollbacks_a == 0 || empty($tt_rollbacks_a)) $tt_rollbacks_a = "0";
			else $tt_rollbacks_a = number_format($tt_rollbacks_a);
		}

		#################################
		#PREP OUR BLOCK READS TOOLTIP VAR
		#################################
		if(!empty(${'db_block_reads_a_' . $i}))
		{
			$tt_block_reads_a = ${'db_block_reads_a_' . $i};
			if($tt_block_reads_a == 0 || empty($tt_block_reads_a)) $tt_block_reads_a = "0";
			else $tt_block_reads_a = number_format($tt_block_reads_a);
		}

		#################################
		#PREP OUR BUFFER HITS TOOLTIP VAR
		#################################
		if(!empty(${'db_buffer_hits_a_' . $i}))
		{
			$tt_buffer_hits_a = ${'db_buffer_hits_a_' . $i};
			if($tt_buffer_hits_a == 0 || empty($tt_buffer_hits_a)) $tt_buffer_hits_a = "0";
			else $tt_buffer_hits_a = number_format($tt_buffer_hits_a);
		}

		############################
		#PREP OUR INSERT TOOLTIP VAR
		############################
		if(!empty(${'db_inserts_a_' . $i}))
		{
			$tt_inserts_a = ${'db_inserts_a_' . $i};
			if($tt_inserts_a == 0 || empty($tt_inserts_a)) $tt_inserts_a = "0";
			else $tt_inserts_a = number_format($tt_inserts_a);
		}

		############################
		#PREP OUR UPDATE TOOLTIP VAR
		############################
		if(!empty(${'db_updates_a_' . $i}))
		{		
			$tt_updates_a = ${'db_updates_a_' . $i};
			if($tt_updates_a == 0 || empty($tt_updates_a)) $tt_updates_a = "0";
			else $tt_updates_a = number_format($tt_updates_a);
		}

		############################
		#PREP OUR DELETE TOOLTIP VAR
		############################
		if(!empty(${'db_deletes_a_' . $i}))
		{
			$tt_deletes_a = ${'db_deletes_a_' . $i};
			if($tt_deletes_a == 0 || empty($tt_deletes_a)) $tt_deletes_a = "0";
			else $tt_deletes_a = number_format($tt_deletes_a);
		}

		##############################
		#PREP OUR SEQ SCAN TOOLTIP VAR
		##############################
		if(!empty(${'db_seq_scans_a_' . $i}))
		{
			$tt_seq_scans_a = ${'db_seq_scans_a_' . $i};
			if($tt_seq_scans_a == 0 || empty($tt_seq_scans_a)) $tt_seq_scans_a = "0";
			else $tt_seq_scans_a = number_format($tt_seq_scans_a);
		}

		##############################
		#PREP OUR IDX SCAN TOOLTIP VAR
		##############################
		if(!empty(${'db_idx_scans_a_' . $i}))
		{
			$tt_idx_scans_a = ${'db_idx_scans_a_' . $i};
			if($tt_idx_scans_a == 0 || empty($tt_idx_scans_a)) $tt_idx_scans_a = "0";
			else $tt_idx_scans_a = number_format($tt_idx_scans_a);
		}

		################
		#PUT IT TOGETHER
		################
		$tooltip_a = "<div style=line-height:17px;>";

		########
		#COMMITS
		########
		if($vs_chartting == "commits") $tooltip_a .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits_a . "</strong></span><br>";
		else $tooltip_a .= "Commits:&nbsp;<strong>" . $tt_commits_a . "</strong><br>";

		##########
		#ROLLBACKS
		##########
		if($vs_chartting == "rollbacks") $tooltip_a .= "<span class=body_highlight><strong>Rollbacks:&nbsp;" . $tt_rollbacks_a . "</strong></span><br>";
		else $tooltip_a .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks_a . "</strong><br>";

		############
		#BLOCK READS
		############
		if($vs_chartting == "block_reads") $tooltip_a .= "<span class=body_highlight><strong>Block&nbsp;Reads:&nbsp;" . $tt_block_reads_a . "</strong></span><br>";
		else $tooltip_a .= "Block&nbsp;Reads:&nbsp;<strong>" . $tt_block_reads_a . "</strong><br>";

		############
		#BUFFER HITS
		############
		if($vs_chartting == "buffer_hits") $tooltip_a .= "<span class=body_highlight><strong>Buffer&nbsp;Hits:&nbsp;" . $tt_buffer_hits_a . "</strong></span><br>";
		else $tooltip_a .= "Buffer&nbsp;Hits:&nbsp;<strong>" . $tt_buffer_hits_a . "</strong><br>";

		########
		#INSERTS
		########
		if($vs_chartting == "inserts") $tooltip_a .= "<span class=body_highlight><strong>Inserts:&nbsp;" . $tt_inserts_a . "</strong></span><br>";
		else $tooltip_a .= "Inserts:&nbsp;<strong>" . $tt_inserts_a . "</strong><br>";

		########
		#UPDATES
		########
		if($vs_chartting == "updates") $tooltip_a .= "<span class=body_highlight><strong>Updates:&nbsp;" . $tt_updates_a . "</strong></span><br>";
		else $tooltip_a .= "Updates:&nbsp;<strong>" . $tt_updates_a . "</strong><br>";

		########
		#DELETES
		########
		if($vs_chartting == "deletes") $tooltip_a .= "<span class=body_highlight><strong>Deletes:&nbsp;" . $tt_deletes_a . "</strong></span><br>";
		else $tooltip_a .= "Deletes:&nbsp;<strong>" . $tt_deletes_a . "</strong><br>";

		##########
		#SEQ SCANS
		##########
		if($vs_chartting == "seq_scans") $tooltip_a .= "<span class=body_highlight><strong>Sequence&nbsp;Scans:&nbsp;" . $tt_seq_scans_a . "</strong></span><br>";
		else $tooltip_a .= "Sequence&nbsp;Scans:&nbsp;<strong>" . $tt_seq_scans_a . "</strong><br>";

		##########
		#IDX SCANS
		##########
		if($vs_chartting == "idx_scans") $tooltip_a .= "<span class=body_highlight><strong>Index&nbsp;Scans:&nbsp;" . $tt_idx_scans_a . "</strong></span><br>";
		else $tooltip_a .= "Index&nbsp;Scans:&nbsp;<strong>" . $tt_idx_scans_a . "</strong><br>";

		#########################
		#COMPLETE THE TOOLTIP DIV
		#########################
		$tooltip_a .= "</div>";

		##################################
		#PUT TOGETHER TOOLTIPS FOR MONTH B
		#FIRST PREP OUR COMMIT TOOLTIP VAR
		##################################
		if(!empty(${'db_commits_b_' . $i}))
		{
			$tt_commits_b = ${'db_commits_b_' . $i};
			$db_stat_b = $tt_commits_b;
			if($tt_commits_b == 0 || empty($tt_commits_b)) $tt_commits_b = "0";
			else $tt_commits_b = number_format($tt_commits_b);
		}

		################################
		#PREP OUR ROLL BACKS TOOLTIP VAR
		################################
		if(!empty(${'db_rollbacks_b_' . $i}))
		{
			$tt_rollbacks_b = ${'db_rollbacks_b_' . $i};
			if($tt_rollbacks_b == 0 || empty($tt_rollbacks_b)) $tt_rollbacks_b = "0";
			else $tt_rollbacks_b = number_format($tt_rollbacks_b);
		}

		#################################
		#PREP OUR BLOCK READS TOOLTIP VAR
		#################################
		if(!empty(${'db_block_reads_b_' . $i}))
		{
			$tt_block_reads_b = ${'db_block_reads_b_' . $i};
			if($tt_block_reads_b == 0 || empty($tt_block_reads_b)) $tt_block_reads_b = "0";
			else $tt_block_reads_b = number_format($tt_block_reads_b);
		}

		#################################
		#PREP OUR BUFFER HITS TOOLTIP VAR
		#################################
		if(!empty(${'db_buffer_hits_b_' . $i}))
		{
			$tt_buffer_hits_b = ${'db_buffer_hits_b_' . $i};
			if($tt_buffer_hits_b == 0 || empty($tt_buffer_hits_b)) $tt_buffer_hits_b = "0";
			else $tt_buffer_hits_b = number_format($tt_buffer_hits_b);
		}

		############################
		#PREP OUR INSERT TOOLTIP VAR
		############################
		if(!empty(${'db_inserts_b_' . $i}))
		{
			$tt_inserts_b = ${'db_inserts_b_' . $i};
			if($tt_inserts_b == 0 || empty($tt_inserts_b)) $tt_inserts_b = "0";
			else $tt_inserts_b = number_format($tt_inserts_b);
		}

		############################
		#PREP OUR UPDATE TOOLTIP VAR
		############################
		if(!empty(${'db_updates_b_' . $i}))
		{
			$tt_updates_b = ${'db_updates_b_' . $i};
			if($tt_updates_b == 0 || empty($tt_updates_b)) $tt_updates_b = "0";
			else $tt_updates_b = number_format($tt_updates_b);
		}

		############################
		#PREP OUR DELETE TOOLTIP VAR
		############################
		if(!empty(${'db_deletes_b_' . $i}))
		{
			$tt_deletes_b = ${'db_deletes_b_' . $i};
			if($tt_deletes_b == 0 || empty($tt_deletes_b)) $tt_deletes_b = "0";
			else $tt_deletes_b = number_format($tt_deletes_b);
		}

		##############################
		#PREP OUR SEQ SCAN TOOLTIP VAR
		##############################
		if(!empty(${'db_seq_scans_b_' . $i}))
		{
			$tt_seq_scans_b = ${'db_seq_scans_b_' . $i};
			if($tt_seq_scans_b == 0 || empty($tt_seq_scans_b)) $tt_seq_scans_b = "0";
			else $tt_seq_scans_b = number_format($tt_seq_scans_b);
		}

		##############################
		#PREP OUR IDX SCAN TOOLTIP VAR
		##############################
		if(!empty(${'db_idx_scans_b_' . $i}))
		{
			$tt_idx_scans_b = ${'db_idx_scans_b_' . $i};
			if($tt_idx_scans_b == 0 || empty($tt_idx_scans_b)) $tt_idx_scans_b = "0";
			else $tt_idx_scans_b = number_format($tt_idx_scans_b);
		}

		################
		#PUT IT TOGETHER
		################
		$tooltip_b = "<div style=line-height:17px;>";

		########
		#COMMITS
		########
		if($vs_chartting == "commits") $tooltip_b .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits_b . "</strong></span><br>";
		else $tooltip_b .= "Commits:&nbsp;<strong>" . $tt_commits_b . "</strong><br>";

		##########
		#ROLLBACKS
		##########
		if($vs_chartting == "rollbacks") $tooltip_b .= "<span class=body_highlight><strong>Rollbacks:&nbsp;" . $tt_rollbacks_b . "</strong></span><br>";
		else $tooltip_b .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks_b . "</strong><br>";

		############
		#BLOCK READS
		############
		if($vs_chartting == "block_reads") $tooltip_b .= "<span class=body_highlight><strong>Block&nbsp;Reads:&nbsp;" . $tt_block_reads_b . "</strong></span><br>";
		else $tooltip_b .= "Block&nbsp;Reads:&nbsp;<strong>" . $tt_block_reads_b . "</strong><br>";

		############
		#BUFFER HITS
		############
		if($vs_chartting == "buffer_hits") $tooltip_b .= "<span class=body_highlight><strong>Buffer&nbsp;Hits:&nbsp;" . $tt_buffer_hits_b . "</strong></span><br>";
		else $tooltip_b .= "Buffer&nbsp;Hits:&nbsp;<strong>" . $tt_buffer_hits_b . "</strong><br>";

		########
		#INSERTS
		########
		if($vs_chartting == "inserts") $tooltip_b .= "<span class=body_highlight><strong>Inserts:&nbsp;" . $tt_inserts_b . "</strong></span><br>";
		else $tooltip_b .= "Inserts:&nbsp;<strong>" . $tt_inserts_b . "</strong><br>";

		########
		#UPDATES
		########
		if($vs_chartting == "updates") $tooltip_b .= "<span class=body_highlight><strong>Updates:&nbsp;" . $tt_updates_b . "</strong></span><br>";
		else $tooltip_b .= "Updates:&nbsp;<strong>" . $tt_updates_b . "</strong><br>";

		########
		#DELETES
		########
		if($vs_chartting == "deletes") $tooltip_b .= "<span class=body_highlight><strong>Deletes:&nbsp;" . $tt_deletes_b . "</strong></span><br>";
		else $tooltip_b .= "Deletes:&nbsp;<strong>" . $tt_deletes_b . "</strong><br>";

		##########
		#SEQ SCANS
		##########
		if($vs_chartting == "seq_scans") $tooltip_b .= "<span class=body_highlight><strong>Sequence&nbsp;Scans:&nbsp;" . $tt_seq_scans_b . "</strong></span><br>";
		else $tooltip_b .= "Sequence&nbsp;Scans:&nbsp;<strong>" . $tt_seq_scans_b . "</strong><br>";

		##########
		#IDX SCANS
		##########
		if($vs_chartting == "idx_scans") $tooltip_b .= "<span class=body_highlight><strong>Index&nbsp;Scans:&nbsp;" . $tt_idx_scans_b . "</strong></span><br>";
		else $tooltip_b .= "Index&nbsp;Scans:&nbsp;<strong>" . $tt_idx_scans_b . "</strong><br>";

		#########################
		#COMPLETE THE TOOLTIP DIV
		#########################
		$tooltip_b .= "</div>";

		############################################################################################################
		#DETERMINE WHICH HIGHEST VAR IS HIGHER, A OR B. WE'LL USE THE HIGHEST TO DETERMIN ALL BAR HEIGHT PERCENTAGES
		############################################################################################################
		if(${'highest_' . $vs_chartting . '_month_a'} > ${'highest_' . $vs_chartting . '_month_b'}) $peak_vs = 'a';
		else $peak_vs = 'b';

		#######################################################
		#GET THE COMMIT PERCENT BASED ON WHICH STAT IS SELECTED
		#######################################################
		if(!empty(${'highest_' . $vs_chartting . '_month_' . $peak_vs}) && !empty(${'db_' . $vs_chartting . '_a_' . $i})) { $stat_percent_a = ${'db_' . $vs_chartting . '_a_' . $i} / ${'highest_' . $vs_chartting . '_month_' . $peak_vs}; } else { $stat_percent_a = 0; }
		if(!empty(${'highest_' . $vs_chartting . '_month_' . $peak_vs}) && !empty(${'db_' . $vs_chartting . '_b_' . $i})) { $stat_percent_b = ${'db_' . $vs_chartting . '_b_' . $i} / ${'highest_' . $vs_chartting . '_month_' . $peak_vs}; } else { $stat_percent_b = 0; }

		$bar_height_percent_a = $stat_percent_a;
		$bar_height_percent_b = $stat_percent_b;

		$stat_percent_a = (number_format($stat_percent_a, 4, '.', '') * 100);
		$stat_percent_b = (number_format($stat_percent_b, 4, '.', '') * 100);

		#####################
		#CALCULATE BAR HEIGHT
		#####################
		$bar_height_final_a = $bar_height_vert * $bar_height_percent_a;
		$bar_height_final_b = $bar_height_vert * $bar_height_percent_b;

		$bar_height_a = number_format($bar_height_final_a, 0, '.', '');
		$bar_height_b = number_format($bar_height_final_b, 0, '.', '');

		############################
		#FORCE BAR HEIGHT - FOR SAFE
		############################
		if($bar_height_a > $bar_height_vert) $bar_height_a = $bar_height_vert;
		if($bar_height_b > $bar_height_vert) $bar_height_b = $bar_height_vert;

		#########################################################################################
		#CREATE THE ALT BAR (LIGHT GREY) WHICH WILL BE DISPLAYED ABOVE THE DARK GREY VERTICAL BAR
		#########################################################################################
		$bar_height_alt_a = $bar_height_vert - $bar_height_a;
		$bar_height_alt_b = $bar_height_vert - $bar_height_b;

		##################################
		#PUT SOME MORE TABLE WORK TOGETHER
		##################################

##################
#PREP FINAL WIDTHS
##################
$table_width_px = $table_width . "px";
$bar_width_px = $bar_width . "px";

############
#START IT UP
############
$ajax_return .= <<<AJR
<td valign="bottom" style="text-align:center;">
<table class="stat_chart_table">
<tr>
<td style="padding-right:5px;">
<table class="stat_chart_table" align="left" style="width:$bar_width_px; empty-cells:show; border:1px solid $bar_border_color;">
AJR;

		#################################################
		#CHECK FOR THE ALT (NO DATA) TOP COLOR OF THE BAR
		#################################################
		$bar_height_alt_a_px = $bar_height_alt_a . "px";
		if($bar_height_alt_a > 0) $ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_a_px; background-color:$bar_vert_alt_color;\"></td></tr>";

		#################################
		#CHECK FOR THE BOTTOM 'a' BAR NOW
		#################################
		$bar_height_a_px = $bar_height_a . "px";
		if($bar_height_a > 0) $ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_a_px; background-color:$bar_vert_color_a;\" onMouseOver=\"tooltip.show('$tooltip_a')\" onMouseOut='tooltip.hide()'></td></tr>";

		##########
		#FINISH UP
		##########
		$ajax_return .= "</table>";

		#########
		#VS TABLE
		#########
		$ajax_return .= "<table class=\"stat_chart_table\" align=\"left\" style=\"width:$bar_width_px; empty-cells:show; border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;\">";

		#################################################
		#CHECK FOR THE ALT (NO DATA) TOP COLOR OF THE BAR
		#################################################
		$bar_height_alt_b_px = $bar_height_alt_b . "px";
		if($bar_height_alt_b > 0) $ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_b_px; background-color:$bar_vert_alt_color;\"></td></tr>";

		#################################
		#CHECK FOR THE BOTTOM 'b' BAR NOW
		#################################
		$bar_height_b_px = $bar_height_b . "px";
		if($bar_height_b > 0) $ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_b_px; background-color:$bar_vert_color_b;\" onMouseOver=\"tooltip.show('$tooltip_b')\" onMouseOut='tooltip.hide()'></td></tr>";

		########################
		#COMPLETE BAR TABLE WORK
		########################
		$ajax_return .= "</table>";

########################
#COMPLETE OUR TABLE WORK
########################
$ajax_return .= <<<AJR
</td>
</tr>
<tr>
<td><div class="body_secondary" style="text-align:center; padding:5px 5px 0 0;">$i</div></td>
</tr>
</table>
</td>
AJR;

	}

	#########################
	#COMPLETE OUR CHART TABLE
	#########################
	$ajax_return .= "</tr><tr></table>";

	###################################
	#GET ABBREVIATED MONTHS FOR A AND B
	###################################
	$recap_month_a = date_get_friendly_month($month_a);
	$recap_month_b = date_get_friendly_month($month_b);

	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($vs_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes"; break;
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}

	##############################################
	#DETERMINE RECAP STATS FOR THIS CHARTTING TYPE
	##############################################
	$recap_peak_a = number_format(${'highest_' . $vs_chartting . '_month_a'});
	$recap_total_a = number_format(${'total_' . $vs_chartting . '_month_a'});
	$recap_peak_b = number_format(${'highest_' . $vs_chartting . '_month_b'});
	$recap_total_b = number_format(${'total_' . $vs_chartting . '_month_b'});

	#################################
	#CREATE MONTH A AND B COLOR CUBES
	#################################
	$month_a_cube = "<div style=\"float:left; height:12px; width:12px; background-color:$bar_vert_color_a; margin-right:5px; border:1px solid $bar_border_color;\"></div>";
	$month_b_cube = "<div style=\"float:left; height:12px; width:12px; background-color:$bar_vert_color_b; margin-right:5px; border:1px solid $bar_border_color;\"></div>";

	############################
	#DISPLAY MONTH A RECAP STATS
	############################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_primary\" style=\"float:left; margin:20px 15px 0 0; padding:10px; font-weight:bold;\">";
	$ajax_return .= "$month_a_cube<div style=\"float:left;\"><span class=\"body_highlight\">$recap_month_a&nbsp;$stat_title:&nbsp;</span>$recap_total_a";

	##################################
	#SHOW MONTH A PEAK DAY STAT / LINK
	##################################
	$ajax_return .= "<span class=\"body_secondary\" style=\"font-weight:normal;\">&nbsp;&nbsp;|&nbsp;&nbsp;Peak&nbsp;Day:&nbsp;</span><span class=\"body_primary\">";
	if($recap_peak_a > 0) $ajax_return .= "$recap_peak_a";
	else $ajax_return .= "N/A";
	$ajax_return .= "</span>";

	####################################
	#SHOW THE PEAK DAY ONLY IF IT EXISTS
	####################################
	if(isset(${'highest_' . $vs_chartting . '_day_month_a'}) && isset(${'highest_' . $vs_chartting . '_date_month_a'}))
	{
		#########################
		#MAKE SURE IT HAS A VALUE
		#########################
		if(${'highest_' . $vs_chartting . '_month_a'} > 0)
		{
			###################################
			#ABBREVIATE DATE AND SHOW PEAK LINK
			###################################
			$peak_link_a_abbr = $month_a . "-" . ${'highest_' . $vs_chartting . '_day_month_a'} . "-" . substr($year_a,2,2);
			$ajax_return .= "&nbsp;<span class=\"body_highlight\" style=\"font-weight:normal;\" onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'>(<a href=javascript:retrieve_daily_db_stats('$month_a','${'highest_' . $vs_chartting . '_day_month_a'}','$year_a')>$peak_link_a_abbr</a>)</span>";
		}
	}

	$ajax_return .= "</div></div>";

	############################
	#DISPLAY MONTH B RECAP STATS
	############################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_primary\" style=\"float:left; margin:20px 15px 0 0; padding:10px; font-weight:bold;\">";
	$ajax_return .= "$month_b_cube<div style=\"float:left;\"><span class=\"body_highlight\">$recap_month_b&nbsp;$stat_title:&nbsp;</span>$recap_total_b";

	##################################
	#SHOW MONTH B PEAK DAY STAT / LINK
	##################################
	$ajax_return .= "<span class=\"body_secondary\" style=\"font-weight:normal;\">&nbsp;&nbsp;|&nbsp;&nbsp;Peak&nbsp;Day:&nbsp;</span><span class=\"body_primary\">";
	if($recap_peak_b > 0) $ajax_return .= "$recap_peak_b";
	else $ajax_return .= "N/A";
	$ajax_return .= "</span>";

	####################################
	#SHOW THE PEAK DAY ONLY IF IT EXISTS
	####################################
	if(isset(${'highest_' . $vs_chartting . '_day_month_b'}) && isset(${'highest_' . $vs_chartting . '_date_month_b'}))
	{
		#########################
		#MAKE SURE IT HAS A VALUE
		#########################
		if(${'highest_' . $vs_chartting . '_month_b'} > 0)
		{
			###################################
			#ABBREVIATE DATE AND SHOW PEAK LINK
			###################################
			$peak_link_b_abbr = $month_b . "-" . ${'highest_' . $vs_chartting . '_day_month_b'} . "-" . substr($year_b,2,2);
			$ajax_return .= "&nbsp;<span class=\"body_highlight\" style=\"font-weight:normal;\" onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'>(<a href=javascript:retrieve_daily_db_stats('$month_b','${'highest_' . $vs_chartting . '_day_month_b'}','$year_b')>$peak_link_b_abbr</a>)</span>";
		}
	}

	$ajax_return .= "</div></div>";

	#####################################################
	#CLEAR FLOATS, COMPLETE DIV AND PROCEED TO CORE STATS
	#####################################################
	$ajax_return .= "<div style=\"clear:both;\"></div></div>";

	#################################
	#CONFIGURE OUR PEAK STATS / LINKS
	#################################
	$stat_type_array = array('commits','rollbacks','block_reads','buffer_hits','inserts','updates','deletes','seq_scans','seq_rows','idx_scans','idx_rows');

	####################
	#START OUR ITERATION
	####################
	foreach($stat_type_array as &$stat)
	{
		if(${'highest_' . $stat . '_month_a'} == 0 && ${'highest_' . $stat . '_month_b'} == 0)
		{
			${'peak_' . $stat} = "N/A";
			${'peak_' . $stat . '_day'} = "N/A";
		}
		elseif(${'highest_' . $stat . '_month_a'} >= ${'highest_' . $stat . '_month_b'})
		{
			${'peak_' . $stat} = number_format(${'highest_' . $stat . '_month_a'});
			${'peak_' . $stat . '_day'} = "<span class=\"body_highlight\"><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$month_a','${'highest_' . $stat . '_day_month_a'}','$year_a')>${'highest_' . $stat . '_date_month_a'}</a></span></span>";
		}
		elseif(${'highest_' . $stat . '_month_a'} < ${'highest_' . $stat . '_month_b'})
		{
			${'peak_' . $stat} = number_format(${'highest_' . $stat . '_month_b'});
			${'peak_' . $stat . '_day'} = "<span class=\"body_highlight\"><span onMouseOver=\"tooltip.show('Retrieve&nbsp;Stats')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_daily_db_stats('$month_b','${'highest_' . $stat . '_day_month_b'}','$year_b')>${'highest_' . $stat . '_date_month_b'}</a></span></span>";
		}
	}

	#######################################
	#PREPARE OUR CORE STATISTICS FOR BOTTOM
	#######################################
	$combined_commits = number_format($total_commits_month_a + $total_commits_month_b);
	$combined_peaks = number_format($highest_commits_month_a + $highest_commits_month_b);
	$combined_rollbacks = number_format($total_rollbacks_month_a + $total_rollbacks_month_b);
	$combined_block_reads = number_format($total_block_reads_month_a + $total_block_reads_month_b);
	$combined_buffer_hits = number_format($total_buffer_hits_month_a + $total_buffer_hits_month_b);
	$combined_inserts = number_format($total_inserts_month_a + $total_inserts_month_b);
	$combined_updates = number_format($total_updates_month_a + $total_updates_month_b);
	$combined_deletes = number_format($total_deletes_month_a + $total_deletes_month_b);
	$combined_seq_scans = number_format($total_seq_scans_month_a + $total_seq_scans_month_b);
	$combined_seq_rows = number_format($total_seq_rows_month_a + $total_seq_rows_month_b);
	$combined_idx_scans = number_format($total_idx_scans_month_a + $total_idx_scans_month_b);
	$combined_idx_rows = number_format($total_idx_rows_month_a + $total_idx_rows_month_b);

	################################################################
	#PUT TOGETHER SOME GENERAL STATISTICS AND FORMATTING FOR MONTH A
	################################################################
	$highest_commits_month_a = number_format($highest_commits_month_a);
	$total_commits_month_a = number_format($total_commits_month_a);
	$total_rollbacks_month_a = number_format($total_rollbacks_month_a);
	$total_block_reads_month_a = number_format($total_block_reads_month_a);
	$total_buffer_hits_month_a = number_format($total_buffer_hits_month_a);
	$total_inserts_month_a = number_format($total_inserts_month_a);
	$total_updates_month_a = number_format($total_updates_month_a);
	$total_deletes_month_a = number_format($total_deletes_month_a);
	$total_seq_scans_month_a = number_format($total_seq_scans_month_a);
	$total_seq_rows_month_a = number_format($total_seq_rows_month_a);
	$total_idx_scans_month_a = number_format($total_idx_scans_month_a);
	$total_idx_rows_month_a = number_format($total_idx_rows_month_a);

	################################################################
	#PUT TOGETHER SOME GENERAL STATISTICS AND FORMATTING FOR MONTH B
	################################################################
	$highest_commits_month_b = number_format($highest_commits_month_b);
	$total_commits_month_b = number_format($total_commits_month_b);
	$total_rollbacks_month_b = number_format($total_rollbacks_month_b);
	$total_block_reads_month_b = number_format($total_block_reads_month_b);
	$total_buffer_hits_month_b = number_format($total_buffer_hits_month_b);
	$total_inserts_month_b = number_format($total_inserts_month_b);
	$total_updates_month_b = number_format($total_updates_month_b);
	$total_deletes_month_b = number_format($total_deletes_month_b);
	$total_seq_scans_month_b = number_format($total_seq_scans_month_b);
	$total_seq_rows_month_b = number_format($total_seq_rows_month_b);
	$total_idx_scans_month_b = number_format($total_idx_scans_month_b);
	$total_idx_rows_month_b = number_format($total_idx_rows_month_b);

########################
#DISPLAY CORE STATISTICS
########################
$ajax_return .= <<<AJR
<div class="stat_wrapper">
<table class="ps_core_table body_secondary">
<tr>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Statistic</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">$month_a_cube<div style="float:left;">$month_a_str</div></td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">$month_b_cube<div style="float:left;">$month_b_str</div></td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Combined</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Peak&nbsp;Stat</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Peak&nbsp;Day</td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">Commits</td>
<td>$total_commits_month_a</td>
<td>$total_commits_month_b</td>
<td>$combined_commits</td>
<td>$peak_commits</td>
<td>$peak_commits_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">Rollbacks</td>
<td>$total_rollbacks_month_a</td>
<td>$total_rollbacks_month_b</td>
<td>$combined_rollbacks</td>
<td>$peak_rollbacks</td>
<td>$peak_rollbacks_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">Block&nbsp;Reads</td>
<td>$total_block_reads_month_a</td>
<td>$total_block_reads_month_b</td>
<td>$combined_block_reads</td>
<td>$peak_block_reads</td>
<td>$peak_block_reads_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">Buffer&nbsp;Hits</td>
<td>$total_buffer_hits_month_a</td>
<td>$total_buffer_hits_month_b</td>
<td>$combined_buffer_hits</td>
<td>$peak_buffer_hits</td>
<td>$peak_buffer_hits_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">User&nbsp;Inserts</td>
<td>$total_inserts_month_a</td>
<td>$total_inserts_month_b</td>
<td>$combined_inserts</td>
<td>$peak_inserts</td>
<td>$peak_inserts_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">User&nbsp;Updates</td>
<td>$total_updates_month_a</td>
<td>$total_updates_month_b</td>
<td>$combined_updates</td>
<td>$peak_updates</td>
<td>$peak_updates_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">User&nbsp;Deletes</td>
<td>$total_deletes_month_a</td>
<td>$total_deletes_month_b</td>
<td>$combined_deletes</td>
<td>$peak_deletes</td>
<td>$peak_deletes_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">SEQ&nbsp;Scans</td>
<td>$total_seq_scans_month_a</td>
<td>$total_seq_scans_month_b</td>
<td>$combined_seq_scans</td>
<td>$peak_seq_scans</td>
<td>$peak_seq_scans_day</td>
</tr>
<tr>
<td>Rows&nbsp;Used</td>
<td>$total_seq_rows_month_a</td>
<td>$total_seq_rows_month_b</td>
<td>$combined_seq_rows</td>
<td>$peak_seq_rows</td>
<td>$peak_seq_rows_day</td>
</tr>
<tr>
<td colspan="6" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="font-weight:bold;">IDX&nbsp;Scans</td>
<td>$total_idx_scans_month_a</td>
<td>$total_idx_scans_month_b</td>
<td>$combined_idx_scans</td>
<td>$peak_idx_scans</td>
<td>$peak_idx_scans_day</td>
</tr>
<tr>
<td>Rows&nbsp;Used</td>
<td>$total_idx_rows_month_a</td>
<td>$total_idx_rows_month_b</td>
<td>$combined_idx_rows</td>
<td>$peak_idx_rows</td>
<td>$peak_idx_rows_day</td>
</tr>
</table>
</div>
AJR;

/*
#############################
#GET TOTAL SCRIPT PEAK MEMORY
#############################
$script_mem = number_format(((memory_get_peak_usage(true)/1024)/1024),3) . "M";
$ajax_return .= <<<AJR
<div style="text-align:left; padding-top:10px;">Script Peak Memory: $script_mem</div>
AJR;
*/

	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_WEEKLY_TREND
##########################################################################################################
##########################################################################################################
function retrieve_weekly_trend($postgrestats_dir,$log_month,$log_year,$graphics_src,$weekly_chartting)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($log_month) || empty($log_year))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}

	##################################################################
	#DOUBLE CHECK THE CHARTTING VAR AND MAKE SURE IT HAS A LEGIT VALUE
	##################################################################
	if($weekly_chartting != "commits" && $weekly_chartting != "rollbacks" && $weekly_chartting != "block_reads" && $weekly_chartting != "buffer_hits" && $weekly_chartting != "inserts" && $weekly_chartting != "updates" && $weekly_chartting != "deletes" && $weekly_chartting != "seq_scans" && $weekly_chartting != "idx_scans")
	{
		###########################################
		#DEFAULT THE CHART TO SHOWING COMMIT VALUES
		###########################################
		$weekly_chartting = "commits";
	}

	#############################################################
	#PUT THE LOG FILE NAME TOGETHER AND MAKE SURE THE FILE EXISTS
	#############################################################
	$log_file_name = $log_month;

	###########################
	#MAKE DOUBLE SURE IT EXISTS
	###########################
	if(!file_exists("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name"))
	{
		$stat_msg = "Zero stats have been recorded for this month.";
		echo $stat_msg;
		exit;
	}

	#########################
	#ATTEMPT TO OPEN THE FILE
	#########################
	if(!$open_log = fopen("$postgrestats_dir/postgrestats/data/$log_year/$log_file_name", "r"))
	{
		$stat_msg = "Unable to open this log file.";
		echo $stat_msg;
		exit;
	}

	###########################
	#INITIATE SOME GENERAL VARS
	###########################
	$bar_height_vert = 150;
	$bar_width = 25;

	##########################
	#FOR TRACKING DAYS STATTED
	##########################
	$total_days_tracked = 0;
	$total_days_tracked_sun = 0;
	$total_days_tracked_mon = 0;
	$total_days_tracked_tue = 0;
	$total_days_tracked_wed = 0;
	$total_days_tracked_thu = 0;
	$total_days_tracked_fri = 0;
	$total_days_tracked_sat = 0;

	###############################################
	#VARS WHICH WILL HOUSE THE TOTALS FOR ALL STATS
	###############################################
	$total_commits = 0;
	$total_rollbacks = 0;
	$total_block_reads = 0;
	$total_buffer_hits = 0;
	$total_inserts = 0;
	$total_updates = 0;
	$total_deletes = 0;
	$total_seq_scans = 0;
	$total_seq_rows = 0;
	$total_idx_scans = 0;
	$total_idx_rows = 0;

	#################################################
	#VARS WHICH WILL HOUSE THE HIGHESTS FOR ALL STATS
	#################################################
	$highest_commits = 0;
	$highest_rollbacks = 0;
	$highest_block_reads = 0;
	$highest_buffer_hits = 0;
	$highest_inserts = 0;
	$highest_updates = 0;
	$highest_deletes = 0;
	$highest_seq_scans = 0;
	$highest_seq_rows = 0;
	$highest_idx_scans = 0;
	$highest_idx_rows = 0;

	######################################################################
	#VARS WHICH WILL HOUSE THE TOTAL AND HIGHEST STATS PER DAY OF THE WEEK
	######################################################################
	$day_array = array("sun","mon","tue","wed","thu","fri","sat");

	#####################
	#GET THROUGH THEM NOW
	#####################
	foreach($day_array as &$day)
	{
		${'total_commits_' . $day} = 0;
		${'total_rollbacks_' . $day} = 0;
		${'total_block_reads_' . $day} = 0;
		${'total_buffer_hits_' . $day} = 0;
		${'total_inserts_' . $day} = 0;
		${'total_updates_' . $day} = 0;
		${'total_deletes_' . $day} = 0;
		${'total_seq_scans_' . $day} = 0;
		${'total_seq_rows_' . $day} = 0;
		${'total_idx_scans_' . $day} = 0;
		${'total_idx_rows_' . $day} = 0;

		${'highest_commits_' . $day} = 0;
		${'highest_rollbacks_' . $day} = 0;
		${'highest_block_reads_' . $day} = 0;
		${'highest_buffer_hits_' . $day} = 0;
		${'highest_inserts_' . $day} = 0;
		${'highest_updates_' . $day} = 0;
		${'highest_deletes_' . $day} = 0;

		${'highest_seq_scans_' . $day} = 0;
		${'highest_seq_rows_' . $day} = 0;
		${'highest_idx_scans_' . $day} = 0;
		${'highest_idx_rows_' . $day} = 0;
	}

	#####################
	#BAR CHART COLOR VARS
	#####################
	$bar_vert_alt_color = "#FFFFFF";
	$bar_vert_peak_color = "#FF6633";
	$bar_vert_color = "#7B7B7B";
	$bar_border_color = "#444444";

	#########
	#IUD VARS
	#########
	$iud_insert_color = "#7B7B7B";
	$iud_update_color = "#C8C8C8";
	$iud_delete_color = "#F1F1F1";
	$iud_nodata_color = "#000000";
	$iud_bar_width = 770;
	$iud_bar_height = 20;

	############################################################
	#GO THROUGH THE LOG - LINE BY LINE AND RETRIEVE ALL THE DATA
	############################################################
	while(!feof($open_log))
	{
		#############
		#GET ONE LINE
		#############
		$db_line_single = fgets($open_log);

		###################################
		#MAKE SURE WE HAVE A LINE WITH DATA
		###################################
		if(!empty($db_line_single))
		{
			#####################################################################################
			#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE - GET THE COMMIT TOTALS
			#####################################################################################

			################################################################
			#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
			#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
			################################################################

			if(substr($db_line_single,0,2) == $log_month && substr($db_line_single,6,4) == $log_year)
			{
				#############
				#FLAG THE DAY
				#############
				$day_flag = substr($db_line_single,3,2);

				#################################################
				#FIND OUT WHAT DAY OF THE WEEK THIS DAY IS
				#WILL YIELD: MON, TUE, WED, THU, FRI, SAT AND SUN
				#################################################		
				$date_check = mktime(0, 0, 0, $log_month, $day_flag, $log_year);
				$day_of_week = date('D', $date_check);

				#############
				#EXPLODE LINE
				#############
				$log_line = explode(",",$db_line_single);

				##################################################
				#OBTAIN OUR STATS FROM THIS LINE
				#FORCE TYPE CAST BY ADDING ZERO TO EACH STAT VALUE
				##################################################
				if(!empty($log_line[3])) { $commits = $log_line[3] + 0; } else { $commits = 0; }
				if(!empty($log_line[4])) { $rollbacks = $log_line[4] + 0; } else { $rollbacks = 0; }
				if(!empty($log_line[5])) { $block_reads = $log_line[5] + 0; } else { $block_reads = 0; }
				if(!empty($log_line[6])) { $buffer_hits = $log_line[6] + 0; } else { $buffer_hits = 0; }
				if(!empty($log_line[7])) { $inserts = $log_line[7] + 0; } else { $inserts = 0; }
				if(!empty($log_line[8])) { $updates = $log_line[8] + 0; } else { $updates = 0; }
				if(!empty($log_line[9])) { $deletes = $log_line[9] + 0; } else { $deletes = 0; }
				if(!empty($log_line[10])) { $seq_scans = $log_line[10] + 0; } else { $seq_scans = 0; }
				if(!empty($log_line[11])) { $idx_scans = $log_line[11] + 0; } else { $idx_scans = 0; }
				if(!empty($log_line[12])) { $seq_rows = $log_line[12] + 0; } else { $seq_rows = 0; }
				if(!empty($log_line[13])) { $idx_rows = $log_line[13] + 0; } else { $idx_rows = 0; }

				################
				#TALLY UP TOTALS
				################
				$total_commits += $commits;
				$total_rollbacks += $rollbacks;
				$total_block_reads += $block_reads;
				$total_buffer_hits += $buffer_hits;
				$total_inserts += $inserts;
				$total_updates += $updates;
				$total_deletes += $deletes;
				$total_seq_scans += $seq_scans;
				$total_idx_scans += $idx_scans;
				$total_seq_rows += $seq_rows;
				$total_idx_rows += $idx_rows;

				#############################################################################
				#TALL UP THE DAYS TRACKED, TOTAL COMMIT, INSERTS, UPDATES AND DELETES PER DAY
				#############################################################################
				switch($day_of_week)
				{
					case "Sun":
					$total_commits_sun += $commits;
					$total_rollbacks_sun += $rollbacks;
					$total_block_reads_sun += $block_reads;
					$total_buffer_hits_sun += $buffer_hits;
					$total_inserts_sun += $inserts;
					$total_updates_sun += $updates;
					$total_deletes_sun += $deletes;
					$total_seq_scans_sun += $seq_scans;
					$total_idx_scans_sun += $idx_scans;
					$total_seq_rows_sun += $seq_rows;
					$total_idx_rows_sun += $idx_rows;
					break;

					case "Mon":
					$total_commits_mon += $commits;
					$total_rollbacks_mon += $rollbacks;
					$total_block_reads_mon += $block_reads;
					$total_buffer_hits_mon += $buffer_hits;
					$total_inserts_mon += $inserts;
					$total_updates_mon += $updates;
					$total_deletes_mon += $deletes;
					$total_seq_scans_mon += $seq_scans;
					$total_idx_scans_mon += $idx_scans;
					$total_seq_rows_mon += $seq_rows;
					$total_idx_rows_mon += $idx_rows;
					break;

					case "Tue":
					$total_commits_tue += $commits;
					$total_rollbacks_tue += $rollbacks;
					$total_block_reads_tue += $block_reads;
					$total_buffer_hits_tue += $buffer_hits;
					$total_inserts_tue += $inserts;
					$total_updates_tue += $updates;
					$total_deletes_tue += $deletes;
					$total_seq_scans_tue += $seq_scans;
					$total_idx_scans_tue += $idx_scans;
					$total_seq_rows_tue += $seq_rows;
					$total_idx_rows_tue += $idx_rows;
					break;

					case "Wed":
					$total_commits_wed += $commits;
					$total_rollbacks_wed += $rollbacks;
					$total_block_reads_wed += $block_reads;
					$total_buffer_hits_wed += $buffer_hits;
					$total_inserts_wed += $inserts;
					$total_updates_wed += $updates;
					$total_deletes_wed += $deletes;
					$total_seq_scans_wed += $seq_scans;
					$total_idx_scans_wed += $idx_scans;
					$total_seq_rows_wed += $seq_rows;
					$total_idx_rows_wed += $idx_rows;
					break;

					case "Thu":
					$total_commits_thu += $commits;
					$total_rollbacks_thu += $rollbacks;
					$total_block_reads_thu += $block_reads;
					$total_buffer_hits_thu += $buffer_hits;
					$total_inserts_thu += $inserts;
					$total_updates_thu += $updates;
					$total_deletes_thu += $deletes;
					$total_seq_scans_thu += $seq_scans;
					$total_idx_scans_thu += $idx_scans;
					$total_seq_rows_thu += $seq_rows;
					$total_idx_rows_thu += $idx_rows;
					break;

					case "Fri":
					$total_commits_fri += $commits;
					$total_rollbacks_fri += $rollbacks;
					$total_block_reads_fri += $block_reads;
					$total_buffer_hits_fri += $buffer_hits;
					$total_inserts_fri += $inserts;
					$total_updates_fri += $updates;
					$total_deletes_fri += $deletes;
					$total_seq_scans_fri += $seq_scans;
					$total_idx_scans_fri += $idx_scans;
					$total_seq_rows_fri += $seq_rows;
					$total_idx_rows_fri += $idx_rows;
					break;

					case "Sat":
					$total_commits_sat += $commits;
					$total_rollbacks_sat += $rollbacks;
					$total_block_reads_sat += $block_reads;
					$total_buffer_hits_sat += $buffer_hits;
					$total_inserts_sat += $inserts;
					$total_updates_sat += $updates;
					$total_deletes_sat += $deletes;
					$total_seq_scans_sat += $seq_scans;
					$total_idx_scans_sat += $idx_scans;
					$total_seq_rows_sat += $seq_rows;
					$total_idx_rows_sat += $idx_rows;
				}

				##########################
				#TALLY UP OUR HIGHEST VARS
				##########################
				if($commits > $highest_commits) $highest_commits = $commits;
				if($rollbacks > $highest_rollbacks) $highest_rollbacks = $rollbacks;
				if($block_reads > $highest_block_reads) $highest_block_reads = $block_reads;
				if($buffer_hits > $highest_buffer_hits) $highest_buffer_hits = $buffer_hits;
				if($inserts > $highest_inserts) $highest_inserts = $inserts;
				if($updates > $highest_updates) $highest_updates = $updates;
				if($deletes > $highest_deletes) $highest_deletes = $deletes;
				if($seq_scans > $highest_seq_scans) $highest_seq_scans = $seq_scans;
				if($idx_scans > $highest_idx_scans) $highest_idx_scans = $idx_scans;
				if($seq_rows > $highest_seq_rows) $highest_seq_rows = $seq_rows;
				if($idx_rows > $highest_idx_rows) $highest_idx_rows = $idx_rows;

				#####################################################################################
				#TALLY UP OUR "HIGHEST" VARS PER STAT SO THAT WE CAN DETERMINE BAR CHART VALUES LATER
				#####################################################################################
				switch($day_of_week)
				{
					case "Sun":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_sun) $highest_commits_sun = $commits;
					if($rollbacks > $highest_rollbacks_sun) $highest_rollbacks_sun = $rollbacks;
					if($block_reads > $highest_block_reads_sun) $highest_block_reads_sun = $block_reads;
					if($buffer_hits > $highest_buffer_hits_sun) $highest_buffer_hits_sun = $buffer_hits;
					if($inserts > $highest_inserts_sun) $highest_inserts_sun = $inserts;
					if($updates > $highest_updates_sun) $highest_updates_sun = $updates;
					if($deletes > $highest_deletes_sun) $highest_deletes_sun = $deletes;
					if($seq_scans > $highest_seq_scans_sun) $highest_seq_scans_sun = $seq_scans;
					if($idx_scans > $highest_idx_scans_sun) $highest_idx_scans_sun = $idx_scans;
					if($seq_rows > $highest_seq_rows_sun) $highest_seq_rows_sun = $seq_rows;
					if($idx_rows > $highest_idx_rows_sun) $highest_idx_rows_sun = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_sun++;
					break;

					case "Mon":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_mon) $highest_commits_mon = $commits;
					if($rollbacks > $highest_rollbacks_mon) $highest_rollbacks_mon = $rollbacks;
					if($block_reads > $highest_block_reads_mon) $highest_block_reads_mon = $block_reads;
					if($buffer_hits > $highest_buffer_hits_mon) $highest_buffer_hits_mon = $buffer_hits;
					if($inserts > $highest_inserts_mon) $highest_inserts_mon = $inserts;
					if($updates > $highest_updates_mon) $highest_updates_mon = $updates;
					if($deletes > $highest_deletes_mon) $highest_deletes_mon = $deletes;
					if($seq_scans > $highest_seq_scans_mon) $highest_seq_scans_mon = $seq_scans;
					if($idx_scans > $highest_idx_scans_mon) $highest_idx_scans_mon = $idx_scans;
					if($seq_rows > $highest_seq_rows_mon) $highest_seq_rows_mon = $seq_rows;
					if($idx_rows > $highest_idx_rows_mon) $highest_idx_rows_mon = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_mon++;
					break;

					case "Tue":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_tue) $highest_commits_tue = $commits;
					if($rollbacks > $highest_rollbacks_tue) $highest_rollbacks_tue = $rollbacks;
					if($block_reads > $highest_block_reads_tue) $highest_block_reads_tue = $block_reads;
					if($buffer_hits > $highest_buffer_hits_tue) $highest_buffer_hits_tue = $buffer_hits;
					if($inserts > $highest_inserts_tue) $highest_inserts_tue = $inserts;
					if($updates > $highest_updates_tue) $highest_updates_tue = $updates;
					if($deletes > $highest_deletes_tue) $highest_deletes_tue = $deletes;
					if($seq_scans > $highest_seq_scans_tue) $highest_seq_scans_tue = $seq_scans;
					if($idx_scans > $highest_idx_scans_tue) $highest_idx_scans_tue = $idx_scans;
					if($seq_rows > $highest_seq_rows_tue) $highest_seq_rows_tue = $seq_rows;
					if($idx_rows > $highest_idx_rows_tue) $highest_idx_rows_tue = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_tue++;
					break;

					case "Wed":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_wed) $highest_commits_wed = $commits;
					if($rollbacks > $highest_rollbacks_wed) $highest_rollbacks_wed = $rollbacks;
					if($block_reads > $highest_block_reads_wed) $highest_block_reads_wed = $block_reads;
					if($buffer_hits > $highest_buffer_hits_wed) $highest_buffer_hits_wed = $buffer_hits;
					if($inserts > $highest_inserts_wed) $highest_inserts_wed = $inserts;
					if($updates > $highest_updates_wed) $highest_updates_wed = $updates;
					if($deletes > $highest_deletes_wed) $highest_deletes_wed = $deletes;
					if($seq_scans > $highest_seq_scans_wed) $highest_seq_scans_wed = $seq_scans;
					if($idx_scans > $highest_idx_scans_wed) $highest_idx_scans_wed = $idx_scans;
					if($seq_rows > $highest_seq_rows_wed) $highest_seq_rows_wed = $seq_rows;
					if($idx_rows > $highest_idx_rows_wed) $highest_idx_rows_wed = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_wed++;
					break;

					case "Thu":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_thu) $highest_commits_thu = $commits;
					if($rollbacks > $highest_rollbacks_thu) $highest_rollbacks_thu = $rollbacks;
					if($block_reads > $highest_block_reads_thu) $highest_block_reads_thu = $block_reads;
					if($buffer_hits > $highest_buffer_hits_thu) $highest_buffer_hits_thu = $buffer_hits;
					if($inserts > $highest_inserts_thu) $highest_inserts_thu = $inserts;
					if($updates > $highest_updates_thu) $highest_updates_thu = $updates;
					if($deletes > $highest_deletes_thu) $highest_deletes_thu = $deletes;
					if($seq_scans > $highest_seq_scans_thu) $highest_seq_scans_thu = $seq_scans;
					if($idx_scans > $highest_idx_scans_thu) $highest_idx_scans_thu = $idx_scans;
					if($seq_rows > $highest_seq_rows_thu) $highest_seq_rows_thu = $seq_rows;
					if($idx_rows > $highest_idx_rows_thu) $highest_idx_rows_thu = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_thu++;
					break;

					case "Fri":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_fri) $highest_commits_fri = $commits;
					if($rollbacks > $highest_rollbacks_fri) $highest_rollbacks_fri = $rollbacks;
					if($block_reads > $highest_block_reads_fri) $highest_block_reads_fri = $block_reads;
					if($buffer_hits > $highest_buffer_hits_fri) $highest_buffer_hits_fri = $buffer_hits;
					if($inserts > $highest_inserts_fri) $highest_inserts_fri = $inserts;
					if($updates > $highest_updates_fri) $highest_updates_fri = $updates;
					if($deletes > $highest_deletes_fri) $highest_deletes_fri = $deletes;
					if($seq_scans > $highest_seq_scans_fri) $highest_seq_scans_fri = $seq_scans;
					if($idx_scans > $highest_idx_scans_fri) $highest_idx_scans_fri = $idx_scans;
					if($seq_rows > $highest_seq_rows_fri) $highest_seq_rows_fri = $seq_rows;
					if($idx_rows > $highest_idx_rows_fri) $highest_idx_rows_fri = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_fri++;
					break;

					case "Sat":
					#####################
					#HANDLE HIGHEST STATS
					#####################
					if($commits > $highest_commits_sat) $highest_commits_sat = $commits;
					if($rollbacks > $highest_rollbacks_sat) $highest_rollbacks_sat = $rollbacks;
					if($block_reads > $highest_block_reads_sat) $highest_block_reads_sat = $block_reads;
					if($buffer_hits > $highest_buffer_hits_sat) $highest_buffer_hits_sat = $buffer_hits;
					if($inserts > $highest_inserts_sat) $highest_inserts_sat = $inserts;
					if($updates > $highest_updates_sat) $highest_updates_sat = $updates;
					if($deletes > $highest_deletes_sat) $highest_deletes_sat = $deletes;
					if($seq_scans > $highest_seq_scans_sat) $highest_seq_scans_sat = $seq_scans;
					if($idx_scans > $highest_idx_scans_sat) $highest_idx_scans_sat = $idx_scans;
					if($seq_rows > $highest_seq_rows_sat) $highest_seq_rows_sat = $seq_rows;
					if($idx_rows > $highest_idx_rows_sat) $highest_idx_rows_sat = $idx_rows;

					############################
					#TALLY UP TOTAL TRACKED DAYS
					############################
					$total_days_tracked_sat++;
				}

				####################################################
				#TALLY UP TOTAL DAYS TRACKED REGARDLESS OF STAT TYPE
				####################################################
				$total_days_tracked++;
			}	
		}
	}

	#############################
	#CLOSE UP THE FILE - ALL DONE
	#############################
	fclose($open_log);

################################
#START CREATING OUR INITIAL HTML
################################
$ajax_return = <<<AJR
<div class="stat_wrapper" style="margin:10px 0 25px 0;">
<table class="stat_chart_table" style="width:auto;">
<tr>
AJR;

	#######################################################################
	#GET THE HIGHEST OF THIS TYPE SO WE CAN CREATE OUR BAR PERCENTAGE LATER
	#######################################################################
	$peak_stat = 0;
	$peak_day = "";

	########################
	#THE ARRAY FOR ITERATION
	########################
	$day_array = array("sun","mon","tue","wed","thu","fri","sat");
	
	#####################
	#GET THROUGH THEM NOW
	#####################
	foreach($day_array as &$day)
	{
		if(${'total_' . $weekly_chartting . '_' . $day} > $peak_stat)
		{
			###########################################
			#FLAG THE HIGHEST PEAK STAT AND DAY AS WELL
			###########################################
			$peak_stat = ${'total_' . $weekly_chartting . '_' . $day};
			$peak_day = $day;
		}
	}
	
	#########################################################################
	#PUT TOGETHER THE MONTH TABLEWORK NOW - ITERATING THROUGH ALL OF OUR VARS
	#########################################################################
	
	###############################
	#ITERATE THROUGH EACH STAT TYPE
	###############################
	foreach($day_array as &$day)
	{
		################################
		#INITIATE/RESET VARS IN THE LOOP
		################################
		$tooltip = "";
		$commit_percent = 0;
		$bar_height_percent = 0;
		$bar_height = 0;
		$bar_height_final = 0;
		$bar_height_alt = 0;

		##################################
		#FIRST PREP OUR COMMIT TOOLTIP VAR
		##################################
		$tt_commits = ${'total_commits_' . $day};
		if($tt_commits == 0 || empty($tt_commits)) $tt_commits = "0";
		else $tt_commits = number_format($tt_commits);
		
		###############################
		#PREP OUR ROLLBACKS TOOLTIP VAR
		###############################
		$tt_rollbacks = ${'total_rollbacks_' . $day};
		if($tt_rollbacks == 0 || empty($tt_rollbacks)) $tt_rollbacks = "0";
		else $tt_rollbacks = number_format($tt_rollbacks);
		
		#################################
		#PREP OUR BLOCK READS TOOLTIP VAR
		#################################
		$tt_block_reads = ${'total_block_reads_' . $day};
		if($tt_block_reads == 0 || empty($tt_block_reads)) $tt_block_reads = "0";
		else $tt_block_reads = number_format($tt_block_reads);
		
		#################################
		#PREP OUR BUFFER HITS TOOLTIP VAR
		#################################
		$tt_buffer_hits = ${'total_buffer_hits_' . $day};
		if($tt_buffer_hits == 0 || empty($tt_buffer_hits)) $tt_buffer_hits = "0";
		else $tt_buffer_hits = number_format($tt_buffer_hits);
		
		############################
		#PREP OUR INSERT TOOLTIP VAR
		############################
		$tt_inserts = ${'total_inserts_' . $day};
		if($tt_inserts == 0 || empty($tt_inserts)) $tt_inserts = "0";
		else $tt_inserts = number_format($tt_inserts);
		
		############################
		#PREP OUR UPDATE TOOLTIP VAR
		############################
		$tt_updates = ${'total_updates_' . $day};
		if($tt_updates == 0 || empty($tt_updates)) $tt_updates = "0";
		else $tt_updates = number_format($tt_updates);
		
		############################
		#PREP OUR DELETE TOOLTIP VAR
		############################
		$tt_deletes = ${'total_deletes_' . $day};
		if($tt_deletes == 0 || empty($tt_deletes)) $tt_deletes = "0";
		else $tt_deletes = number_format($tt_deletes);
		
		####################################
		#PREP OUR SEQUENCE SCANS TOOLTIP VAR
		####################################
		$tt_seq_scans = ${'total_seq_scans_' . $day};
		if($tt_seq_scans == 0 || empty($tt_seq_scans)) $tt_seq_scans = "0";
		else $tt_seq_scans = number_format($tt_seq_scans);
		
		#################################
		#PREP OUR INDEX SCANS TOOLTIP VAR
		############################
		$tt_idx_scans = ${'total_idx_scans_' . $day};
		if($tt_idx_scans == 0 || empty($tt_idx_scans)) $tt_idx_scans = "0";
		else $tt_idx_scans = number_format($tt_idx_scans);
				
		################
		#PUT IT TOGETHER
		################
		$tooltip = "<div style=line-height:17px;>";

		########
		#COMMITS
		########
		if($weekly_chartting == "commits") $tooltip .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits . "</strong></span><br>";
		else $tooltip .= "Commits:&nbsp;<strong>" . $tt_commits . "</strong>" . "<br>";

		##########
		#ROLLBACKS
		##########
		if($weekly_chartting == "rollbacks") $tooltip .= "<span class=body_highlight><strong>Rollbacks:&nbsp;" . $tt_rollbacks . "</strong></span><br>";
		else $tooltip .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks . "</strong>" . "<br>";

		############
		#BLOCK READS
		############
		if($weekly_chartting == "block_reads") $tooltip .= "<span class=body_highlight><strong>Block&nbsp;Reads:&nbsp;" . $tt_block_reads . "</strong></span><br>";
		else $tooltip .= "Block&nbsp;Reads:&nbsp;<strong>" . $tt_block_reads . "</strong>" . "<br>";

		############
		#BUFFER HITS
		############
		if($weekly_chartting == "buffer_hits") $tooltip .= "<span class=body_highlight><strong>Buffer&nbsp;Hits:&nbsp;" . $tt_buffer_hits . "</strong></span><br>";
		else $tooltip .= "Buffer&nbsp;Hits:&nbsp;<strong>" . $tt_buffer_hits . "</strong>" . "<br>";

		########
		#INSERTS
		########
		if($weekly_chartting == "inserts") $tooltip .= "<span class=body_highlight><strong>Inserts:&nbsp;" . $tt_inserts . "</strong></span><br>";
		else $tooltip .= "Inserts:&nbsp;<strong>" . $tt_inserts . "</strong>" . "<br>";

		########
		#UPDATES
		########
		if($weekly_chartting == "updates") $tooltip .= "<span class=body_highlight><strong>Updates:&nbsp;" . $tt_updates . "</strong></span><br>";
		else $tooltip .= "Updates:&nbsp;<strong>" . $tt_updates . "</strong>" . "<br>";

		########
		#DELETES
		########
		if($weekly_chartting == "deletes") $tooltip .= "<span class=body_highlight><strong>Deletes:&nbsp;" . $tt_deletes . "</strong></span><br>";
		else $tooltip .= "Deletes:&nbsp;<strong>" . $tt_deletes . "</strong>" . "<br>";

		##########
		#SEQ SCANS
		##########
		if($weekly_chartting == "seq_scans") $tooltip .= "<span class=body_highlight><strong>Sequence&nbsp;Scans:&nbsp;" . $tt_seq_scans . "</strong></span><br>";
		else $tooltip .= "Sequence&nbsp;Scans:&nbsp;<strong>" . $tt_seq_scans . "</strong>" . "<br>";

		##########
		#IDX SCANS
		##########
		if($weekly_chartting == "idx_scans") $tooltip .= "<span class=body_highlight><strong>Index&nbsp;Scans:&nbsp;" . $tt_idx_scans . "</strong></span><br>";
		else $tooltip .= "Index&nbsp;Scans:&nbsp;<strong>" . $tt_idx_scans . "</strong>" . "<br>";

		#########################
		#COMPLETE THE TOOLTIP DIV
		#########################
		$tooltip .= "</div>";

		#############################################################
		#GET THE PERCENTAGE NOW SO WE CAN PUT OUR BAR / DATA TOGETHER
		#############################################################
		if(!empty($peak_stat))
		{
			$stat_percent = ${'total_' . $weekly_chartting . '_' . $day} / $peak_stat;
			$bar_height_percent = $stat_percent;
		}
		else
		{
			$bar_height_percent = 0;
		}
		
		#################################
		#COMPLETE OUR MATH FOR THE HEIGHT
		#################################
		$bar_height = $bar_height_vert * $bar_height_percent;
		$bar_height_final = number_format($bar_height, 0, '.', '');
		
		############################
		#CHECK THE HEIGHT - FOR SAFE
		############################
		if($bar_height_final > $bar_height_vert) $bar_height_final = $bar_height_vert;
		
		####################################################################################
		#CREATE THE HEIGHT FOR THE ALT LIGHT GREY BAR WHICH SITS ON TOP OF THE DARK GREY BAR
		####################################################################################
		$bar_height_alt = $bar_height_vert - $bar_height_final;
		
		###########################################################
		#HANDLE THE FINAL SOURCE GRAPHIC AND CHECK FOR THE PEAK DAY
		###########################################################
		if($peak_stat == ${'total_' . $weekly_chartting . '_' . $day}) $bar_vert_color_final = $bar_vert_peak_color;
		else $bar_vert_color_final = $bar_vert_color;
		$ajax_return .= "<td valign=\"bottom\" style=\"text-align:center; padding-right:5px;\">";

		#############################
		#CREATE FINAL CELL DIMENSIONS
		#############################
		$bar_width_px = $bar_width . "px";

		#######################################################
		#START UP THE TABLEWORK WHICH WILL HOUSE THE BAR ITSELF
		#######################################################
		$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$bar_width_px; empty-cells:show;\">";

		############################
		#DETERMINE BORDER STYLES NOW
		############################
		$bar_border = "border:1px solid $bar_border_color;";
		if($bar_height_final <= 0) $bar_alt_border = "border:1px solid $bar_border_color;";
		else $bar_alt_border = "border-color: $bar_border_color; border-width: 1px 1px 0px 1px; border-style: solid;";
		
		#####################################################################################
		#HANDLE THE ALT COLOR - WILL EITHER BE THE ENTIRE BAR (NO DATA) OR THE TOP OF THE BAR
		#####################################################################################
		if($bar_height_alt > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_alt_px = $bar_height_alt . "px";

			###########
			#DISPLAY IT
			###########
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_px; background-color:$bar_vert_alt_color; $bar_alt_border\"></td></tr>";
		}
		
		################################################
		#HANDLE THE BOTTOM OF THE BAR WHERE WE HAVE DATA
		################################################
		if(isset(${'total_' . $weekly_chartting . '_' . $day}) && ${'total_' . $weekly_chartting . '_' . $day} > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_final_px = $bar_height_final . "px";

			###########
			#DISPLAY IT
			###########
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_final_px; background-color:$bar_vert_color_final; $bar_border\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'></td></tr>";
		}

		###################
		#COMPLETE TABLEWORK
		###########	########
		$ajax_return .= "</table>";
		
		################################################
		#FINISHED WITH THE BAR TABLE - CONTINUE OUR HTML
		################################################
		$ajax_return .= "</td>";
	}

	###############################################################################################
	#DETERMINE THE TOTAL STATS FOR THIS TYPE - THIS WILL BE DISPLAYED TO THE RIGHT OF THE BAR CHART
	###############################################################################################
	if(!empty(${'total_' . $weekly_chartting . '_' . $peak_day})) { $recap_peak = ${'total_' . $weekly_chartting . '_' . $peak_day}; } else { $recap_peak = 0; }
	if(!empty($recap_peak) && !empty(${'total_days_tracked_' . $peak_day})) { $recap_avg_on_peak = $recap_peak / ${'total_days_tracked_' . $peak_day}; } else { $recap_avg_on_peak = 0; }

	################
	#FORMAT THE VARS
	################
	$recap_peak = number_format($recap_peak);
	$recap_total = number_format(${'total_' . $weekly_chartting});
	$recap_avg_on_peak = number_format($recap_avg_on_peak, 2, '.', ',');

	#####################################
	#CREATE A NICE TITLE FOR THE PEAK DAY
	#####################################
	$recap_peak_day = "";
	
	switch($peak_day)
	{
		case "sun": $recap_peak_day = "Sunday"; break;
		case "mon": $recap_peak_day = "Monday"; break;
		case "tue": $recap_peak_day = "Tuesday"; break;
		case "wed": $recap_peak_day = "Wednesday"; break;
		case "thu": $recap_peak_day = "Thursday"; break;
		case "fri": $recap_peak_day = "Friday"; break;
		case "sat": $recap_peak_day = "Saturday";
	}

	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($weekly_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes"; break;
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}

############################
#PUT THE TABLE WORK TOGETHER
############################
$ajax_return .= <<<AJR
</tr>
<tr>
AJR;

	###############################################################################################
	#CREATE THE MONTH NUMBERS AND IF THERE ARE COMMITS FOR EACH DAY - MAKE A QUICK LINK TO THAT DAY
	###############################################################################################
	for($i=1; $i<8; $i++)
	{	
		####################################################
		#PREPARE A DAY LABEL FOR THE BOTTOM OF THE BAR CHART
		####################################################
		switch($i)
		{
			case 1: $day_label = "Sun"; break;
			case 2: $day_label = "Mon"; break;
			case 3: $day_label = "Tue"; break;
			case 4: $day_label = "Wed"; break;
			case 5: $day_label = "Thu"; break;
			case 6: $day_label = "Fri"; break;
			case 7: $day_label = "Sat";
		}

		$ajax_return .= "<td class=\"body_secondary\" style=\"text-align:center; padding:5px 5px 0 0;\">";
		$ajax_return .= $day_label;
		$ajax_return .= "</td>";
	}

	#################################
	#COMPLETE THE CHART ROW AND TABLE
	#################################
	$ajax_return .= "</tr></table>";

	########################################################################
	#DETERMINE THE TOTAL STATS FOR THIS TYPE - FOR DISPLAY IN THE RECAP AREA
	########################################################################
	if(!empty(${'total_' . $weekly_chartting . '_' . $peak_day})) { $recap_peak = ${'total_' . $weekly_chartting . '_' . $peak_day}; } else { $recap_peak = 0; }
	if(!empty($recap_peak) && !empty(${'total_days_tracked_' . $peak_day})) { $recap_avg_on_peak = $recap_peak / ${'total_days_tracked_' . $peak_day}; } else { $recap_avg_on_peak = 0; }

	################
	#FORMAT THE VARS
	################
	$recap_peak = number_format($recap_peak);
	$recap_total = number_format(${'total_' . $weekly_chartting});
	$recap_avg_on_peak = number_format($recap_avg_on_peak, 2, '.', ',');

	#####################################
	#CREATE A NICE TITLE FOR THE PEAK DAY
	#####################################
	$recap_peak_day = "";

	switch($peak_day)
	{
		case "sun": $recap_peak_day = "Sunday"; break;
		case "mon": $recap_peak_day = "Monday"; break;
		case "tue": $recap_peak_day = "Tuesday"; break;
		case "wed": $recap_peak_day = "Wednesday"; break;
		case "thu": $recap_peak_day = "Thursday"; break;
		case "fri": $recap_peak_day = "Friday"; break;
		case "sat": $recap_peak_day = "Saturday";
	}

	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($weekly_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes"; break;
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}

	####################
	#RECAP TARGET TOTALS
	####################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_highlight\" style=\"float:left; margin:20px 15px 0 0; padding:10px; font-weight:bold;\">";
	$ajax_return .= "Total&nbsp;$stat_title:&nbsp;<span class=\"body_primary\">$recap_total</span></div>";

	#################
	#PEAK DAY AND SUM
	#################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Peak&nbsp;Day&nbsp;/&nbsp;Sum:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">";
	if($recap_peak > 0) $ajax_return .= "$recap_peak_day&nbsp;/&nbsp;$recap_peak";
	else $ajax_return .= "N/A";
	$ajax_return .= "</span></div>";

	####################
	#AVERAGE ON PEAK DAY
	####################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
	if($recap_peak > 0) $ajax_return .= "Average&nbsp;/&nbsp;$recap_peak_day:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">$recap_avg_on_peak</span>";
	else $ajax_return .= "Average:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">N/A</span>";
	$ajax_return .= "</div>";

	########################################
	#CLEAR FLOATS AND COMPLETE THE UPPER DIV
	########################################
	$ajax_return .= "<div style=\"clear:both;\"></div>";
	$ajax_return .= "</div>";

############################################
#PREPARE THE INSERT UPDATE DELETE QUICK VIEW
############################################
$ajax_return .= <<<AJR
<div class="stat_wrapper" style="margin:0 0 25px 0;">
<div class="body_highlight" style="margin-right:10px; float:left; font-weight:bold;">Insert&nbsp;/&nbsp;Update&nbsp;/&nbsp;Delete&nbsp;Quick&nbsp;View:&nbsp;</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_insert_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Inserts</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_update_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Updates</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_delete_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Deletes</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_nodata_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">No&nbsp;Data</div>
<div style="clear:both;"></div>
AJR;

	##############################################################
	#PUT TOGETHER OUR IUD STATS - FIRST INITIATE SOME GENERAL VARS
	##############################################################
	$total_iud = $total_inserts + $total_updates + $total_deletes;
	
	##############################################
	#GET THE PERCENTAGES OF EACH AGAINST THE TOTAL
	##############################################
	if(!empty($total_iud)) { $iud_ins_percent = number_format($total_inserts / $total_iud, 2, '.', ','); } else { $iud_ins_percent = 0; }
	if(!empty($total_iud)) { $iud_upd_percent = number_format($total_updates / $total_iud, 2, '.', ','); } else { $iud_upd_percent = 0; }
	if(!empty($total_iud)) { $iud_del_percent = number_format($total_deletes / $total_iud, 2, '.', ','); } else { $iud_del_percent = 0; }
	
	#################################################
	#PREPARE THE BAR WIDTHS AND ROUND THE NUMBERS OUT
	#################################################
	$iud_ins_bar_w = number_format($iud_bar_width * $iud_ins_percent);
	$iud_upd_bar_w = number_format($iud_bar_width * $iud_upd_percent);
	$iud_del_bar_w = number_format($iud_bar_width * $iud_del_percent);
	
	#######################################################################################################
	#TALLY UP THE WIDTHS AND TAKE ANY NOMINAL REMAINDER +/- IUD_BAR_WIDTH AND +/- IT FROM THE LARGEST SCORE
	#######################################################################################################
	$iud_bar_total = $iud_ins_bar_w + $iud_upd_bar_w + $iud_del_bar_w;
	
	if($iud_bar_total < $iud_bar_width && $iud_bar_total > 0) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_width-$iud_bar_total;
		
		############################
		#ADD IT TO HIGHEST IUD SCORE
		############################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w += $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w += $iud_remain;
		else $iud_del_bar_w += $iud_remain;
	}
	elseif($iud_bar_total > $iud_bar_width) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_total-$iud_bar_width;
		
		###################################
		#SUBTRACT IT FROM HIGHEST IUD SCORE
		###################################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w -= $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w -= $iud_remain;
		else $iud_del_bar_w -= $iud_remain;
	}
	
	##################################
	#PUT TOGETHER OUR IUD TOOLTIP VARS
	##################################
	$iud_tt_ins = "<div style=line-height:12px;>";
	$iud_tt_ins .= "Total Inserts:&nbsp;<strong class=body_highlight>" . number_format($total_inserts) . "</strong>";
	$iud_tt_ins .= "</div>";
	
	$iud_tt_upd = "<div style=line-height:12px;>";
	$iud_tt_upd .= "Total Updates:&nbsp;<strong class=body_highlight>" . number_format($total_updates) . "</strong>";
	$iud_tt_upd .= "</div>";
	
	$iud_tt_del = "<div style=line-height:12px;>";
	$iud_tt_del .= "Total Deletes:&nbsp;<strong class=body_highlight>" . number_format($total_deletes) . "</strong>";
	$iud_tt_del .= "</div>";
	
	###########################################################################################
	#SHOW EITHER THE THREE IUD BARS TOGETHER OR THE DEFAULT NO-DATA BAR IF THERE IS NO IUD DATA
	###########################################################################################

	################################
	#PREP SOME FINAL CELL DIMENSIONS
	################################
	$iud_bar_width_px = $iud_bar_width . "px";
	$iud_bar_height_px = $iud_bar_height . "px";

	####################################
	#START WITH OUR BEGINNING TABLE WORK
	####################################
	$ajax_return .= "<div style=\"margin-top:10px;\">";
	$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$iud_bar_width_px; empty-cells:show;\">";
	$ajax_return .= "<tr>";

	if($total_iud > 0)
	{
		###########
		#INSERT BAR
		###########
		if($iud_ins_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			$ins_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_ins_bar_w_px = $iud_ins_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_ins_bar_w_px; height:$iud_bar_height_px; background-color:#7B7B7B; $ins_border\" onMouseOver=\"tooltip.show('$iud_tt_ins')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#UPDATE BAR
		###########
		if($iud_upd_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			if($iud_ins_bar_w > 0) $upd_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $upd_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_upd_bar_w_px = $iud_upd_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_upd_bar_w_px; height:$iud_bar_height_px; background-color:#C9C9C9; $upd_border\" onMouseOver=\"tooltip.show('$iud_tt_upd')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#DELETE BAR
		###########
		if($iud_del_bar_w > 0) 
		{
			############
			#PREP BORDER
			############
			if($iud_upd_bar_w > 0 || $iud_ins_bar_w > 0) $del_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $del_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_del_bar_w_px = $iud_del_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_del_bar_w_px; height:$iud_bar_height_px; background-color:#F1F1F1; $del_border\" onMouseOver=\"tooltip.show('$iud_tt_del')\" onMouseOut='tooltip.hide()'></td>";
		}
	}
	else
	{
		##################################
		#DEFAULT THE NO-DATA BAR AND WIDTH
		##################################
		$ajax_return .= "<td style=\"width:$iud_bar_width_px; height:$iud_bar_height_px; background-color:#000000; border:1px solid $bar_border_color;\" onMouseOver=\"tooltip.show('No data to report.')\" onMouseOut='tooltip.hide()'></td>";
	}

	##############################################
	#COMPLETE OUR HTML TABLE WORK FOR THE IUD AREA
	##############################################
	$ajax_return .= "</tr>";
	$ajax_return .= "</table>";
	$ajax_return .= "</div></div>";

	###########################################################
	#GET THE BOTTOM TABLE OF STATS TOGETHER - START UP THE HTML
	#GET OUR HIGHEST SUM STATS PRIOR TO FORMATTING BELOW
	###########################################################
	$highest_sum_commits = $highest_commits_sun + $highest_commits_mon + $highest_commits_tue + $highest_commits_wed + $highest_commits_thu + $highest_commits_fri + $highest_commits_sat;
	$highest_sum_rollbacks = $highest_rollbacks_sun + $highest_rollbacks_mon + $highest_rollbacks_tue + $highest_rollbacks_wed + $highest_rollbacks_thu + $highest_rollbacks_fri + $highest_rollbacks_sat;
	$highest_sum_block_reads = $highest_block_reads_sun + $highest_block_reads_mon + $highest_block_reads_tue + $highest_block_reads_wed + $highest_block_reads_thu + $highest_block_reads_fri + $highest_block_reads_sat;
	$highest_sum_buffer_hits = $highest_buffer_hits_sun + $highest_buffer_hits_mon + $highest_buffer_hits_tue + $highest_buffer_hits_wed + $highest_buffer_hits_thu + $highest_buffer_hits_fri + $highest_buffer_hits_sat;
	$highest_sum_inserts = $highest_inserts_sun + $highest_inserts_mon + $highest_inserts_tue + $highest_inserts_wed + $highest_inserts_thu + $highest_inserts_fri + $highest_inserts_sat;
	$highest_sum_updates = $highest_updates_sun + $highest_updates_mon + $highest_updates_tue + $highest_updates_wed + $highest_updates_thu + $highest_updates_fri + $highest_updates_sat;
	$highest_sum_deletes = $highest_deletes_sun + $highest_deletes_mon + $highest_deletes_tue + $highest_deletes_wed + $highest_deletes_thu + $highest_deletes_fri + $highest_deletes_sat;
	$highest_sum_seq_scans = $highest_seq_scans_sun + $highest_seq_scans_mon + $highest_seq_scans_tue + $highest_seq_scans_wed + $highest_seq_scans_thu + $highest_seq_scans_fri + $highest_seq_scans_sat;
	$highest_sum_idx_scans = $highest_idx_scans_sun + $highest_idx_scans_mon + $highest_idx_scans_tue + $highest_idx_scans_wed + $highest_idx_scans_thu + $highest_idx_scans_fri + $highest_idx_scans_sat;
	$highest_sum_seq_rows = $highest_seq_rows_sun + $highest_seq_rows_mon + $highest_seq_rows_tue + $highest_seq_rows_wed + $highest_seq_rows_thu + $highest_seq_rows_fri + $highest_seq_rows_sat;
	$highest_sum_idx_rows = $highest_idx_rows_sun + $highest_idx_rows_mon + $highest_idx_rows_tue + $highest_idx_rows_wed + $highest_idx_rows_thu + $highest_idx_rows_fri + $highest_idx_rows_sat;

	###########################################################
	#FORMAT OUR TOTAL AND HIGHEST STAT VARS FOR THE TABLE BELOW
	#CREATE AN ARRAY OF THE DAYS FIRST SO WE CAN ITERATE
	###########################################################
	$day_array = array("sun","mon","tue","wed","thu","fri","sat");

	#####################
	#GET THROUGH THEM NOW
	#####################
	foreach($day_array as &$day)
	{
		${'total_commits_' . $day} = number_abbr(${'total_commits_' . $day},1);
		${'total_rollbacks_' . $day} = number_abbr(${'total_rollbacks_' . $day},1);
		${'total_block_reads_' . $day} = number_abbr(${'total_block_reads_' . $day},1);
		${'total_buffer_hits_' . $day} = number_abbr(${'total_buffer_hits_' . $day},1);
		${'total_inserts_' . $day} = number_abbr(${'total_inserts_' . $day},1);
		${'total_updates_' . $day} = number_abbr(${'total_updates_' . $day},1);
		${'total_deletes_' . $day} = number_abbr(${'total_deletes_' . $day},1);
		${'total_seq_scans_' . $day} = number_abbr(${'total_seq_scans_' . $day},1);
		${'total_idx_scans_' . $day} = number_abbr(${'total_idx_scans_' . $day},1);
		${'total_seq_rows_' . $day} = number_abbr(${'total_seq_rows_' . $day},1);
		${'total_idx_rows_' . $day} = number_abbr(${'total_idx_rows_' . $day},1);
		
		${'highest_commits_' . $day} = number_abbr(${'highest_commits_' . $day},1);
		${'highest_rollbacks_' . $day} = number_abbr(${'highest_rollbacks_' . $day},1);
		${'highest_block_reads_' . $day} = number_abbr(${'highest_block_reads_' . $day},1);
		${'highest_buffer_hits_' . $day} = number_abbr(${'highest_buffer_hits_' . $day},1);
		${'highest_inserts_' . $day} = number_abbr(${'highest_inserts_' . $day},1);
		${'highest_updates_' . $day} = number_abbr(${'highest_updates_' . $day},1);
		${'highest_deletes_' . $day} = number_abbr(${'highest_deletes_' . $day},1);
		${'highest_seq_scans_' . $day} = number_abbr(${'highest_seq_scans_' . $day},1);
		${'highest_idx_scans_' . $day} = number_abbr(${'highest_idx_scans_' . $day},1);
		${'highest_seq_rows_' . $day} = number_abbr(${'highest_seq_rows_' . $day},1);
		${'highest_idx_rows_' . $day} = number_abbr(${'highest_idx_rows_' . $day},1);
	}

	#######################################
	#FORMAT TOTAL VARS AND HIGHEST SUM VARS
	#######################################
	$total_commits = number_abbr($total_commits,3);
	$total_rollbacks = number_abbr($total_rollbacks,3);
	$total_block_reads = number_abbr($total_block_reads,3);
	$total_buffer_hits = number_abbr($total_buffer_hits,3);
	$total_inserts = number_abbr($total_inserts,3);
	$total_updates = number_abbr($total_updates,3);
	$total_deletes = number_abbr($total_deletes,3);
	$total_seq_scans = number_abbr($total_seq_scans,3);
	$total_idx_scans = number_abbr($total_idx_scans,3);
	$total_seq_rows = number_abbr($total_seq_rows,3);
	$total_idx_rows = number_abbr($total_idx_rows,3);

	#####################
	#FORMAT OUR PEAK SUMS
	#####################
	$highest_sum_commits = number_abbr($highest_sum_commits,3);
	$highest_sum_rollbacks = number_abbr($highest_sum_rollbacks,3);
	$highest_sum_block_reads = number_abbr($highest_sum_block_reads,3);
	$highest_sum_buffer_hits = number_abbr($highest_sum_buffer_hits,3);
	$highest_sum_inserts = number_abbr($highest_sum_inserts,3);
	$highest_sum_updates = number_abbr($highest_sum_updates,3);
	$highest_sum_deletes = number_abbr($highest_sum_deletes,3);
	$highest_sum_seq_scans = number_abbr($highest_sum_seq_scans,3);
	$highest_sum_idx_scans = number_abbr($highest_sum_idx_scans,3);
	$highest_sum_seq_rows = number_abbr($highest_sum_seq_rows,3);
	$highest_sum_idx_rows = number_abbr($highest_sum_idx_rows,3);

####################################
#PREPARE CORE STATISTICS TABLE BELOW
####################################
$ajax_return .= <<<AJR
<div class="stat_wrapper">
<table class="ps_core_table body_secondary">
<tr>
<td>&nbsp;</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Sunday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Monday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Tuesday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Wednesday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Thursday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Friday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Saturday</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Total</td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Commits</td>
<td class="body_highlight">$total_commits_sun</td>
<td class="body_highlight">$total_commits_mon</td>
<td class="body_highlight">$total_commits_tue</td>
<td class="body_highlight">$total_commits_wed</td>
<td class="body_highlight">$total_commits_thu</td>
<td class="body_highlight">$total_commits_fri</td>
<td class="body_highlight">$total_commits_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_commits</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_commits_sun</td>
<td>$highest_commits_mon</td>
<td>$highest_commits_tue</td>
<td>$highest_commits_wed</td>
<td>$highest_commits_thu</td>
<td>$highest_commits_fri</td>
<td>$highest_commits_sat</td>
<td>$highest_sum_commits</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Rollbacks</td>
<td class="body_highlight">$total_rollbacks_sun</td>
<td class="body_highlight">$total_rollbacks_mon</td>
<td class="body_highlight">$total_rollbacks_tue</td>
<td class="body_highlight">$total_rollbacks_wed</td>
<td class="body_highlight">$total_rollbacks_thu</td>
<td class="body_highlight">$total_rollbacks_fri</td>
<td class="body_highlight">$total_rollbacks_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_rollbacks</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_rollbacks_sun</td>
<td>$highest_rollbacks_mon</td>
<td>$highest_rollbacks_tue</td>
<td>$highest_rollbacks_wed</td>
<td>$highest_rollbacks_thu</td>
<td>$highest_rollbacks_fri</td>
<td>$highest_rollbacks_sat</td>
<td>$highest_sum_rollbacks</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Block&nbsp;Reads</td>
<td class="body_highlight">$total_block_reads_sun</td>
<td class="body_highlight">$total_block_reads_mon</td>
<td class="body_highlight">$total_block_reads_tue</td>
<td class="body_highlight">$total_block_reads_wed</td>
<td class="body_highlight">$total_block_reads_thu</td>
<td class="body_highlight">$total_block_reads_fri</td>
<td class="body_highlight">$total_block_reads_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_block_reads</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_block_reads_sun</td>
<td>$highest_block_reads_mon</td>
<td>$highest_block_reads_tue</td>
<td>$highest_block_reads_wed</td>
<td>$highest_block_reads_thu</td>
<td>$highest_block_reads_fri</td>
<td>$highest_block_reads_sat</td>
<td>$highest_sum_block_reads</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Buffer&nbsp;Hits</td>
<td class="body_highlight">$total_buffer_hits_sun</td>
<td class="body_highlight">$total_buffer_hits_mon</td>
<td class="body_highlight">$total_buffer_hits_tue</td>
<td class="body_highlight">$total_buffer_hits_wed</td>
<td class="body_highlight">$total_buffer_hits_thu</td>
<td class="body_highlight">$total_buffer_hits_fri</td>
<td class="body_highlight">$total_buffer_hits_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_buffer_hits</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_buffer_hits_sun</td>
<td>$highest_buffer_hits_mon</td>
<td>$highest_buffer_hits_tue</td>
<td>$highest_buffer_hits_wed</td>
<td>$highest_buffer_hits_thu</td>
<td>$highest_buffer_hits_fri</td>
<td>$highest_buffer_hits_sat</td>
<td>$highest_sum_buffer_hits</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Inserts</td>
<td class="body_highlight">$total_inserts_sun</td>
<td class="body_highlight">$total_inserts_mon</td>
<td class="body_highlight">$total_inserts_tue</td>
<td class="body_highlight">$total_inserts_wed</td>
<td class="body_highlight">$total_inserts_thu</td>
<td class="body_highlight">$total_inserts_fri</td>
<td class="body_highlight">$total_inserts_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_inserts</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_inserts_sun</td>
<td>$highest_inserts_mon</td>
<td>$highest_inserts_tue</td>
<td>$highest_inserts_wed</td>
<td>$highest_inserts_thu</td>
<td>$highest_inserts_fri</td>
<td>$highest_inserts_sat</td>
<td>$highest_sum_inserts</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Updates</td>
<td class="body_highlight">$total_updates_sun</td>
<td class="body_highlight">$total_updates_mon</td>
<td class="body_highlight">$total_updates_tue</td>
<td class="body_highlight">$total_updates_wed</td>
<td class="body_highlight">$total_updates_thu</td>
<td class="body_highlight">$total_updates_fri</td>
<td class="body_highlight">$total_updates_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_updates</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_updates_sun</td>
<td>$highest_updates_mon</td>
<td>$highest_updates_tue</td>
<td>$highest_updates_wed</td>
<td>$highest_updates_thu</td>
<td>$highest_updates_fri</td>
<td>$highest_updates_sat</td>
<td>$highest_sum_updates</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Deletes</td>
<td class="body_highlight">$total_deletes_sun</td>
<td class="body_highlight">$total_deletes_mon</td>
<td class="body_highlight">$total_deletes_tue</td>
<td class="body_highlight">$total_deletes_wed</td>
<td class="body_highlight">$total_deletes_thu</td>
<td class="body_highlight">$total_deletes_fri</td>
<td class="body_highlight">$total_deletes_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_deletes</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_deletes_sun</td>
<td>$highest_deletes_mon</td>
<td>$highest_deletes_tue</td>
<td>$highest_deletes_wed</td>
<td>$highest_deletes_thu</td>
<td>$highest_deletes_fri</td>
<td>$highest_deletes_sat</td>
<td>$highest_sum_deletes</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">SEQ&nbsp;Scans</td>
<td class="body_highlight">$total_seq_scans_sun</td>
<td class="body_highlight">$total_seq_scans_mon</td>
<td class="body_highlight">$total_seq_scans_tue</td>
<td class="body_highlight">$total_seq_scans_wed</td>
<td class="body_highlight">$total_seq_scans_thu</td>
<td class="body_highlight">$total_seq_scans_fri</td>
<td class="body_highlight">$total_seq_scans_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_seq_scans</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_seq_scans_sun</td>
<td>$highest_seq_scans_mon</td>
<td>$highest_seq_scans_tue</td>
<td>$highest_seq_scans_wed</td>
<td>$highest_seq_scans_thu</td>
<td>$highest_seq_scans_fri</td>
<td>$highest_seq_scans_sat</td>
<td>$highest_sum_seq_scans</td>
</tr>
<tr>
<td class="body_highlight" style="padding:10px 10px 0 0; font-weight:bold;">Rows&nbsp;Used</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_sun</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_mon</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_tue</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_wed</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_thu</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_fri</td>
<td class="body_highlight" style="padding-top:10px;">$total_seq_rows_sat</td>
<td class="body_highlight" style="padding-top:10px; font-weight:bold;">$total_seq_rows</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_seq_rows_sun</td>
<td>$highest_seq_rows_mon</td>
<td>$highest_seq_rows_tue</td>
<td>$highest_seq_rows_wed</td>
<td>$highest_seq_rows_thu</td>
<td>$highest_seq_rows_fri</td>
<td>$highest_seq_rows_sat</td>
<td>$highest_sum_seq_rows</td>
</tr>
<tr>
<td colspan="9" style="height:5px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">IDX&nbsp;Scans</td>
<td class="body_highlight">$total_idx_scans_sun</td>
<td class="body_highlight">$total_idx_scans_mon</td>
<td class="body_highlight">$total_idx_scans_tue</td>
<td class="body_highlight">$total_idx_scans_wed</td>
<td class="body_highlight">$total_idx_scans_thu</td>
<td class="body_highlight">$total_idx_scans_fri</td>
<td class="body_highlight">$total_idx_scans_sat</td>
<td class="body_highlight" style="font-weight:bold;">$total_idx_scans</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_idx_scans_sun</td>
<td>$highest_idx_scans_mon</td>
<td>$highest_idx_scans_tue</td>
<td>$highest_idx_scans_wed</td>
<td>$highest_idx_scans_thu</td>
<td>$highest_idx_scans_fri</td>
<td>$highest_idx_scans_sat</td>
<td>$highest_sum_idx_scans</td>
</tr>
<tr>
<td class="body_highlight" style="padding:10px 10px 0 0; font-weight:bold;">Rows&nbsp;Used</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_sun</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_mon</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_tue</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_wed</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_thu</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_fri</td>
<td class="body_highlight" style="padding-top:10px;">$total_idx_rows_sat</td>
<td class="body_highlight" style="padding-top:10px; font-weight:bold;">$total_idx_rows</td>
</tr>
<tr>
<td class="body_secondary" style="padding-right:10px;">Peak&nbsp;Day</td>
<td>$highest_idx_rows_sun</td>
<td>$highest_idx_rows_mon</td>
<td>$highest_idx_rows_tue</td>
<td>$highest_idx_rows_wed</td>
<td>$highest_idx_rows_thu</td>
<td>$highest_idx_rows_fri</td>
<td>$highest_idx_rows_sat</td>
<td>$highest_sum_idx_rows</td>
</tr>
</table>
</div>
AJR;

/*
#############################
#GET TOTAL SCRIPT PEAK MEMORY
#############################
$script_mem = number_format(((memory_get_peak_usage(true)/1024)/1024),3) . "M";
$ajax_return .= <<<AJR
<div style="text-align:left; padding-top:10px;">Script Peak Memory: $script_mem</div>
AJR;
*/

	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}

##########################################################################################################
##########################################################################################################
#FUNCTION: RETRIEVE_YEARLY_SNAPSHOT
##########################################################################################################
##########################################################################################################
function retrieve_yearly_snapshot($postgrestats_dir,$log_year,$graphics_src,$year_chartting)
{
	######################
	#INITIALIZE RETURN VAR
	######################
	$stat_msg = "";

	##################################
	#CHECK TO MAKE SURE OUR VARS EXIST
	##################################
	if(empty($log_year))
	{
		$stat_msg = "Error retrieving log file parameters.";
		echo $stat_msg;
		exit;
	}
	
	##################################################################
	#DOUBLE CHECK THE CHARTTING VAR AND MAKE SURE IT HAS A LEGIT VALUE
	##################################################################
	if($year_chartting != "commits" && $year_chartting != "rollbacks" && $year_chartting != "block_reads" && $year_chartting != "buffer_hits" && $year_chartting != "inserts" && $year_chartting != "updates" && $year_chartting != "deletes" && $year_chartting != "seq_scans" && $year_chartting != "idx_scans")
	{
		###########################################
		#DEFAULT THE CHART TO SHOWING COMMIT VALUES
		###########################################
		$year_chartting = "commits";
	}
	
	#################################################################################################
	#CHECK TO SEE THIS YEAR DATA DIRECTORY EXISTS AND THAT THERE IS AT LEAST ONE MONTH OF DATA INSIDE
	#################################################################################################
	if(!file_exists("$postgrestats_dir/postgrestats/data/$log_year/"))
	{
		$stat_msg = "Zero stats have been recorded for this year.";
		echo $stat_msg;
		exit;
	}
	else
	{
		###############################################################################
		#SET UP SOME DEFAULT VARS AND PREPARE TO START OUT ITERATION THROUGH THE MONTHS
		###############################################################################
		$stats_available = false;
		$bar_height_vert = 150;
		$bar_width = 25;

		#####################
		#BAR CHART COLOR VARS
		#####################
		$bar_vert_alt_color = "#FFFFFF";
		$bar_vert_peak_color = "#FF6633";
		$bar_vert_color = "#7B7B7B";
		$bar_border_color = "#444444";

		#########
		#IUD VARS
		#########
		$iud_insert_color = "#7B7B7B";
		$iud_update_color = "#C8C8C8";
		$iud_delete_color = "#F1F1F1";
		$iud_nodata_color = "#000000";
		$iud_bar_width = 770;
		$iud_bar_height = 20;
		$iud_inserts_change = "";
		$iud_updates_change = "";
		$iud_deletes_change = "";

		#####################
		#VARS FOR TOTAL STATS
		#####################
		$total_commits = 0;
		$total_rollbacks = 0;
		$total_block_reads = 0;
		$total_buffer_hits = 0;
		$total_inserts = 0;
		$total_updates = 0;
		$total_deletes = 0;
		$total_seq_scans = 0;
		$total_idx_scans = 0;
		$total_seq_rows = 0;
		$total_idx_rows = 0;

		################################################
		#VARS WHICH WILL TRACK THE HIGHEST STAT PER TYPE
		################################################
		$total_months_tracked = 0;
		$highest_commits = 0;
		$highest_commits_month = "";
		$highest_commits_date = "";

		$highest_rollbacks = 0;
		$highest_rollbacks_month = "";
		$highest_rollbacks_date = "";

		$highest_block_reads = 0;
		$highest_block_reads_month = "";
		$highest_block_reads_date = "";

		$highest_buffer_hits = 0;
		$highest_buffer_hits_month = "";
		$highest_buffer_hits_date = "";

		$highest_inserts = 0;
		$highest_inserts_month = "";
		$highest_inserts_date = "";

		$highest_updates = 0;
		$highest_updates_month = "";
		$highest_updates_date = "";

		$highest_deletes = 0;
		$highest_deletes_month = "";
		$highest_deletes_date = "";

		$highest_seq_scans = 0;
		$highest_seq_scans_month = "";
		$highest_seq_scans_date = "";

		$highest_idx_scans = 0;
		$highest_idx_scans_month = "";
		$highest_idx_scans_date = "";

		$highest_seq_rows = 0;
		$highest_seq_rows_month = "";
		$highest_seq_rows_date = "";

		$highest_idx_rows = 0;
		$highest_idx_rows_month = "";
		$highest_idx_rows_date = "";

		#######################
		#START UP OUR ITERATION
		#######################
		for($i=1; $i<13; $i++)
		{
			#####################
			#HANDLE LEADING ZEROS
			#####################
			if(strlen($i) == 1) $i = "0$i";
			
			####################################
			#INITIALIZE STAT VARS FOR EACH MONTH
			####################################			
			${'commits_' . $i} = 0;
			${'rollbacks_' . $i} = 0;
			${'block_reads_' . $i} = 0;
			${'buffer_hits_' . $i} = 0;
			${'inserts_' . $i} = 0;
			${'updates_' . $i} = 0;
			${'deletes_' . $i} = 0;
			${'seq_scans_' . $i} = 0;
			${'idx_scans_' . $i} = 0;
			${'seq_rows_' . $i} = 0;
			${'idx_rows_' . $i} = 0;

			$file_check = "$postgrestats_dir/postgrestats/data/$log_year/$i";
			
			##############################################################################
			#IF MONTLY STATS EXIST, ATTEMPT TO OPEN THE FILE AND OBTAIN SOME GENERAL STATS
			##############################################################################
			if(file_exists($file_check))
			{
				$stats_available = true;
				
				if(!$open_log = fopen($file_check, "r"))
				{
					$stat_msg = "Unable to open this log file.";
					echo $stat_msg;
					exit;
				}
				else
				{
					############################################################
					#GO THROUGH THE LOG - LINE BY LINE AND RETRIEVE ALL THE DATA
					############################################################
					while(!feof($open_log))
					{
						#############
						#GET ONE LINE
						#############
						$db_line_single = fgets($open_log);
						
						###################################
						#MAKE SURE WE HAVE A LINE WITH DATA
						###################################
						if(!empty($db_line_single))
						{
							#####################################################################################
							#SEARCH FOR THE FIRST LINE IN EACH DAY BLOCK - USING THE DATE - GET THE COMMIT TOTALS
							#####################################################################################
							
							################################################################
							#SAMPLE FIRST LINE FROM EVERY LOG FILE GROUP
							#09,30,2010,25824,17,25767,2291361,37,14,6,21385,337,4666697,337
							################################################################
							
							if(substr($db_line_single,5,6) == ",$log_year,")
							{
								###############
								#FLAG THE MONTH
								###############
								$month_flag = $i;
								
								##############################################
								#EXPLODE LINE AND GET THE COMMITS FOR THIS DAY
								##############################################
								$log_line = explode(",",$db_line_single);
								
								#################
								#GET SOME FIGURES
								#################
								$log_line = explode(",",$db_line_single);
								if(!empty($log_line[3])) { $commits = $log_line[3]; } else { $commits = 0; }
								if(!empty($log_line[4])) { $rollbacks = $log_line[4]; } else { $rollbacks = 0; }
								if(!empty($log_line[5])) { $block_reads = $log_line[5]; } else { $block_reads = 0; }
								if(!empty($log_line[6])) { $buffer_hits = $log_line[6]; } else { $buffer_hits = 0; }
								if(!empty($log_line[7])) { $inserts = $log_line[7]; } else { $inserts = 0; }
								if(!empty($log_line[8])) { $updates = $log_line[8]; } else { $updates = 0; }
								if(!empty($log_line[9])) { $deletes = $log_line[9]; } else { $deletes = 0; }
								if(!empty($log_line[10])) { $seq_scans = $log_line[10]; } else { $seq_scans = 0; }
								if(!empty($log_line[11])) { $idx_scans = $log_line[11]; } else { $idx_scans = 0; }
								if(!empty($log_line[12])) { $seq_rows = $log_line[12]; } else { $seq_rows = 0; }
								if(!empty($log_line[13])) { $idx_rows = $log_line[13]; } else { $idx_rows = 0; }
								
								###########################################
								#ASSIGN THEM TO OUR DYNAMIC MONTH VARIABLES
								###########################################
								${'commits_' . $month_flag} += $commits;
								${'rollbacks_' . $month_flag} += $rollbacks;
								${'block_reads_' . $month_flag} += $block_reads;
								${'buffer_hits_' . $month_flag} += $buffer_hits;
								${'inserts_' . $month_flag} += $inserts;
								${'updates_' . $month_flag} += $updates;
								${'deletes_' . $month_flag} += $deletes;
								${'seq_scans_' . $month_flag} += $seq_scans;
								${'idx_scans_' . $month_flag} += $idx_scans;
								${'seq_rows_' . $month_flag} += $seq_rows;
								${'idx_rows_' . $month_flag} += $idx_rows;
								
								################
								#TALLY UP TOTALS
								################
								$total_commits += $commits;
								$total_rollbacks += $rollbacks;
								$total_block_reads += $block_reads;
								$total_buffer_hits += $buffer_hits;
								$total_inserts += $inserts;
								$total_updates += $updates;
								$total_deletes += $deletes;
								$total_seq_scans += $seq_scans;
								$total_idx_scans += $idx_scans;
								$total_seq_rows += $seq_rows;
								$total_idx_rows += $idx_rows;
							}
						}	
					}
					
					##################
					#CLOSE THIS LOG UP
					##################
					fclose($open_log);
					
					##################################################
					#TALLY UP THE HIGHEST COMMIT AND SET THE PEAK VARS
					##################################################
					if(${'commits_' . $month_flag} > $highest_commits)
					{
						$highest_commits = ${'commits_' . $month_flag};
						$highest_commits_month = $month_flag;
						$highest_commits_date = "$month_flag-$log_year";
					}
					
					####################################
					#TALLY UP THE REST OF OUR PEAK STATS
					####################################
					if(${'rollbacks_' . $month_flag} > $highest_rollbacks)
					{
						$highest_rollbacks = ${'rollbacks_' . $month_flag};
						$highest_rollbacks_month = $month_flag;
						$highest_rollbacks_date = "$month_flag-$log_year";
					}
					
					if(${'block_reads_' . $month_flag} > $highest_block_reads)
					{
						$highest_block_reads = ${'block_reads_' . $month_flag};
						$highest_block_reads_month = $month_flag;
						$highest_block_reads_date = "$month_flag-$log_year";
					}
					
					if(${'buffer_hits_' . $month_flag} > $highest_buffer_hits)
					{
						$highest_buffer_hits = ${'buffer_hits_' . $month_flag};
						$highest_buffer_hits_month = $month_flag;
						$highest_buffer_hits_date = "$month_flag-$log_year";
					}
					
					if(${'inserts_' . $month_flag} > $highest_inserts)
					{
						$highest_inserts = ${'inserts_' . $month_flag};
						$highest_inserts_month = $month_flag;
						$highest_inserts_date = "$month_flag-$log_year";
					}
					
					if(${'updates_' . $month_flag} > $highest_updates)
					{
						$highest_updates = ${'updates_' . $month_flag};
						$highest_updates_month = $month_flag;
						$highest_updates_date = "$month_flag-$log_year";
					}
					
					if(${'deletes_' . $month_flag} > $highest_deletes)
					{
						$highest_deletes = ${'deletes_' . $month_flag};
						$highest_deletes_month = $month_flag;
						$highest_deletes_date = "$month_flag-$log_year";
					}
					if(${'seq_scans_' . $month_flag} > $highest_seq_scans)
					{
						$highest_seq_scans = ${'seq_scans_' . $month_flag};
						$highest_seq_scans_month = $month_flag;
						$highest_seq_scans_date = "$month_flag-$log_year";
					}
					if(${'idx_scans_' . $month_flag} > $highest_idx_scans)
					{
						$highest_idx_scans = ${'idx_scans_' . $month_flag};
						$highest_idx_scans_month = $month_flag;
						$highest_idx_scans_date = "$month_flag-$log_year";
					}
					if(${'seq_rows_' . $month_flag} > $highest_seq_rows)
					{
						$highest_seq_rows = ${'seq_rows_' . $month_flag};
						$highest_seq_rows_month = $month_flag;
						$highest_seq_rows_date = "$month_flag-$log_year";
					}
					if(${'idx_rows_' . $month_flag} > $highest_idx_rows)
					{
						$highest_idx_rows = ${'idx_rows_' . $month_flag};
						$highest_idx_rows_month = $month_flag;
						$highest_idx_rows_date = "$month_flag-$log_year";
					}

					################################################
					#TALLY UP THE TOTAL MONTHS TRACKED TO THIS POINT
					################################################
					$total_months_tracked++;
				}
			}
		}
		
		#####################################
		#SEND THEM BACK IF THERE ARE NO STATS
		#####################################
		if($stats_available == false)
		{
			$stat_msg = "Zero stats have been recorded for this year.";
			echo $stat_msg;
			exit;
		}
	}
	
################################
#START CREATING OUR INITIAL HTML
################################
$ajax_return = <<<AJR
<div class="stat_wrapper" style="margin:10px 0 25px 0;">
<table class="stat_chart_table" style="width:auto;">
<tr>
AJR;
	
	#####################################################################################
	#PUT TOGETHER THE MONTH TABLEWORK - ITERATING THROUGH OUR DB VARS - CHECKING EACH DAY
	#####################################################################################
	for($i=1; $i<13; $i++)
	{
		################################
		#INITIATE/RESET VARS IN THE LOOP
		################################
		$tooltip = "";
		$bar_height_percent = 0;
		$bar_height = 0;
		$bar_height_final = 0;
		$bar_height_alt = 0;
		
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";
		
		#####################
		#CHECK EACH MONTH NOW
		#####################
		if(!isset(${"commits_$i"}))
		{
			##########################################################################
			#HANDLE DAYS WITH ZERO COMMITS - DEFAULT THE LIGHT GREY BAR HEIGHT TO 100%
			##########################################################################
			$bar_height_alt = $bar_height_vert;
		}
		else
		{
			########################
			#DAYS WITH COMMIT VALUES
			########################
			
			##################################
			#FIRST PREP OUR COMMIT TOOLTIP VAR
			##################################
			$tt_commits = ${"commits_$i"};
			if($tt_commits == 0 || empty($tt_commits)) $tt_commits = "0";
			else $tt_commits = number_format($tt_commits);
			
			###############################
			#PREP OUR ROLLBACKS TOOLTIP VAR
			###############################
			$tt_rollbacks = ${"rollbacks_$i"};
			if($tt_rollbacks == 0 || empty($tt_rollbacks)) $tt_rollbacks = "0";
			else $tt_rollbacks = number_format($tt_rollbacks);
			
			#################################
			#PREP OUR BLOCK READS TOOLTIP VAR
			#################################
			$tt_block_reads = ${"block_reads_$i"};
			if($tt_block_reads == 0 || empty($tt_block_reads)) $tt_block_reads = "0";
			else $tt_block_reads = number_format($tt_block_reads);
			
			#################################
			#PREP OUR BUFFER HITS TOOLTIP VAR
			#################################
			$tt_buffer_hits = ${"buffer_hits_$i"};
			if($tt_buffer_hits == 0 || empty($tt_buffer_hits)) $tt_buffer_hits = "0";
			else $tt_buffer_hits = number_format($tt_buffer_hits);

			############################
			#PREP OUR INSERT TOOLTIP VAR
			############################
			$tt_inserts = ${"inserts_$i"};
			if($tt_inserts == 0 || empty($tt_inserts)) $tt_inserts = "0";
			else $tt_inserts = number_format($tt_inserts);
			
			############################
			#PREP OUR UPDATE TOOLTIP VAR
			############################
			$tt_updates = ${"updates_$i"};
			if($tt_updates == 0 || empty($tt_updates)) $tt_updates = "0";
			else $tt_updates = number_format($tt_updates);
			
			############################
			#PREP OUR DELETE TOOLTIP VAR
			############################
			$tt_deletes = ${"deletes_$i"};
			if($tt_deletes == 0 || empty($tt_deletes)) $tt_deletes = "0";
			else $tt_deletes = number_format($tt_deletes);

			####################################
			#PREP OUR SEQUENCE SCANS TOOLTIP VAR
			####################################
			$tt_seq_scans = ${"seq_scans_$i"};
			if($tt_seq_scans == 0 || empty($tt_seq_scans)) $tt_seq_scans = "0";
			else $tt_seq_scans = number_format($tt_seq_scans);
			
			#################################
			#PREP OUR INDEX SCANS TOOLTIP VAR
			#################################
			$tt_idx_scans = ${"idx_scans_$i"};
			if($tt_idx_scans == 0 || empty($tt_idx_scans)) $tt_idx_scans = "0";
			else $tt_idx_scans = number_format($tt_idx_scans);
			
			################
			#PUT IT TOGETHER
			################
			$tooltip = "<div style=line-height:17px;>";

			########
			#COMMITS
			########
			if($year_chartting == "commits") $tooltip .= "<span class=body_highlight><strong>Commits:&nbsp;" . $tt_commits . "</strong></span><br>";
			else $tooltip .= "Commits:&nbsp;<strong>" . $tt_commits . "</strong>" . "<br>";

			##########
			#ROLLBACKS
			##########
			if($year_chartting == "rollbacks") $tooltip .= "<span class=body_highlight><strong>Rollbacks:&nbsp;" . $tt_rollbacks . "</strong></span><br>";
			else $tooltip .= "Rollbacks:&nbsp;<strong>" . $tt_rollbacks . "</strong>" . "<br>";

			############
			#BLOCK READS
			############
			if($year_chartting == "block_reads") $tooltip .= "<span class=body_highlight><strong>Block&nbsp;Reads:&nbsp;" . $tt_block_reads . "</strong></span><br>";
			else $tooltip .= "Block&nbsp;Reads:&nbsp;<strong>" . $tt_block_reads . "</strong>" . "<br>";

			############
			#BUFFER HITS
			############
			if($year_chartting == "buffer_hits") $tooltip .= "<span class=body_highlight><strong>Buffer&nbsp;Hits:&nbsp;" . $tt_buffer_hits . "</strong></span><br>";
			else $tooltip .= "Buffer&nbsp;Hits:&nbsp;<strong>" . $tt_buffer_hits . "</strong>" . "<br>";

			########
			#INSERTS
			########
			if($year_chartting == "inserts") $tooltip .= "<span class=body_highlight><strong>Inserts:&nbsp;" . $tt_inserts . "</strong></span><br>";
			else $tooltip .= "Inserts:&nbsp;<strong>" . $tt_inserts . "</strong>" . "<br>";

			########
			#UPDATES
			########
			if($year_chartting == "updates") $tooltip .= "<span class=body_highlight><strong>Updates:&nbsp;" . $tt_updates . "</strong></span><br>";
			else $tooltip .= "Updates:&nbsp;<strong>" . $tt_updates . "</strong>" . "<br>";

			########
			#DELETES
			########
			if($year_chartting == "deletes") $tooltip .= "<span class=body_highlight><strong>Deletes:&nbsp;" . $tt_deletes . "</strong></span><br>";
			else $tooltip .= "Deletes:&nbsp;<strong>" . $tt_deletes . "</strong>" . "<br>";

			##########
			#SEQ SCANS
			##########
			if($year_chartting == "seq_scans") $tooltip .= "<span class=body_highlight><strong>Sequence&nbsp;Scans:&nbsp;" . $tt_seq_scans . "</strong></span><br>";
			else $tooltip .= "Sequence&nbsp;Scans:&nbsp;<strong>" . $tt_seq_scans . "</strong>" . "<br>";

			##########
			#IDX SCANS
			##########
			if($year_chartting == "idx_scans") $tooltip .= "<span class=body_highlight><strong>Index&nbsp;Scans:&nbsp;" . $tt_idx_scans . "</strong></span><br>";
			else $tooltip .= "Index&nbsp;Scans:&nbsp;<strong>" . $tt_idx_scans . "</strong>" . "<br>";

			#########################
			#COMPLETE THE TOOLTIP DIV
			#########################
			$tooltip .= "</div>";

			#######################################################
			#GET THE COMMIT PERCENT BASED ON WHICH STAT IS SELECTED
			#######################################################
			if(!empty($year_chartting) && !empty(${$year_chartting . '_' . $i})) { $stat_percent = ${$year_chartting . '_' . $i} / ${'highest_' . $year_chartting}; } else { $stat_percent = 0; }
			$bar_height_percent = $stat_percent;
			$stat_percent = (number_format($stat_percent, 4, '.', '') * 100);
			$bar_height = $bar_height_vert * $bar_height_percent;
			$bar_height_final = number_format($bar_height, 0, '.', '');
			
			############################
			#CHECK THE HEIGHT - FOR SAFE
			############################
			if($bar_height_final > $bar_height_vert) $bar_height_final = $bar_height_vert;
			
			####################################################################################
			#CREATE THE HEIGHT FOR THE ALT LIGHT GREY BAR WHICH SITS ON TOP OF THE DARK GREY BAR
			####################################################################################
			$bar_height_alt = $bar_height_vert - $bar_height_final;
			
			#############################################################################
			#HANDLE THE FINAL SOURCE GRAPHIC AND CHECK FOR THE PEAK DAY - FLAGGED EARLIER
			#############################################################################
			if($i == ${'highest_' . $year_chartting . "_month"}) $bar_vert_color_final = $bar_vert_peak_color;
			else $bar_vert_color_final = $bar_vert_color;
		}

		$ajax_return .= "<td valign=\"bottom\" style=\"text-align:center; padding-right:5px;\">";

		#############################
		#CREATE FINAL CELL DIMENSIONS
		#############################
		$bar_width_px = $bar_width . "px";

		#######################################################
		#START UP THE TABLEWORK WHICH WILL HOUSE THE BAR ITSELF
		#######################################################
		$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$bar_width_px; empty-cells:show;\">";

		############################
		#DETERMINE BORDER STYLES NOW
		############################
		$bar_border = "border:1px solid $bar_border_color;";
		if($bar_height_final <= 0) $bar_alt_border = "border:1px solid $bar_border_color;";
		else $bar_alt_border = "border-color: $bar_border_color; border-width: 1px 1px 0px 1px; border-style: solid;";
		
		#####################################################################################
		#HANDLE THE ALT COLOR - WILL EITHER BE THE ENTIRE BAR (NO DATA) OR THE TOP OF THE BAR
		#####################################################################################
		if($bar_height_alt > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_alt_px = $bar_height_alt . "px";

			###########
			#DISPLAY IT
			###########
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_alt_px; background-color:$bar_vert_alt_color; $bar_alt_border\"></td></tr>";
		}
		
		################################################
		#HANDLE THE BOTTOM OF THE BAR WHERE WE HAVE DATA
		################################################
		if($bar_height_final > 0)
		{
			#############################
			#CREATE FINAL CELL DIMENSIONS
			#############################
			$bar_height_final_px = $bar_height_final . "px";

			###########
			#DISPLAY IT
			###########
			$ajax_return .= "<tr><td style=\"width:$bar_width_px; height:$bar_height_final_px; background-color:$bar_vert_color_final; $bar_border\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'></td></tr>";
		}

		$ajax_return .= "</table>";
		
		################################################
		#FINISHED WITH THE BAR TABLE - CONTINUE OUR HTML
		################################################
		$ajax_return .= "</td>";
	}
	
	###############################################################################################
	#DETERMINE THE TOTAL STATS FOR THIS TYPE - THIS WILL BE DISPLAYED TO THE RIGHT OF THE BAR CHART
	###############################################################################################
	$recap_average_month = ${'total_' . $year_chartting} / $total_months_tracked;
	$recap_average_month = number_format($recap_average_month, 0, '', ',');
	$recap_peak = number_format(${'highest_' . $year_chartting}, 0, '', ',');
	$recap_total = number_format(${'total_' . $year_chartting});
	
	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($year_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes";
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}
	
	################################################################
	#PARSE THE PEAK MONTH DATE SO WE CAN LINK IT TO THE MONTHLY VIEW
	################################################################
	$peak_month = substr(${'highest_' . $year_chartting . '_date'},0,2);
	$peak_year = $log_year;
	$tooltip = "Retrieve Stats";
	
########################################
#THE ACTUAL TABLEWORK FOR THE RIGHT SIDE
########################################
$ajax_return .= <<<AJR
</tr>
<tr>
AJR;
	
	###############################################################################################
	#CREATE THE MONTH NUMBERS AND IF THERE ARE COMMITS FOR EACH DAY - MAKE A QUICK LINK TO THAT DAY
	###############################################################################################
	for($i=1; $i<13; $i++)
	{
		#####################
		#HANDLE LEADING ZEROS
		#####################
		if(strlen($i) == 1) $i = "0$i";
		
		switch($i)
		{
			case 1: $month = "Jan"; break;
			case 2: $month = "Feb"; break;
			case 3: $month = "Mar"; break;
			case 4: $month = "Apr"; break;
			case 5: $month = "May"; break;
			case 6: $month = "Jun"; break;
			case 7: $month = "Jul"; break;
			case 8: $month = "Aug"; break;
			case 9: $month = "Sep"; break;
			case 10: $month = "Oct"; break;
			case 11: $month = "Nov"; break;
			case 12: $month = "Dec"; break;
		}
		
		##############################
		#HANDLE THE TOOLTIP COMMIT MSG
		##############################
		$tooltip = "Retrieve&nbsp;Stats";
		$ajax_return .= "<td class=\"body_secondary\" style=\"text-align:center; padding:5px 5px 0 0;\">";

		###############################
		#CHECK COMMITS AND SET THE LINK
		###############################
		if(isset(${"commits_$i"}))
		{
			###########
			#LINK IT UP
			###########
			$ajax_return .= "<span onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'><a href=javascript:retrieve_monthly_snapshot('$i','$log_year')>$month</a></span>";
		}
		else
		{
			################
			#JUST SHOW MONTH
			################
			$ajax_return .= $month;
		}

		##################
		#COMPLETE THE CELL
		##################
		$ajax_return .= "</td>";
	}

	###############################################################################################
	#DETERMINE THE TOTAL STATS FOR THIS TYPE - THIS WILL BE DISPLAYED TO THE RIGHT OF THE BAR CHART
	###############################################################################################
	$recap_average_month = ${'total_' . $year_chartting} / $total_months_tracked;
	$recap_average_month = number_format($recap_average_month, 0, '', ',');
	$recap_peak = number_format(${'highest_' . $year_chartting}, 0, '', ',');
	$recap_total = number_format(${'total_' . $year_chartting});

	#################################################################
	#CAPS THE FIRST LETTER IN THE YEAR_CHARTTING VAR FOR A NICE TITLE
	#################################################################
	switch($year_chartting)
	{
		case "commits"; $stat_title = "Commits"; break;
		case "rollbacks"; $stat_title = "Roll&nbsp;Backs"; break;
		case "block_reads"; $stat_title = "Block&nbsp;Reads"; break;
		case "buffer_hits"; $stat_title = "Buffer&nbsp;Hits"; break;
		case "inserts"; $stat_title = "Inserts"; break;
		case "updates"; $stat_title = "Updates"; break;
		case "deletes"; $stat_title = "Deletes"; break;
		case "seq_scans"; $stat_title = "SEQ&nbsp;Scans"; break;
		case "idx_scans"; $stat_title = "IDX&nbsp;Scans";
	}

	################################################################
	#PARSE THE PEAK MONTH DATE SO WE CAN LINK IT TO THE MONTHLY VIEW
	################################################################
	$peak_month = substr(${'highest_' . $year_chartting . '_date'},0,2);
	$peak_year = $log_year;
	$tooltip = "Retrieve Stats";

	###########################################
	#CREATE A FRIENDLY TITLE FOR THE PEAK MONTH
	###########################################
	$recap_peak_month = "";
	switch($peak_month)
	{
		case "01": $recap_peak_month = "January"; break;
		case "02": $recap_peak_month = "February"; break;
		case "03": $recap_peak_month = "March"; break;
		case "04": $recap_peak_month = "April"; break;
		case "05": $recap_peak_month = "May"; break;
		case "06": $recap_peak_month = "June"; break;
		case "07": $recap_peak_month = "July"; break;
		case "08": $recap_peak_month = "August"; break;
		case "09": $recap_peak_month = "September"; break;
		case "10": $recap_peak_month = "October"; break;
		case "11": $recap_peak_month = "November"; break;
		case "12": $recap_peak_month = "December";
	}

	###################################
	#COMPLETE THE CHART TABLEWORK ABOVE
	###################################
	$ajax_return .= "</tr>";
	$ajax_return .= "</table>";

	#################
	#RECAP STATISTICS
	#################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_highlight\" style=\"float:left; margin:20px 15px 0 0; padding:10px; font-weight:bold;\">";
	$ajax_return .= "Total&nbsp;$stat_title:&nbsp;<span class=\"body_primary\">$recap_total</span></div>";

	####################
	#PEAK MONTH AND DATE
	####################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Peak&nbsp;Month:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">";
	if($recap_peak > 0) $ajax_return .= "$recap_peak_month&nbsp;/&nbsp;$recap_peak&nbsp;<span class=\"body_highlight\" style=\"font-weight:normal;\" onMouseOver=\"tooltip.show('$tooltip')\" onMouseOut='tooltip.hide()'>(<a href=javascript:retrieve_monthly_snapshot('$peak_month','$peak_year')>${'highest_' . $year_chartting . '_date'}</a>)</span>";
	else $ajax_return .= "N/A";
	$ajax_return .= "</span></div>";

	##################
	#AVERAGE PER MONTH
	##################
	$ajax_return .= "<div class=\"stat_sub_wrapper body_secondary\" style=\"float:left; margin:20px 15px 0 0; padding:10px;\">";
	$ajax_return .= "Average&nbsp;/&nbsp;Month:&nbsp;<span class=\"body_primary\" style=\"font-weight:bold;\">";
	if($recap_average_month > 0) $ajax_return .= "$recap_average_month";
	else $ajax_return .= "N/A";
	$ajax_return .= "</span></div>";

	########################################
	#CLEAR FLOATS AND COMPLETE THE UPPER DIV
	########################################
	$ajax_return .= "<div style=\"clear:both;\"></div>";
	$ajax_return .= "</div>";

###############################
#START UP IUD DIV AND TABLEWORK
###############################
$ajax_return .= <<<AJR
<div class="stat_wrapper" style="margin:0 0 25px 0;">
AJR;

###############################################################
#START UP THE TABLE WORK WHICH WILL HOUSE OUR IUD BAR AND STATS
###############################################################
$ajax_return .= <<<AJR
<div class="body_highlight" style="margin-right:10px; float:left; font-weight:bold;">Insert&nbsp;/&nbsp;Update&nbsp;/&nbsp;Delete&nbsp;Quick&nbsp;View:&nbsp;</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_insert_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Inserts</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_update_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Updates</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_delete_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">Deletes</div>
<div style="float:left; height:10px; width:10px; background-color:$iud_nodata_color; margin-right:3px; border:1px solid $bar_border_color;"></div><div style="float:left; margin-right:10px;">No&nbsp;Data</div>
<div style="clear:both;"></div>
AJR;

	##############################################################
	#PUT TOGETHER OUR IUD STATS - FIRST INITIATE SOME GENERAL VARS
	##############################################################
	$total_iud = $total_inserts + $total_updates + $total_deletes;
	
	##############################################
	#GET THE PERCENTAGES OF EACH AGAINST THE TOTAL
	##############################################
	if(!empty($total_iud)) { $iud_ins_percent = number_format($total_inserts / $total_iud, 2, '.', ','); } else { $iud_ins_percent = 0; }
	if(!empty($total_iud)) { $iud_upd_percent = number_format($total_updates / $total_iud, 2, '.', ','); } else { $iud_upd_percent = 0; }
	if(!empty($total_iud)) { $iud_del_percent = number_format($total_deletes / $total_iud, 2, '.', ','); } else { $iud_del_percent = 0; }

	#################################################
	#PREPARE THE BAR WIDTHS AND ROUND THE NUMBERS OUT
	#################################################
	$iud_ins_bar_w = number_format($iud_bar_width * $iud_ins_percent);
	$iud_upd_bar_w = number_format($iud_bar_width * $iud_upd_percent);
	$iud_del_bar_w = number_format($iud_bar_width * $iud_del_percent);
	
	#######################################################################################################
	#TALLY UP THE WIDTHS AND TAKE ANY NOMINAL REMAINDER +/- IUD_BAR_WIDTH AND +/- IT FROM THE LARGEST SCORE
	#######################################################################################################
	$iud_bar_total = $iud_ins_bar_w + $iud_upd_bar_w + $iud_del_bar_w;
	
	if($iud_bar_total < $iud_bar_width && $iud_bar_total > 0) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_width-$iud_bar_total;
		
		############################
		#ADD IT TO HIGHEST IUD SCORE
		############################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w += $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w += $iud_remain;
		else $iud_del_bar_w += $iud_remain;
	}
	elseif($iud_bar_total > $iud_bar_width) 
	{
		###################
		#GET THE DIFFERENCE
		###################
		$iud_remain = $iud_bar_total-$iud_bar_width;
		
		###################################
		#SUBTRACT IT FROM HIGHEST IUD SCORE
		###################################
		if($total_inserts > $total_updates && $total_inserts > $total_deletes) $iud_ins_bar_w -= $iud_remain;
		elseif($total_updates > $total_inserts && $total_updates > $total_deletes) $iud_upd_bar_w -= $iud_remain;
		else $iud_del_bar_w -= $iud_remain;
	}

	##################################
	#PUT TOGETHER OUR IUD TOOLTIP VARS
	##################################
	$iud_tt_ins = "<div style=line-height:12px;>";
	$iud_tt_ins .= "Total Inserts:&nbsp;<strong class=body_highlight>" . number_format($total_inserts) . "</strong>" . $iud_inserts_change;
	$iud_tt_ins .= "</div>";

	$iud_tt_upd = "<div style=line-height:12px;>";
	$iud_tt_upd .= "Total Updates:&nbsp;<strong class=body_highlight>" . number_format($total_updates) . "</strong>" . $iud_updates_change;
	$iud_tt_upd .= "</div>";

	$iud_tt_del = "<div style=line-height:12px;>";
	$iud_tt_del .= "Total Deletes:&nbsp;<strong class=body_highlight>" . number_format($total_deletes) . "</strong>" . $iud_deletes_change;
	$iud_tt_del .= "</div>";

	###########################################################################################
	#SHOW EITHER THE THREE IUD BARS TOGETHER OR THE DEFAULT NO-DATA BAR IF THERE IS NO IUD DATA
	###########################################################################################

	################################
	#PREP SOME FINAL CELL DIMENSIONS
	################################
	$iud_bar_width_px = $iud_bar_width . "px";
	$iud_bar_height_px = $iud_bar_height . "px";

	####################################
	#START WITH OUR BEGINNING TABLE WORK
	####################################
	$ajax_return .= "<div style=\"margin-top:10px;\">";
	$ajax_return .= "<table class=\"stat_chart_table\" style=\"width:$iud_bar_width_px; empty-cells:show;\">";
	$ajax_return .= "<tr>";

	if($total_iud > 0)
	{
		###########
		#INSERT BAR
		###########
		if($iud_ins_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			$ins_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_ins_bar_w_px = $iud_ins_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_ins_bar_w_px; height:$iud_bar_height_px; background-color:#7B7B7B; $ins_border\" onMouseOver=\"tooltip.show('$iud_tt_ins')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#UPDATE BAR
		###########
		if($iud_upd_bar_w > 0)
		{
			############
			#PREP BORDER
			############
			if($iud_ins_bar_w > 0) $upd_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $upd_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_upd_bar_w_px = $iud_upd_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_upd_bar_w_px; height:$iud_bar_height_px; background-color:#C9C9C9; $upd_border\" onMouseOver=\"tooltip.show('$iud_tt_upd')\" onMouseOut='tooltip.hide()'></td>";
		}

		###########
		#DELETE BAR
		###########
		if($iud_del_bar_w > 0) 
		{
			############
			#PREP BORDER
			############
			if($iud_upd_bar_w > 0 || $iud_ins_bar_w > 0) $del_border = "border-color: $bar_border_color; border-width: 1px 1px 1px 0px; border-style: solid;";
			else $del_border = "border:1px solid $bar_border_color;";

			###########################
			#PREP FINAL CELL DIMENSIONS
			###########################
			$iud_del_bar_w_px = $iud_del_bar_w . "px";

			#############
			#DISPLAY CELL
			#############
			$ajax_return .= "<td style=\"width:$iud_del_bar_w_px; height:$iud_bar_height_px; background-color:#F1F1F1; $del_border\" onMouseOver=\"tooltip.show('$iud_tt_del')\" onMouseOut='tooltip.hide()'></td>";
		}
	}
	else
	{
		##################################
		#DEFAULT THE NO-DATA BAR AND WIDTH
		##################################
		$ajax_return .= "<td style=\"width:$iud_bar_width_px; height:$iud_bar_height_px; background-color:#000000; border:1px solid $bar_border_color;\" onMouseOver=\"tooltip.show('No data to report.')\" onMouseOut='tooltip.hide()'></td>";
	}

##############################################
#COMPLETE OUR HTML TABLE WORK FOR THE IUD AREA
##############################################
$ajax_return .= <<<AJR
</tr>
</table>
</div></div>
AJR;

	###########################################
	#FORMAT CORE STATISTICS FOR THE BELOW TABLE
	###########################################
	for($i=1; $i<13; $i++)
	{
		if(strlen($i) == 1) $ix = "0$i";
		else $ix = "$i";
		
		${'commits_' . $ix} = number_abbr(${'commits_' . $ix},1);
		${'rollbacks_' . $ix} = number_abbr(${'rollbacks_' . $ix},1);
		${'block_reads_' . $ix} = number_abbr(${'block_reads_' . $ix},1);
		${'buffer_hits_' . $ix} = number_abbr(${'buffer_hits_' . $ix},1);
		${'inserts_' . $ix} = number_abbr(${'inserts_' . $ix},1);
		${'updates_' . $ix} = number_abbr(${'updates_' . $ix},1);
		${'deletes_' . $ix} = number_abbr(${'deletes_' . $ix},1);
		${'seq_scans_' . $ix} = number_abbr(${'seq_scans_' . $ix},1);
		${'idx_scans_' . $ix} = number_abbr(${'idx_scans_' . $ix},1);
		${'seq_rows_' . $ix} = number_abbr(${'seq_rows_' . $ix},1);
		${'idx_rows_' . $ix} = number_abbr(${'idx_rows_' . $ix},1);
	}

	###################################################
	#FORMAT THE REST OF OUR VARS FOR THE BOTTOM SECTION
	###################################################
	$total_commits = number_abbr($total_commits,3);
	$total_rollbacks = number_abbr($total_rollbacks,3);
	$total_block_reads = number_abbr($total_block_reads,3);
	$total_buffer_hits = number_abbr($total_buffer_hits,3);
	$total_inserts = number_abbr($total_inserts,3);
	$total_updates = number_abbr($total_updates,3);
	$total_deletes = number_abbr($total_deletes,3);
	$total_seq_scans = number_abbr($total_seq_scans,3);
	$total_idx_scans = number_abbr($total_idx_scans,3);
	$total_seq_rows = number_abbr($total_seq_rows,3);
	$total_idx_rows = number_abbr($total_idx_rows,3);

##############################
#CORE STATISTICS WILL GO BELOW
##############################
$ajax_return .= <<<AJR
<div class="stat_wrapper">
<table class="ps_core_table body_secondary">
<tr>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Statistic</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Jan</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Feb</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Mar</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Apr</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">May</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Jun</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Jul</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Aug</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Sep</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Oct</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Nov</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Dec</td>
<td class="body_primary" style="padding-bottom:10px; font-weight:bold;">Combined</td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Commits</td>
<td>$commits_01</td>
<td>$commits_02</td>
<td>$commits_03</td>
<td>$commits_04</td>
<td>$commits_05</td>
<td>$commits_06</td>
<td>$commits_07</td>
<td>$commits_08</td>
<td>$commits_09</td>
<td>$commits_10</td>
<td>$commits_11</td>
<td>$commits_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_commits</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Rollbacks</td>
<td>$rollbacks_01</td>
<td>$rollbacks_02</td>
<td>$rollbacks_03</td>
<td>$rollbacks_04</td>
<td>$rollbacks_05</td>
<td>$rollbacks_06</td>
<td>$rollbacks_07</td>
<td>$rollbacks_08</td>
<td>$rollbacks_09</td>
<td>$rollbacks_10</td>
<td>$rollbacks_11</td>
<td>$rollbacks_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_rollbacks</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Block&nbsp;Reads</td>
<td>$block_reads_01</td>
<td>$block_reads_02</td>
<td>$block_reads_03</td>
<td>$block_reads_04</td>
<td>$block_reads_05</td>
<td>$block_reads_06</td>
<td>$block_reads_07</td>
<td>$block_reads_08</td>
<td>$block_reads_09</td>
<td>$block_reads_10</td>
<td>$block_reads_11</td>
<td>$block_reads_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_block_reads</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">Buffer&nbsp;Hits</td>
<td>$buffer_hits_01</td>
<td>$buffer_hits_02</td>
<td>$buffer_hits_03</td>
<td>$buffer_hits_04</td>
<td>$buffer_hits_05</td>
<td>$buffer_hits_06</td>
<td>$buffer_hits_07</td>
<td>$buffer_hits_08</td>
<td>$buffer_hits_09</td>
<td>$buffer_hits_10</td>
<td>$buffer_hits_11</td>
<td>$buffer_hits_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_buffer_hits</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Inserts</td>
<td>$inserts_01</td>
<td>$inserts_02</td>
<td>$inserts_03</td>
<td>$inserts_04</td>
<td>$inserts_05</td>
<td>$inserts_06</td>
<td>$inserts_07</td>
<td>$inserts_08</td>
<td>$inserts_09</td>
<td>$inserts_10</td>
<td>$inserts_11</td>
<td>$inserts_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_inserts</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Updates</td>
<td>$updates_01</td>
<td>$updates_02</td>
<td>$updates_03</td>
<td>$updates_04</td>
<td>$updates_05</td>
<td>$updates_06</td>
<td>$updates_07</td>
<td>$updates_08</td>
<td>$updates_09</td>
<td>$updates_10</td>
<td>$updates_11</td>
<td>$updates_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_updates</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">User&nbsp;Deletes</td>
<td>$deletes_01</td>
<td>$deletes_02</td>
<td>$deletes_03</td>
<td>$deletes_04</td>
<td>$deletes_05</td>
<td>$deletes_06</td>
<td>$deletes_07</td>
<td>$deletes_08</td>
<td>$deletes_09</td>
<td>$deletes_10</td>
<td>$deletes_11</td>
<td>$deletes_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_deletes</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">SEQ&nbsp;Scans</td>
<td>$seq_scans_01</td>
<td>$seq_scans_02</td>
<td>$seq_scans_03</td>
<td>$seq_scans_04</td>
<td>$seq_scans_05</td>
<td>$seq_scans_06</td>
<td>$seq_scans_07</td>
<td>$seq_scans_08</td>
<td>$seq_scans_09</td>
<td>$seq_scans_10</td>
<td>$seq_scans_11</td>
<td>$seq_scans_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_seq_scans</td>
</tr>
<tr>
<td style="padding-right:10px;">Rows&nbsp;Used</td>
<td>$seq_rows_01</td>
<td>$seq_rows_02</td>
<td>$seq_rows_03</td>
<td>$seq_rows_04</td>
<td>$seq_rows_05</td>
<td>$seq_rows_06</td>
<td>$seq_rows_07</td>
<td>$seq_rows_08</td>
<td>$seq_rows_09</td>
<td>$seq_rows_10</td>
<td>$seq_rows_11</td>
<td>$seq_rows_12</td>
<td style="font-weight:bold;">$total_seq_rows</td>
</tr>
<tr>
<td colspan="14" style="height:30px;"><hr size="1"/></td>
</tr>
<tr>
<td class="body_highlight" style="padding-right:10px; font-weight:bold;">IDX&nbsp;Scans</td>
<td>$idx_scans_01</td>
<td>$idx_scans_02</td>
<td>$idx_scans_03</td>
<td>$idx_scans_04</td>
<td>$idx_scans_05</td>
<td>$idx_scans_06</td>
<td>$idx_scans_07</td>
<td>$idx_scans_08</td>
<td>$idx_scans_09</td>
<td>$idx_scans_10</td>
<td>$idx_scans_11</td>
<td>$idx_scans_12</td>
<td class="body_highlight" style="font-weight:bold;">$total_idx_scans</td>
</tr>
<tr>
<td style="padding-right:10px;">Rows&nbsp;Used</td>
<td>$idx_rows_01</td>
<td>$idx_rows_02</td>
<td>$idx_rows_03</td>
<td>$idx_rows_04</td>
<td>$idx_rows_05</td>
<td>$idx_rows_06</td>
<td>$idx_rows_07</td>
<td>$idx_rows_08</td>
<td>$idx_rows_09</td>
<td>$idx_rows_10</td>
<td>$idx_rows_11</td>
<td>$idx_rows_12</td>
<td style="font-weight:bold;">$total_idx_rows</td>
</tr>
</table>
</div>
AJR;

/*
#############################
#GET TOTAL SCRIPT PEAK MEMORY
#############################
$script_mem = number_format(((memory_get_peak_usage(true)/1024)/1024),3) . "M";
$ajax_return .= <<<AJR
<div style="text-align:left; padding-top:10px;">Script Peak Memory: $script_mem</div>
AJR;
*/
	
	#######################################
	#RETURN IT ALL TO THE AJAX RESPONSE VAR
	#######################################
	echo $ajax_return;
}
?>